@@ -9,7 +9,8 @@
 #include <QtCore>
 #include <stdlib.h>
 
-#include "mathstuff.h"
+#include "util/math.h"
+#include "util/assert.h"
 
 /*
  * Copyright 2006 dnk <dnk@bjum.net>
@@ -124,6 +125,10 @@ static MP4TrackId mp4_get_track(MP4FileHandle *handle)
     return MP4_INVALID_TRACK_ID;
 }
 
+static void mp4_init(struct input_plugin_data *ip_data) {
+    memset(ip_data, 0, sizeof(*ip_data));
+}
+
 static int mp4_open(struct input_plugin_data *ip_data)
 {
     struct mp4_private *priv;
@@ -163,8 +168,12 @@ static int mp4_open(struct input_plugin_data *ip_data)
     //         << neaac_cfg->dontUpSampleImplicitSBR;
     faacDecSetConfiguration(priv->decoder, neaac_cfg);
 
-    /* open mpeg-4 file */
-    priv->mp4.handle = MP4Read(ip_data->filename, 0);
+    /* open mpeg-4 file, check for >= ver 1.9.1 */
+#if MP4V2_PROJECT_version_hex <= 0x00010901
+    priv->mp4.handle = MP4Read(ip_data->filenameUtf8, 0);
+#else
+    priv->mp4.handle = MP4Read(ip_data->filenameUtf8);
+#endif
     if (!priv->mp4.handle) {
         qDebug() << "MP4Read failed";
         goto out;
@@ -198,7 +207,7 @@ static int mp4_open(struct input_plugin_data *ip_data)
 
     /* init decoder according to mpeg-4 audio config */
     if (faacDecInit2(priv->decoder, buf, buf_size,
-            (SAMPLERATE_TYPE*)&priv->sample_rate, &priv->channels) < 0) {
+            &priv->sample_rate, &priv->channels) < 0) {
         free(buf);
         goto out;
     }
@@ -217,9 +226,9 @@ out:
         MP4Close(priv->mp4.handle);
     if (priv->decoder)
         faacDecClose(priv->decoder);
-    delete priv;
     delete [] priv->sample_buf;
     delete [] priv->aac_data;
+    delete priv;
     return -IP_ERROR_FILE_FORMAT;
 }
 
@@ -238,7 +247,7 @@ static int mp4_close(struct input_plugin_data *ip_data)
     if (priv->sample_buf) {
         delete [] priv->sample_buf;
     }
-    
+
     if (priv->aac_data) {
         delete [] priv->aac_data;
     }
@@ -399,7 +408,10 @@ static int mp4_seek_sample(struct input_plugin_data *ip_data, int sample)
     struct mp4_private *priv;
     priv = (mp4_private*) ip_data->private_ipd;
 
-    Q_ASSERT(sample >= 0);
+    // Ignore the seek if the sample is invalid.
+    DEBUG_ASSERT_AND_HANDLE(sample >= 0) {
+        return mp4_current_sample(ip_data);
+    }
     // The first frame is samples 0 through 2047. The first sample of the second
     // frame is 2048. 2048 / 2048 = 1, so frame_for_sample will be 2 on the
     // 2048'th sample. The frame_offset_samples is how many samples into the frame
@@ -425,8 +437,8 @@ static int mp4_seek_sample(struct input_plugin_data *ip_data, int sample)
         // pops). This is akin to seeking in a video and seeing MPEG
         // artifacts. Figure out how many frames we need to go backward -- 1 seems
         // to work.
-        const int how_many_backwards = 1;
-        int start_frame = math_max(frame_for_sample - how_many_backwards, 1);
+        const unsigned int how_many_backwards = 1;
+        int start_frame = math_max(frame_for_sample - how_many_backwards, 1U);
         priv->mp4.sample = start_frame;
 
         // rryan 9/2009 -- the documentation is sketchy on this, but I think that
@@ -460,30 +472,6 @@ static int mp4_seek_sample(struct input_plugin_data *ip_data, int sample)
     return mp4_current_sample(ip_data);
 }
 
-static int mp4_seek(struct input_plugin_data *ip_data, double offset)
-{
-    struct mp4_private *priv;
-    MP4SampleId sample;
-    uint32_t scale;
-
-    priv = (mp4_private*) ip_data->private_ipd;
-
-    scale = MP4GetTrackTimeScale(priv->mp4.handle, priv->mp4.track);
-    if (scale == 0)
-        return -IP_ERROR_INTERNAL;
-
-    sample = MP4GetSampleIdFromTime(priv->mp4.handle, priv->mp4.track,
-        (MP4Timestamp)(offset * (double)scale), 0);
-    if (sample == MP4_INVALID_SAMPLE_ID)
-        return -IP_ERROR_INTERNAL;
-
-    qDebug() << "seeking from sample" << priv->mp4.sample << "to sample" << sample;
-    priv->mp4.sample = sample;
-    priv->overflow_buf_len = 0;
-
-    return priv->mp4.sample;
-}
-
 /* commented because we use TagLib now ??? -- bkgood
 static int mp4_read_comments(struct input_plugin_data *ip_data,
         struct keyval **comments)
@@ -552,22 +540,6 @@ static int mp4_read_comments(struct input_plugin_data *ip_data,
 }
 */
 
-static int mp4_duration(struct input_plugin_data *ip_data)
-{
-    struct mp4_private *priv;
-    uint32_t scale;
-    uint64_t duration;
-
-    priv = (mp4_private*) ip_data->private_ipd;
-
-    scale = MP4GetTrackTimeScale(priv->mp4.handle, priv->mp4.track);
-    if (scale == 0)
-        return 0;
-
-    duration = MP4GetTrackDuration(priv->mp4.handle, priv->mp4.track);
-
-    return duration / scale;
-}
 /*
 const struct input_plugin_ops ip_ops = {
     .open = mp4_open,
