@@ -1,7 +1,7 @@
 #ifndef ANALYSER_H
 #define ANALYSER_H
 
-#include "defs.h"
+#include "util/types.h"
 
 /*
  * An Analyser is an object which wants to process an entire song to
@@ -16,9 +16,11 @@
 class Analyser {
 
 public:
-    virtual void initialise(TrackPointer tio, int sampleRate, int totalSamples) { }
+    virtual bool initialise(TrackPointer tio, int sampleRate, int totalSamples) = 0;
+    virtual bool loadStored(TrackPointer tio) const = 0;
     virtual void process(const CSAMPLE* pIn, const int iLen) = 0;
-    virtual void finalise(TrackPointer tio) { }
+    virtual void cleanup(TrackPointer tio) = 0;
+    virtual void finalise(TrackPointer tio) = 0;
     virtual ~Analyser() {}
 };
 
