@@ -8,13 +8,15 @@
 #ifndef ANALYSERBEATS_H_
 #define ANALYSERBEATS_H_
 
+#include <QHash>
+
 #include "analyser.h"
 #include "configobject.h"
 #include "vamp/vampanalyser.h"
 
 class AnalyserBeats: public Analyser {
   public:
-    AnalyserBeats(ConfigObject<ConfigValue> *_config);
+    AnalyserBeats(ConfigObject<ConfigValue>* pConfig);
     virtual ~AnalyserBeats();
     bool initialise(TrackPointer tio, int sampleRate, int totalSamples);
     bool loadStored(TrackPointer tio) const;
@@ -23,12 +25,13 @@ class AnalyserBeats: public Analyser {
     void finalise(TrackPointer tio);
 
   private:
+    static QHash<QString, QString> getExtraVersionInfo(
+        QString pluginId, bool bPreferencesFastAnalysis);
     QVector<double> correctedBeats(QVector<double> rawbeats);
 
-    ConfigObject<ConfigValue> *m_pConfig;
+    ConfigObject<ConfigValue>* m_pConfig;
     VampAnalyser* m_pVamp;
     QString m_pluginId;
-    bool m_bShouldAnalyze;
     bool m_bPreferencesReanalyzeOldBpm;
     bool m_bPreferencesFixedTempo;
     bool m_bPreferencesOffsetCorrection;
