@@ -1,3 +1,5 @@
+#include <typeinfo>
+
 #include <QtDebug>
 #include <QMutexLocker>
 
@@ -6,41 +8,59 @@
 #include "analyserqueue.h"
 #include "soundsourceproxy.h"
 #include "playerinfo.h"
+#include "sampleutil.h"
 #include "util/timer.h"
 #include "library/trackcollection.h"
-
-#ifdef __TONAL__
-#include "tonal/tonalanalyser.h"
-#endif
-
 #include "analyserwaveform.h"
-#include "analyserbpm.h"
 #include "analyserrg.h"
-#ifdef __VAMP__
 #include "analyserbeats.h"
+#include "analyserkey.h"
 #include "vamp/vampanalyser.h"
-#endif
+#include "util/compatibility.h"
+#include "util/event.h"
+#include "util/trace.h"
 
-#include <typeinfo>
+// Measured in 0.1%,
+// 0 for no progress during finalize
+// 1 to display the text "finalizing"
+// 100 for 10% step after finalize
+#define FINALIZE_PERCENT 1
 
-#define FINALIZE_PERCENT 1 // in 0.1%,
-                           // 0 for no progress during finalize
-                           // 1 to display the text "finalizing"
-                           // 100 for 10% step after finalize
+// We need to use a smaller block size becuase on Linux, the AnalyserQueue
+// can starve the CPU of its resources, resulting in xruns.. A block size of
+// 8192 seems to do fine.
+const int kAnalysisBlockSize = 8192;
 
-
-AnalyserQueue::AnalyserQueue(TrackCollection* pTrackCollection) :
-        m_aq(),
-        m_exit(false),
-        m_aiCheckPriorities(false),
-        m_tioq(),
-        m_qm(),
-        m_qwait(),
-        m_queue_size(0) {
+AnalyserQueue::AnalyserQueue(TrackCollection* pTrackCollection)
+        : m_aq(),
+          m_exit(false),
+          m_aiCheckPriorities(false),
+          m_pSamplesPCM(new SAMPLE[kAnalysisBlockSize]),
+          m_pSamples(new CSAMPLE[kAnalysisBlockSize]),
+          m_tioq(),
+          m_qm(),
+          m_qwait(),
+          m_queue_size(0) {
+    Q_UNUSED(pTrackCollection);
     connect(this, SIGNAL(updateProgress()),
             this, SLOT(slotUpdateProgress()));
-    connect(this, SIGNAL(trackDone(TrackPointer)),
-            &pTrackCollection->getTrackDAO(), SLOT(saveTrack(TrackPointer)));
+}
+
+AnalyserQueue::~AnalyserQueue() {
+    stop();
+    m_progressInfo.sema.release();
+    wait(); //Wait until thread has actually stopped before proceeding.
+
+    QListIterator<Analyser*> it(m_aq);
+    while (it.hasNext()) {
+        Analyser* an = it.next();
+        //qDebug() << "AnalyserQueue: deleting " << typeid(an).name();
+        delete an;
+    }
+    //qDebug() << "AnalyserQueue::~AnalyserQueue()";
+
+    delete [] m_pSamplesPCM;
+    delete [] m_pSamples;
 }
 
 void AnalyserQueue::addAnalyser(Analyser* an) {
@@ -48,12 +68,14 @@ void AnalyserQueue::addAnalyser(Analyser* an) {
 }
 
 // This is called from the AnalyserQueue thread
-bool AnalyserQueue::isLoadedTrackWaiting(TrackPointer tio) {
-    QMutexLocker queueLocker(&m_qm);
-
-    const PlayerInfo& info = PlayerInfo::Instance();
+bool AnalyserQueue::isLoadedTrackWaiting(TrackPointer analysingTrack) {
+    const PlayerInfo& info = PlayerInfo::instance();
     TrackPointer pTrack;
     bool trackWaiting = false;
+    QList<TrackPointer> progress100List;
+    QList<TrackPointer> progress0List;
+
+    m_qm.lock();
     QMutableListIterator<TrackPointer> it(m_tioq);
     while (it.hasNext()) {
         TrackPointer& pTrack = it.next();
@@ -78,16 +100,27 @@ bool AnalyserQueue::isLoadedTrackWaiting(TrackPointer tio) {
                 }
             }
             if (!processTrack) {
-                emitUpdateProgress(pTrack, 1000);
-                it.remove();
+                progress100List.append(pTrack);
+                it.remove(); // since pTrack is a reference it is invalid now.
             } else {
-                emitUpdateProgress(pTrack, 0);
+                progress0List.append(pTrack);
             }
         } else if (progress == 1000) {
             it.remove();
         }
     }
-    if (info.isTrackLoaded(tio)) {
+
+    m_qm.unlock();
+
+    // update progress after unlock to avoid a deadlock
+    foreach (TrackPointer pTrack, progress100List) {
+        emitUpdateProgress(pTrack, 1000);
+    }
+    foreach (TrackPointer pTrack, progress0List) {
+        emitUpdateProgress(pTrack, 0);
+    }
+
+    if (info.isTrackLoaded(analysingTrack)) {
         return false;
     }
     return trackWaiting;
@@ -97,7 +130,9 @@ bool AnalyserQueue::isLoadedTrackWaiting(TrackPointer tio) {
 TrackPointer AnalyserQueue::dequeueNextBlocking() {
     m_qm.lock();
     if (m_tioq.isEmpty()) {
+        Event::end("AnalyserQueue process");
         m_qwait.wait(&m_qm);
+        Event::start("AnalyserQueue process");
 
         if (m_exit) {
             m_qm.unlock();
@@ -105,7 +140,7 @@ TrackPointer AnalyserQueue::dequeueNextBlocking() {
         }
     }
 
-    const PlayerInfo& info = PlayerInfo::Instance();
+    const PlayerInfo& info = PlayerInfo::instance();
     TrackPointer pLoadTrack;
     QMutableListIterator<TrackPointer> it(m_tioq);
     while (it.hasNext()) {
@@ -124,6 +159,7 @@ TrackPointer AnalyserQueue::dequeueNextBlocking() {
     }
 
     if (!pLoadTrack && !m_tioq.isEmpty()) {
+        // no prioritized track found, use head track
         pLoadTrack = m_tioq.dequeue();
     }
 
@@ -137,20 +173,11 @@ TrackPointer AnalyserQueue::dequeueNextBlocking() {
 }
 
 // This is called from the AnalyserQueue thread
-bool AnalyserQueue::doAnalysis(TrackPointer tio, SoundSourceProxy* pSoundSource) {
-    // TonalAnalyser requires a block size of 65536. Using a different value
-    // breaks the tonal analyser. We need to use a smaller block size becuase on
-    // Linux, the AnalyserQueue can starve the CPU of its resources, resulting
-    // in xruns.. A block size of 8192 seems to do fine.
-    const int ANALYSISBLOCKSIZE = 8192;
-
+bool AnalyserQueue::doAnalysis(TrackPointer tio, const Mixxx::SoundSourcePointer& pSoundSource) {
     int totalSamples = pSoundSource->length();
     //qDebug() << tio->getFilename() << " has " << totalSamples << " samples.";
     int processedSamples = 0;
 
-    SAMPLE* data16 = new SAMPLE[ANALYSISBLOCKSIZE];
-    CSAMPLE* samples = new CSAMPLE[ANALYSISBLOCKSIZE];
-
     QTime progressUpdateInhibitTimer;
     progressUpdateInhibitTimer.start(); // Inhibit Updates for 60 milliseconds
 
@@ -161,11 +188,11 @@ bool AnalyserQueue::doAnalysis(TrackPointer tio, SoundSourceProxy* pSoundSource)
 
     do {
         ScopedTimer t("AnalyserQueue::doAnalysis block");
-        read = pSoundSource->read(ANALYSISBLOCKSIZE, data16);
+        read = pSoundSource->read(kAnalysisBlockSize, m_pSamplesPCM);
 
         // To compare apples to apples, let's only look at blocks that are the
         // full block size.
-        if (read != ANALYSISBLOCKSIZE) {
+        if (read != kAnalysisBlockSize) {
             t.cancel();
         }
 
@@ -184,16 +211,14 @@ bool AnalyserQueue::doAnalysis(TrackPointer tio, SoundSourceProxy* pSoundSource)
             dieflag = true;
         }
 
-        for (int i = 0; i < read; i++) {
-            samples[i] = ((float)data16[i])/32767.0f;
-        }
+        SampleUtil::convertS16ToFloat32(m_pSamples, m_pSamplesPCM, read);
 
         QListIterator<Analyser*> it(m_aq);
 
         while (it.hasNext()) {
             Analyser* an =  it.next();
             //qDebug() << typeid(*an).name() << ".process()";
-            an->process(samples, read);
+            an->process(m_pSamples, read);
             //qDebug() << "Done " << typeid(*an).name() << ".process()";
         }
 
@@ -220,9 +245,8 @@ bool AnalyserQueue::doAnalysis(TrackPointer tio, SoundSourceProxy* pSoundSource)
         //QThread::yieldCurrentThread();
         //QThread::usleep(10);
 
-        //has something new entered the queue?
-        if (m_aiCheckPriorities) {
-            m_aiCheckPriorities = false;
+        // has something new entered the queue?
+        if (m_aiCheckPriorities.fetchAndStoreAcquire(false)) {
             if (isLoadedTrackWaiting(tio)) {
                 qDebug() << "Interrupting analysis to give preference to a loaded track.";
                 dieflag = true;
@@ -239,10 +263,7 @@ bool AnalyserQueue::doAnalysis(TrackPointer tio, SoundSourceProxy* pSoundSource)
         if (dieflag || cancelled) {
             t.cancel();
         }
-    } while(read == ANALYSISBLOCKSIZE && !dieflag);
-
-    delete[] data16;
-    delete[] samples;
+    } while(read == kAnalysisBlockSize && !dieflag);
 
     return !cancelled; //don't return !dieflag or we might reanalyze over and over
 }
@@ -255,41 +276,51 @@ void AnalyserQueue::stop() {
 }
 
 void AnalyserQueue::run() {
-    unsigned static id = 0; //the id of this thread, for debugging purposes
+    unsigned static id = 0; // the id of this thread, for debugging purposes
     QThread::currentThread()->setObjectName(QString("AnalyserQueue %1").arg(++id));
 
     // If there are no analyzers, don't waste time running.
     if (m_aq.size() == 0)
         return;
 
-    m_progressInfo.current_track = TrackPointer();
+    m_progressInfo.current_track.clear();
     m_progressInfo.track_progress = 0;
     m_progressInfo.queue_size = 0;
-    m_progressInfo.sema.release(); // Initalise with one
+    m_progressInfo.sema.release(); // Initialise with one
 
     while (!m_exit) {
         TrackPointer nextTrack = dequeueNextBlocking();
 
         // It's important to check for m_exit here in case we decided to exit
         // while blocking for a new track.
-        if (m_exit)
-            return;
+        if (m_exit) {
+            break;
+        }
 
         // If the track is NULL, try to get the next one.
         // Could happen if the track was queued but then deleted.
         // Or if dequeueNextBlocking is unblocked by exit == true
         if (!nextTrack) {
+            emptyCheck();
             continue;
         }
 
+        Trace trace("AnalyserQueue analyzing track");
+
         // Get the audio
-        SoundSourceProxy* pSoundSource = new SoundSourceProxy(nextTrack);
-        pSoundSource->open(); //Open the file for reading
+        SoundSourceProxy soundSourceProxy(nextTrack);
+        Mixxx::SoundSourcePointer pSoundSource(soundSourceProxy.open());
+        if (pSoundSource.isNull()) {
+            qWarning() << "Failed to open file for analyzing:" << nextTrack->getLocation();
+            emptyCheck();
+            continue;
+        }
+
         int iNumSamples = pSoundSource->length();
         int iSampleRate = pSoundSource->getSampleRate();
 
         if (iNumSamples == 0 || iSampleRate == 0) {
-            qDebug() << "Skipping invalid file:" << nextTrack->getLocation();
+            qWarning() << "Skipping invalid file:" << nextTrack->getLocation();
             continue;
         }
 
@@ -310,7 +341,7 @@ void AnalyserQueue::run() {
             emitUpdateProgress(nextTrack, 0);
             bool completed = doAnalysis(nextTrack, pSoundSource);
             if (!completed) {
-                //This track was cancelled
+                // This track was cancelled
                 QListIterator<Analyser*> itf(m_aq);
                 while (itf.hasNext()) {
                     itf.next()->cleanup(nextTrack);
@@ -332,27 +363,28 @@ void AnalyserQueue::run() {
             emitUpdateProgress(nextTrack, 1000); // 100%
             qDebug() << "Skipping track analysis because no analyzer initialized.";
         }
-
-        delete pSoundSource;
-
-        m_qm.lock();
-        m_queue_size = m_tioq.size();
-        m_qm.unlock();
-        if (m_queue_size == 0) {
-            emit(queueEmpty()); // emit asynchrony for no deadlock
-        }
+        emptyCheck();
     }
     emit(queueEmpty()); // emit in case of exit;
 }
 
+void AnalyserQueue::emptyCheck() {
+    m_qm.lock();
+    m_queue_size = m_tioq.size();
+    m_qm.unlock();
+    if (m_queue_size == 0) {
+        emit(queueEmpty()); // emit asynchrony for no deadlock
+    }
+}
+
 // This is called from the AnalyserQueue thread
-void AnalyserQueue::emitUpdateProgress(TrackPointer tio, int progress) {
-    if (!m_exit) {    
+void AnalyserQueue::emitUpdateProgress(TrackPointer track, int progress) {
+    if (!m_exit) {
         // First tryAcqire will have always success because sema is initialized with on
         // The following tries will success if the previous signal was processed in the GUI Thread
         // This prevent the AnalysisQueue from filling up the GUI Thread event Queue
         // 100 % is emitted in any case
-        if (progress < 1000 - FINALIZE_PERCENT && progress > 0 ) {
+        if (progress < 1000 - FINALIZE_PERCENT && progress > 0) {
             // Signals during processing are not required in any case
             if (!m_progressInfo.sema.tryAcquire()) {
                return;
@@ -360,7 +392,7 @@ void AnalyserQueue::emitUpdateProgress(TrackPointer tio, int progress) {
         } else {
             m_progressInfo.sema.acquire();
         }
-        m_progressInfo.current_track = tio;
+        m_progressInfo.current_track = track;
         m_progressInfo.track_progress = progress;
         m_progressInfo.queue_size = m_queue_size;
         emit(updateProgress());
@@ -370,20 +402,21 @@ void AnalyserQueue::emitUpdateProgress(TrackPointer tio, int progress) {
 //slot
 void AnalyserQueue::slotUpdateProgress() {
     if (m_progressInfo.current_track) {
-        m_progressInfo.current_track->setAnalyserProgress(m_progressInfo.track_progress);
+        m_progressInfo.current_track->setAnalyserProgress(
+        		m_progressInfo.track_progress);
+        m_progressInfo.current_track.clear();
     }
-    emit(trackProgress(m_progressInfo.track_progress/10));
+    emit(trackProgress(m_progressInfo.track_progress / 10));
     if (m_progressInfo.track_progress == 1000) {
         emit(trackFinished(m_progressInfo.queue_size));
     }
     m_progressInfo.sema.release();
 }
 
-//slot
 void AnalyserQueue::slotAnalyseTrack(TrackPointer tio) {
     // This slot is called from the decks and and samplers when the track was loaded.
-    m_aiCheckPriorities = true;
     queueAnalyseTrack(tio);
+    m_aiCheckPriorities = true;
 }
 
 // This is called from the GUI and from the AnalyserQueue thread
@@ -398,56 +431,29 @@ void AnalyserQueue::queueAnalyseTrack(TrackPointer tio) {
 
 // static
 AnalyserQueue* AnalyserQueue::createDefaultAnalyserQueue(
-        ConfigObject<ConfigValue>* _config, TrackCollection* pTrackCollection) {
+        ConfigObject<ConfigValue>* pConfig, TrackCollection* pTrackCollection) {
     AnalyserQueue* ret = new AnalyserQueue(pTrackCollection);
 
-#ifdef __TONAL__
-    ret->addAnalyser(new TonalAnalyser());
-#endif
-
-    ret->addAnalyser(new AnalyserWaveform(_config));
-    ret->addAnalyser(new AnalyserGain(_config));
-#ifdef __VAMP__
+    ret->addAnalyser(new AnalyserWaveform(pConfig));
+    ret->addAnalyser(new AnalyserGain(pConfig));
     VampAnalyser::initializePluginPaths();
-    ret->addAnalyser(new AnalyserBeats(_config));
-    //ret->addAnalyser(new AnalyserVampKeyTest(_config));
-#else
-    ret->addAnalyser(new AnalyserBPM(_config));
-#endif
+    ret->addAnalyser(new AnalyserBeats(pConfig));
+    ret->addAnalyser(new AnalyserKey(pConfig));
 
-    ret->start(QThread::IdlePriority);
+    ret->start(QThread::LowPriority);
     return ret;
 }
 
 // static
-AnalyserQueue* AnalyserQueue::createPrepareViewAnalyserQueue(
-        ConfigObject<ConfigValue>* _config, TrackCollection* pTrackCollection) {
+AnalyserQueue* AnalyserQueue::createAnalysisFeatureAnalyserQueue(
+        ConfigObject<ConfigValue>* pConfig, TrackCollection* pTrackCollection) {
     AnalyserQueue* ret = new AnalyserQueue(pTrackCollection);
 
-    ret->addAnalyser(new AnalyserWaveform(_config));
-    ret->addAnalyser(new AnalyserGain(_config));
-#ifdef __VAMP__
+    ret->addAnalyser(new AnalyserGain(pConfig));
     VampAnalyser::initializePluginPaths();
-    ret->addAnalyser(new AnalyserBeats(_config));
-    //ret->addAnalyser(new AnalyserVampKeyTest(_config));
-#else
-    ret->addAnalyser(new AnalyserBPM(_config));
-#endif
+    ret->addAnalyser(new AnalyserBeats(pConfig));
+    ret->addAnalyser(new AnalyserKey(pConfig));
 
-    ret->start(QThread::IdlePriority);
+    ret->start(QThread::LowPriority);
     return ret;
 }
-
-AnalyserQueue::~AnalyserQueue() {
-    stop();
-    m_progressInfo.sema.release();
-    wait(); //Wait until thread has actually stopped before proceeding.
-
-    QListIterator<Analyser*> it(m_aq);
-    while (it.hasNext()) {
-        Analyser* an = it.next();
-        //qDebug() << "AnalyserQueue: deleting " << typeid(an).name();
-        delete an;
-    }
-    //qDebug() << "AnalyserQueue::~AnalyserQueue()";
-}
