@@ -4,26 +4,19 @@
 #ifndef CACHINGREADER_H
 #define CACHINGREADER_H
 
-#include <QDebug>
-#include <QMutex>
-#include <QQueue>
-#include <QWaitCondition>
+#include <QtDebug>
 #include <QList>
 #include <QVector>
 #include <QLinkedList>
 #include <QHash>
-#include <QThread>
+#include <QVarLengthArray>
 
-#include "defs.h"
+#include "util/types.h"
 #include "configobject.h"
 #include "trackinfoobject.h"
 #include "engine/engineworker.h"
 #include "util/fifo.h"
-
-class ControlObjectThread;
-namespace Mixxx {
-    class SoundSource;
-}
+#include "cachingreaderworker.h"
 
 // A Hint is an indication to the CachingReader that a certain section of a
 // SoundSource will be used 'soon' and so it should be brought into memory by
@@ -42,55 +35,43 @@ typedef struct Hint {
     int priority;
 } Hint;
 
-// A Chunk is a section of audio that is being cached. The chunk_number can be
-// used to figure out the sample number of the first sample in data by using
-// sampleForChunk()
-typedef struct Chunk {
-    int chunk_number;
-    int length;
-    CSAMPLE* data;
-    Chunk* prev_lru;
-    Chunk* next_lru;
-} Chunk;
-
-typedef struct ChunkReadRequest {
-    Chunk* chunk;
-
-    ChunkReadRequest() { chunk = NULL; }
-} ChunkReadRequest;
-
-enum ReaderStatus {
-    INVALID,
-    TRACK_NOT_LOADED,
-    TRACK_LOADED,
-    CHUNK_READ_SUCCESS,
-    CHUNK_READ_EOF,
-    CHUNK_READ_INVALID
-};
-
-typedef struct ReaderStatusUpdate {
-    ReaderStatus status;
-    Chunk* chunk;
-    int trackNumSamples;
-    ReaderStatusUpdate() {
-        status = INVALID;
-        chunk = NULL;
-    }
-} ReaderStatusUpdate;
+// Note that we use a QVarLengthArray here instead of a QVector. Since this list
+// is cleared on every callback and potentially referenced multiples times it's
+// nicer to use a QVarLengthArray over a QVector because of two things:
+//
+// 1) No copy-on-write / implicit sharing behavior. If the reference count rises
+//    above 1 then every non-const operation on a QVector clones it. We'd like
+//    to avoid unnecessary memory allocation in the callback thread so this is
+//    undesirable.
+// 2) QVector::clear deletes the backing store (even if you call reserve) so we
+//    reallocate on every callback. resize(0) should work but a future developer
+//    may see a resize(0) and say "that's a silly way of writing clear()!" and
+//    replace it without realizing.
+typedef QVarLengthArray<Hint, 512> HintVector;
 
 // CachingReader provides a layer on top of a SoundSource for reading samples
-// from a file. A cache is provided so that repeated reads to a certain section
-// of a song do not cause disk seeks or unnecessary SoundSource
-// calls. CachingReader provides a worker thread that can be used to prepare the
-// cache so that areas of a file that will soon be read are present in memory
-// once they are needed. This can be accomplished by issueing 'hints' to the
-// reader of areas of a SoundSource that will be read soon.
-class CachingReader : public EngineWorker {
+// from a file. Since we cannot do file I/O in the audio callback thread
+// CachingReader and CachingReaderWorker (a worker thread) work in concert to
+// read and decode relevant sections of a track in a background thread. The
+// decoded chunks are kept in a cache by CachingReader with a
+// least-recently-used (LRU) eviction policy. CachingReader exposes a method for
+// indicating which chunks should be kept fresh in the cache (see
+// hintAndMaybeWake). For example, the chunks around the playhead, the hotcue
+// positions, and loop points are all portions of the track that the user is
+// likely to dynamically jump to so we should keep them ready.
+//
+// The least recently used policy is implemented by keeping a linked list of the
+// least recently used chunks. When a chunk is "freshened" (i.e. accessed via
+// read or hinted via hintAndMaybeWake) then it is moved to the back of the
+// least-recently-used list. When a chunk needs to be allocated and there are no
+// free chunks then the least recently used chunk is free'd (see
+// allocateChunkExpireLRU).
+class CachingReader : public QObject {
     Q_OBJECT
 
   public:
     // Construct a CachingReader with the given group.
-    CachingReader(const char* _group,
+    CachingReader(QString group,
                   ConfigObject<ConfigValue>* _config);
     virtual ~CachingReader();
 
@@ -104,23 +85,18 @@ class CachingReader : public EngineWorker {
     // that is not in the cache. If any hints do request a chunk not in cache,
     // then wake the reader so that it can process them. Must only be called
     // from the engine callback.
-    virtual void hintAndMaybeWake(QList<Hint>& hintList);
+    virtual void hintAndMaybeWake(const HintVector& hintList);
 
     // Request that the CachingReader load a new track. These requests are
     // processed in the work thread, so the reader must be woken up via wake()
     // for this to take effect.
     virtual void newTrack(TrackPointer pTrack);
 
-    // Wake the reader up so that it will process newTrack requests and hints.
-    virtual void wake();
+    void setScheduler(EngineWorkerScheduler* pScheduler) {
+        m_pWorker->setScheduler(pScheduler);
+    }
 
-    // Run upkeep operations like loading tracks and reading from file. Run by a
-    // thread pool via the EngineWorkerScheduler.
-    virtual void run();
-
-    // A Chunk is a memory-resident section of audio that has been cached. Each
-    // chunk holds a fixed number of samples given by kSamplesPerChunk.
-    const static int kChunkLength, kSamplesPerChunk;
+    const static int maximumChunksInMemory;
 
   signals:
     // Emitted once a new track is loaded and ready to be read from.
@@ -136,19 +112,9 @@ class CachingReader : public EngineWorker {
 
     // Given a sample number, return the chunk number corresponding to it.
     inline static int chunkForSample(int sample_number) {
-        return sample_number / kSamplesPerChunk;
+        return sample_number / CachingReaderWorker::kSamplesPerChunk;
     }
 
-    // Given a chunk number, return the start sample number for the chunk.
-    inline static int sampleForChunk(int chunk_number) {
-        return chunk_number * kSamplesPerChunk;
-    }
-
-    // Initialize the reader by creating all the chunks from the RAM provided to
-    // the CachingReader.
-    void initialize();
-
-    const char* m_pGroup;
     const ConfigObject<ConfigValue>* m_pConfig;
 
     // Thread-safe FIFOs for communication between the engine callback and
@@ -156,19 +122,18 @@ class CachingReader : public EngineWorker {
     FIFO<ChunkReadRequest> m_chunkReadRequestFIFO;
     FIFO<ReaderStatusUpdate> m_readerStatusFIFO;
 
-    // Queue of Tracks to load, and the corresponding lock. Must acquire the
-    // lock to touch.
-    QMutex m_newTrackMutex;
-    TrackPointer m_newTrack;
-
-    ////////////////////////////////////////////////////////////////////////////
-    // The following may /only/ be called within the engine callback
-    ////////////////////////////////////////////////////////////////////////////
+    // Looks for the provided chunk number in the index of in-memory chunks and
+    // returns it if it is present. If not, returns NULL. If it is present then
+    // freshenChunk is called on the chunk to make it the MRU chunk.
+    Chunk* lookupChunkAndFreshen(int chunk_number);
 
     // Looks for the provided chunk number in the index of in-memory chunks and
     // returns it if it is present. If not, returns NULL.
     Chunk* lookupChunk(int chunk_number);
 
+    // Moves the provided chunk to the MRU position.
+    void freshenChunk(Chunk* pChunk);
+
     // Returns a Chunk to the free list
     void freeChunk(Chunk* pChunk);
 
@@ -176,19 +141,19 @@ class CachingReader : public EngineWorker {
     void freeAllChunks();
 
     // Gets a chunk from the free list. Returns NULL if none available.
-    Chunk* allocateChunk();
+    Chunk* allocateChunk(int chunk);
 
     // Gets a chunk from the free list, frees the LRU Chunk if none available.
-    Chunk* allocateChunkExpireLRU();
+    Chunk* allocateChunkExpireLRU(int chunk);
 
     ReaderStatus m_readerStatus;
 
-    // Keeps track of free Chunks we've allocated
+    // Keeps track of all Chunks we've allocated.
     QVector<Chunk*> m_chunks;
-    // List of free chunks available for use.
-    QList<Chunk*> m_freeChunks;
-    // List of reserved chunks with reads in progress
-    QHash<int, Chunk*> m_chunksBeingRead;
+
+    // List of free chunks. Linked list so that we have constant time insertions
+    // and deletions. Iteration is not necessary.
+    QLinkedList<Chunk*> m_freeChunks;
 
     // Keeps track of what Chunks we've allocated and indexes them based on what
     // chunk number they are allocated to.
@@ -201,26 +166,9 @@ class CachingReader : public EngineWorker {
     // The raw memory buffer which is divided up into chunks.
     CSAMPLE* m_pRawMemoryBuffer;
 
-    ////////////////////////////////////////////////////////////////////////////
-    // The following may /only/ be called within the reader thread
-    ////////////////////////////////////////////////////////////////////////////
-
-    // Internal method to load a track. Emits trackLoaded when finished.
-    void loadTrack(TrackPointer pTrack);
-
-    // Read the given chunk_number from the file into pChunk's data
-    // buffer. Fills length/sample information about Chunk* as well.
-    void processChunkReadRequest(ChunkReadRequest* request,
-                                 ReaderStatusUpdate* update);
-
-    // The current sound source of the track loaded
-    Mixxx::SoundSource* m_pCurrentSoundSource;
-    int m_iTrackSampleRate;
-    int m_iTrackNumSamples;
     int m_iTrackNumSamplesCallbackSafe;
 
-    // Temporary buffer for reading from SoundSources
-    SAMPLE* m_pSample;
+    CachingReaderWorker* m_pWorker;
 };
 
 
