@@ -14,36 +14,29 @@
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
-#include <qapplication.h>
 #include "configobject.h"
-#include <qdir.h>
+
+#include <QIODevice>
+#include <QTextStream>
+#include <QApplication>
+#include <QDir>
 #include <QtDebug>
+
 #include "widget/wwidget.h"
+#include "util/cmdlineargs.h"
+#include "util/xml.h"
 
-#ifdef __C_METRICS__
-#include "cmetrics.h"
-#endif
-
-#ifdef __WINDOWS__
-#include <windows.h>
-#endif
-
-#ifdef __APPLE__
-#include <CoreFoundation/CoreFoundation.h>
-#endif
-
-#include <qiodevice.h>
-#include <QTextIStream>
-#include <math.h>
-
-ConfigKey::ConfigKey()
-{
+ConfigKey::ConfigKey() {
 }
 
-ConfigKey::ConfigKey(QString g, QString i)
-{
-    group = g;
-    item = i;
+ConfigKey::ConfigKey(const QString& g, const QString& i)
+    : group(g),
+      item(i) {
+}
+
+ConfigKey::ConfigKey(const char* g, const char* i)
+    : group(g),
+      item(i) {
 }
 
 // static
@@ -69,7 +62,7 @@ ConfigValue::ConfigValue(int _value)
     value = QString::number(_value);
 }
 
-void ConfigValue::valCopy(const ConfigValue _value)
+void ConfigValue::valCopy(const ConfigValue& _value)
 {
     value = _value.value;
 }
@@ -83,29 +76,29 @@ ConfigValueKbd::ConfigValueKbd(QString _value) : ConfigValue(_value)
 {
     QString key;
 
-    QTextIStream(&_value) >> key;
+    QTextStream(&_value) >> key;
     m_qKey = QKeySequence(key);
 }
 
 ConfigValueKbd::ConfigValueKbd(QKeySequence key)
 {
     m_qKey = key;
-    QTextOStream(&value) << m_qKey.toString();
+    QTextStream(&value) << m_qKey.toString();
 //          qDebug() << "value" << value;
 }
 
-void ConfigValueKbd::valCopy(const ConfigValueKbd v)
+void ConfigValueKbd::valCopy(const ConfigValueKbd& v)
 {
     m_qKey = v.m_qKey;
-    QTextOStream(&value) << m_qKey.toString();
+    QTextStream(&value) << m_qKey.toString();
 }
 
-bool operator==(const ConfigValue & s1, const ConfigValue & s2)
+bool operator==(const ConfigValue& s1, const ConfigValue& s2)
 {
     return (s1.value.toUpper() == s2.value.toUpper());
 }
 
-bool operator==(const ConfigValueKbd & s1, const ConfigValueKbd & s2)
+bool operator==(const ConfigValueKbd& s1, const ConfigValueKbd& s2)
 {
     return (s1.value.toUpper() == s2.value.toUpper());
 }
@@ -117,14 +110,21 @@ template <class ValueType> ConfigObject<ValueType>::ConfigObject(QString file)
 
 template <class ValueType> ConfigObject<ValueType>::~ConfigObject()
 {
+    while (m_list.size() > 0) {
+        ConfigOption<ValueType>* pConfigOption = m_list.takeLast();
+        delete pConfigOption;
+    }
 }
 
 template <class ValueType>
 ConfigOption<ValueType> *ConfigObject<ValueType>::set(ConfigKey k, ValueType v)
 {
     // Search for key in list, and set value if found
-    ConfigOption<ValueType> *it;
-    for (it = list.first(); it; it = list.next())
+    QListIterator<ConfigOption<ValueType>* > iterator(m_list);
+    ConfigOption<ValueType>* it;
+    while (iterator.hasNext())
+    {
+        it = iterator.next();
 //         if (QString::compare(it->val->value, v.value, Qt::CaseInsensitive) == 0)
         if (it->key->group == k.group && it->key->item == k.item)
         {
@@ -136,21 +136,24 @@ ConfigOption<ValueType> *ConfigObject<ValueType>::set(ConfigKey k, ValueType v)
             //cout << "2: " << it->val->value << "\n";
             return it;
         }
+    }
 
     // If key is not found, insert it into the list of config objects
     ConfigKey * key = new ConfigKey(k.group, k.item);
     it = new ConfigOption<ValueType>(key, new ValueType(v));
     //qDebug() << "new configobject " << it->val;
-    list.append(it);
+    m_list.append(it);
     return it;
 }
 
 template <class ValueType>
 ConfigOption<ValueType> *ConfigObject<ValueType>::get(ConfigKey k)
 {
-    ConfigOption<ValueType> *it;
-    for (it = list.first(); it; it = list.next())
+    QListIterator<ConfigOption<ValueType>* > iterator(m_list);
+    ConfigOption<ValueType>* it;
+    while (iterator.hasNext())
     {
+        it = iterator.next();
         //qDebug() << it->key->group << k->group << it->key->item << k->item;
         if (it->key->group == k.group && it->key->item == k.item)
         {
@@ -161,17 +164,35 @@ ConfigOption<ValueType> *ConfigObject<ValueType>::get(ConfigKey k)
     // If key is not found, insert into list with null values
     ConfigKey * key = new ConfigKey(k.group, k.item);
     it = new ConfigOption<ValueType>(key, new ValueType(""));
-    list.append(it);
+    m_list.append(it);
     return it;
 }
 
+template <class ValueType>
+bool ConfigObject<ValueType>::exists(ConfigKey k)
+{
+    QListIterator<ConfigOption<ValueType>* > iterator(m_list);
+    ConfigOption<ValueType>* it;
+    while (iterator.hasNext())
+    {
+        it = iterator.next();
+        if (it->key->group == k.group && it->key->item == k.item)
+        {
+            return true;
+        }
+    }
+    return false;
+}
+
 template <class ValueType>
 ConfigKey *ConfigObject<ValueType>::get(ValueType v)
 {
-    ConfigOption<ValueType> *it;
-    for (it = list.first(); it; it = list.next())
+    QListIterator<ConfigOption<ValueType>* > iterator(m_list);
+    ConfigOption<ValueType>* it;
+    while (iterator.hasNext())
     {
-        if (QString::compare(it->val->value, v.value, Qt::CaseInsensitive) == 0){
+        it = iterator.next();
+        if (QString::compare(it->val->value, v.value, Qt::CaseInsensitive) == 0) {
             //qDebug() << "ConfigObject #534: QString::compare match for " << it->key->group << it->key->item;
             return it->key;
         }
@@ -181,7 +202,7 @@ ConfigKey *ConfigObject<ValueType>::get(ValueType v)
             return it->key;
         }
 
-        if (it == list.getLast()) {
+        if (it == m_list.last()) {
             //qDebug() << "ConfigObject: last match attempted" << it->val->value.toUpper() << "with" << v.value.toUpper();
         }
     }
@@ -195,28 +216,40 @@ QString ConfigObject<ValueType>::getValueString(ConfigKey k)
     return get(k)->val->value;
 }
 
+template <class ValueType>
+QString ConfigObject<ValueType>::getValueString(ConfigKey k, const QString& default_string)
+{
+    QString ret = get(k)->val->value;
+    if (ret.isEmpty()) {
+        return default_string;
+    }
+    return ret;
+}
+
 template <class ValueType> bool ConfigObject<ValueType>::Parse()
 {
     // Open file for reading
-    QFile configfile(filename);
-    if (filename.length()<1 || !configfile.open(QIODevice::ReadOnly))
+    QFile configfile(m_filename);
+    if (m_filename.length()<1 || !configfile.open(QIODevice::ReadOnly))
     {
-        qDebug() << "Could not read" << filename;
+        qDebug() << "ConfigObject: Could not read" << m_filename;
         return false;
     }
     else
     {
+        //qDebug() << "ConfigObject: Parse" << m_filename;
         // Parse the file
         int group = 0;
         QString groupStr, line;
         QTextStream text(&configfile);
+        text.setCodec("UTF-8");
 
         while (!text.atEnd())
         {
             line = text.readLine().trimmed();
 
             if (line.length() != 0)
-			{
+            {
                 if (line.startsWith("[") && line.endsWith("]"))
                 {
                     group++;
@@ -226,7 +259,7 @@ template <class ValueType> bool ConfigObject<ValueType>::Parse()
                 else if (group>0)
                 {
                     QString key;
-                    QTextIStream(&line) >> key;
+                    QTextStream(&line) >> key;
                     QString val = line.right(line.length() - key.length()); // finds the value string
                     val = val.trimmed();
                     //qDebug() << "control:" << key << "value:" << val;
@@ -234,52 +267,56 @@ template <class ValueType> bool ConfigObject<ValueType>::Parse()
                     ValueType m(val);
                     set(k, m);
                 }
-			}
+            }
         }
         configfile.close();
     }
     return true;
 }
 
-
 template <class ValueType> void ConfigObject<ValueType>::clear()
 {
+    //Delete the pointers, because that's what we did before we
+    //purged Mixxx of Qt3 code. -- Albert, June 18th 2010 (at 30,000 ft)
+    for (int i = 0; i < m_list.count(); i++)
+        delete m_list[i];
+
     // This shouldn't be done, since objects might have references to
     // members of list. Instead all member values should be set to some
     // null value.
-    list.clear();
+    m_list.clear();
+
 }
 
 template <class ValueType> void ConfigObject<ValueType>::reopen(QString file)
 {
-    list.setAutoDelete(TRUE);
-
-    // First try to open the config file placed in the users .mixxx directory.
-    // If that fails, try the system wide CONFIG_PATH.
-
-    filename = file;
+    m_filename = file;
     Parse();
 }
 
 template <class ValueType> void ConfigObject<ValueType>::Save()
 {
-    QFile file(filename);
+    QFile file(m_filename);
     if (!QDir(QFileInfo(file).absolutePath()).exists()) {
         QDir().mkpath(QFileInfo(file).absolutePath());
     }
     if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
     {
-        qDebug() << "Could not write file" << filename << ", don't worry.";
+        qDebug() << "Could not write file" << m_filename << ", don't worry.";
         return;
     }
     else
     {
-        Q3TextStream stream(&file);
+        QTextStream stream(&file);
+        stream.setCodec("UTF-8");
 
-        ConfigOption<ValueType> *it;
         QString grp = "";
-        for (it = list.first(); it; it = list.next())
+
+        QListIterator<ConfigOption<ValueType>* > iterator(m_list);
+        ConfigOption<ValueType>* it;
+        while (iterator.hasNext())
         {
+            it = iterator.next();
 //            qDebug() << "group:" << it->key->group << "item" << it->key->item << "val" << it->val->value;
             if (it->key->group != grp)
             {
@@ -290,86 +327,79 @@ template <class ValueType> void ConfigObject<ValueType>::Save()
         }
         file.close();
         if (file.error()!=QFile::NoError) //could be better... should actually say what the error was..
-	  qDebug() << "Error while writing configuration file:" << file.errorString();
+      qDebug() << "Error while writing configuration file:" << file.errorString();
     }
 }
 
 template <class ValueType>
-QString ConfigObject<ValueType>::getConfigPath()
-{
-    //
-    // Find the config path, path where midi configuration files, skins etc. are stored.
-    // On Unix the search order is whats listed in mixxx.cfg, then UNIX_SHARE_PATH
-    // On Windows it is always (and only) app dir.
-    // On OS X it is the current directory and then the Resources/ dir in the app bundle
-    //
-    QString qConfigPath; // TODO: this only changes once (on first load) during a run should make this a singleton.
-
+QString ConfigObject<ValueType>::getResourcePath() const {
     // Try to read in the resource directory from the command line
-    QStringList commandLineArgs = QApplication::arguments();
-    int resourcePath = commandLineArgs.indexOf("--resourcePath");
-    if (resourcePath!=-1 && resourcePath + 1 < commandLineArgs.size()) {
-        qDebug() << "Setting qConfigPath from location in resourcePath commandline arg:" << commandLineArgs.at(resourcePath+1);
-        qConfigPath = commandLineArgs.at(resourcePath+1);
-    }
-    if (qConfigPath.isNull() || qConfigPath.isEmpty()) {
+    QString qResourcePath = CmdlineArgs::Instance().getResourcePath();
+
+    if (qResourcePath.isEmpty()) {
+        QDir mixxxDir(QCoreApplication::applicationDirPath());
+        // We used to support using the mixxx.cfg's [Config],Path setting but
+        // this causes issues if you try and use two different versions of Mixxx
+        // on the same computer. See Bug #1392854. We start by checking if we're
+        // running out of a build root ('res' dir exists or our path ends with
+        // '_build') and if not then we fall back on a platform-specific method
+        // of determining the resource path (see comments below).
+        if (mixxxDir.cd("res")) {
+            // We are running out of the repository root.
+            qResourcePath = mixxxDir.absolutePath();
+        } else if (mixxxDir.absolutePath().endsWith("_build") &&
+                   mixxxDir.cdUp() && mixxxDir.cd("res")) {
+            // We are running out of the (lin|win|osx)XX_build folder.
+            qResourcePath = mixxxDir.absolutePath();
+        }
 #ifdef __UNIX__
-    // On Linux, check if the path is stored in the configuration database.
-    if (getValueString(ConfigKey("[Config]","Path")).length()>0 && QDir(getValueString(ConfigKey("[Config]","Path"))).exists())
-        qConfigPath = getValueString(ConfigKey("[Config]","Path"));
-    else
-    {
-        // Set the path according to the compile time define, UNIX_SHARE_PATH
-        qConfigPath = UNIX_SHARE_PATH;
-    }
+        // On Linux if all of the above fail the /usr/share path is the logical
+        // place to look.
+        else {
+            qResourcePath = UNIX_SHARE_PATH;
+        }
 #endif
 #ifdef __WINDOWS__
-    // On Windows, set the config dir relative to the application dir
-    qConfigPath = QCoreApplication::applicationDirPath();
+        // On Windows, set the config dir relative to the application dir if all
+        // of the above fail.
+        else {
+            qResourcePath = QCoreApplication::applicationDirPath();
+        }
 #endif
 #ifdef __APPLE__
-    /*
-    // Set the path relative to the bundle directory
-    CFURLRef pluginRef = CFBundleCopyBundleURL(CFBundleGetMainBundle());
-    CFStringRef macPath = CFURLCopyFileSystemPath(pluginRef, kCFURLPOSIXPathStyle);
-    char utf8path[256];
-    //Attempt to decode obtain the macPath string as UTF-8
-    if (CFStringGetCString(macPath, utf8path, sizeof(utf8path), kCFStringEncodingUTF8))
-    {
-        qConfigPath.fromUtf8(utf8path);
-    }
-    else {
-        //Fallback on the "system encoding"... (this is just our old code, which probably doesn't make any sense
-         //since it plays roullette with the type of text encoding)
-        qConfigPath = CFStringGetCStringPtr(macPath, CFStringGetSystemEncoding());
-    }
-    qConfigPath.append("/Contents/Resources/"); //XXX this should really use QDir, this entire function should
-    */
-    QString mixxxPath = QCoreApplication::applicationDirPath();
-    if (mixxxPath.endsWith("osx_build"))   //Development configuration
-        qConfigPath = mixxxPath + "/../res";
-    else //Release configuraton
-	    qConfigPath = mixxxPath + "/../Resources";
+        else if (mixxxDir.cdUp() && mixxxDir.cd("Resources")) {
+            // Release configuraton
+            qResourcePath = mixxxDir.absolutePath();
+        } else {
+            // TODO(rryan): What should we do here?
+        }
 #endif
+    } else {
+        //qDebug() << "Setting qResourcePath from location in resourcePath commandline arg:" << qResourcePath;
     }
-	if (qConfigPath.length() == 0) qCritical() << "qConfigPath is empty, this can not be so -- did our developer forget to define one of __UNIX__, __WINDOWS__, __APPLE__??";
-    // If the directory does not end with a "/", add one
-    if (!qConfigPath.endsWith("/"))
-        qConfigPath.append("/");
 
-    return qConfigPath;
+    if (qResourcePath.isEmpty()) {
+        reportCriticalErrorAndQuit("qConfigPath is empty, this can not be so -- did our developer forget to define one of __UNIX__, __WINDOWS__, __APPLE__??");
+    }
+
+    // If the directory does not end with a "/", add one
+    if (!qResourcePath.endsWith("/")) {
+        qResourcePath.append("/");
+    }
+
+    qDebug() << "Loading resources from " << qResourcePath;
+
+    return qResourcePath;
 }
 
-
 template <class ValueType> ConfigObject<ValueType>::ConfigObject(QDomNode node) {
-
     if (!node.isNull() && node.isElement()) {
         QDomNode ctrl = node.firstChild();
 
         while (!ctrl.isNull()) {
             if(ctrl.nodeName() == "control") {
-                QString group = WWidget::selectNodeQString(ctrl, "group");
-                QString key = WWidget::selectNodeQString(ctrl, "key");
+                QString group = XmlParse::selectNodeQString(ctrl, "group");
+                QString key = XmlParse::selectNodeQString(ctrl, "key");
                 ConfigKey k(group, key);
                 ValueType m(ctrl);
                 set(k, m);
@@ -379,7 +409,19 @@ template <class ValueType> ConfigObject<ValueType>::ConfigObject(QDomNode node)
     }
 }
 
+template <class ValueType> QString ConfigObject<ValueType>::getSettingsPath() const
+{
+    QFileInfo configFileInfo(m_filename);
+    return configFileInfo.absoluteDir().absolutePath();
+}
+
+template <class ValueType> QHash<ConfigKey, ValueType> ConfigObject<ValueType>::toHash() const {
+    QHash<ConfigKey, ValueType> hash;
+    foreach (ConfigOption<ValueType>* pOption, m_list) {
+        hash.insert(*pOption->key, *pOption->val);
+    }
+    return hash;
+}
 
 template class ConfigObject<ConfigValue>;
 template class ConfigObject<ConfigValueKbd>;
-
