@@ -8,17 +8,25 @@
 #ifndef CONTROLGROUPDELEGATE_H_
 #define CONTROLGROUPDELEGATE_H_
 
+#include <QAbstractItemModel>
 #include <QItemDelegate>
 #include <QModelIndex>
 #include <QObject>
-#include <QSize>
-#include <QComboBox>
-#include <QLabel>
+#include <QPainter>
+#include <QStringList>
+#include <QStyleOptionViewItem>
+#include <QWidget>
 
 #define CONTROLGROUP_CHANNEL1_STRING "[Channel1]"
 #define CONTROLGROUP_CHANNEL2_STRING "[Channel2]"
+#define CONTROLGROUP_SAMPLER1_STRING "[Sampler1]"
+#define CONTROLGROUP_SAMPLER2_STRING "[Sampler2]"
+#define CONTROLGROUP_SAMPLER3_STRING "[Sampler3]"
+#define CONTROLGROUP_SAMPLER4_STRING "[Sampler4]"
 #define CONTROLGROUP_MASTER_STRING   "[Master]"
 #define CONTROLGROUP_PLAYLIST_STRING "[Playlist]"
+#define CONTROLGROUP_FLANGER_STRING  "[Flanger]"
+#define CONTROLGROUP_MICROPHONE_STRING  "[Microphone]"
 
 class ControlGroupDelegate : public QItemDelegate
 {
