@@ -27,12 +27,6 @@ inline QString userPresetsPath(ConfigObject<ConfigValue>* pConfig) {
     return dir.absolutePath().append("/");
 }
 
-inline QString localPresetsPath(ConfigObject<ConfigValue>* pConfig) {
-    QString presetsPath = pConfig->getSettingsPath();
-    QDir dir(presetsPath.append("/presets/"));
-    return dir.absolutePath().append("/");
-}
-
 #define HID_PRESET_EXTENSION ".hid.xml"
 #define MIDI_PRESET_EXTENSION ".midi.xml"
 #define BULK_PRESET_EXTENSION ".bulk.xml"
