@@ -15,25 +15,51 @@ class ControlObject;
 
 class SoftTakeover {
   public:
-    // Enable soft-takeover for the given Control This does nothing on a Control
-    // that's already enabled.
-    void enable(ControlObject* control);
-    // Disable soft-takeover for the given Control
-    void disable(ControlObject* control);
-    // Check to see if the new value for the Control should be ignored
-    bool ignore(ControlObject* control, float newValue, bool midiVal = false);
+    // I would initialize it here but that's C++11 coolness. (Because it's a double.)
+    static const double kDefaultTakeoverThreshold;
 
-    static uint currentTimeMsecs();
+    SoftTakeover();
+    bool ignore(ControlObject* control, double newParameter);
+    void ignoreNext();
+    void setThreshold(double threshold);
+    
+    struct TestAccess;
 
   private:
     // If a new value is received within this amount of time, jump to it
     // regardless. This allows quickly whipping controls to work while retaining
     // the benefits of soft-takeover for slower movements.  Setting this too
     // high will defeat the purpose of soft-takeover.
-    static const uint SUBSEQUENT_VALUE_OVERRIDE_TIME_MILLIS = 50;
+    static const qint64 SUBSEQUENT_VALUE_OVERRIDE_TIME_MILLIS = 50;
 
-    QHash<ControlObject*, uint> m_times;
-    QHash<ControlObject*, double> m_prevValues;
+    qint64 m_time;
+    double m_prevParameter;
+    double m_dThreshold;
+};
+
+struct SoftTakeover::TestAccess {
+    static qint64 getTimeThreshold() {
+        return SUBSEQUENT_VALUE_OVERRIDE_TIME_MILLIS;
+    }
+};
+
+class SoftTakeoverCtrl {
+  public:
+    SoftTakeoverCtrl();
+    ~SoftTakeoverCtrl();
+
+    // Enable soft-takeover for the given Control.
+    // This does nothing on a control that already has soft-takeover enabled.
+    void enable(ControlObject* control);
+    // Disable soft-takeover for the given Control
+    void disable(ControlObject* control);
+    // Check to see if the new value for the Control should be ignored
+    bool ignore(ControlObject* control, double newMidiParameter);
+    // Ignore the next supplied parameter
+    void ignoreNext(ControlObject* control);
+
+  private:
+    QHash<ControlObject*, SoftTakeover*> m_softTakeoverHash;
 };
 
 #endif
