@@ -20,31 +20,12 @@
    Purpose: Creates a new rotary encoder
    Input:   key
    -------- ------------------------------------------------------ */
-ControlTTRotary::ControlTTRotary(ConfigKey key) : ControlObject(key)
-{
+ControlTTRotary::ControlTTRotary(ConfigKey key) : ControlObject(key) {
+    if (m_pControl) {
+        m_pControl->setBehavior(
+                new ControlTTRotaryBehavior());
+    }
 }
 
-double ControlTTRotary::getValueFromWidget(double dValue)
-{
-    // Non-linear scaling
-    double temp = (((dValue-64.)*(dValue-64.))/64.);
-    if ((dValue-64.)<0)
-        temp = -temp;
 
-    //qDebug() << "tt rotary in " << dValue << ", out " << temp;
-
-    return temp; //dValue-64.;
-}
-
-double ControlTTRotary::getValueToWidget(double dValue)
-{
-    return dValue*200.+64.;
-}
-
-void ControlTTRotary::setValueFromMidi(MidiOpCode o, double v)
-{
-    Q_UNUSED(o);
-    m_dValue = v;
-    emit(valueChanged(m_dValue));
-}
 
