@@ -18,115 +18,130 @@
 #ifndef DLGPREFERENCES_H
 #define DLGPREFERENCES_H
 
- #include <QDialog>
+#include <QDialog>
+#include <QEvent>
+#include <QRect>
+#include <QStringList>
 
-#include <qevent.h>
-#include <QtGui>
 #include "ui_dlgpreferencesdlg.h"
 #include "configobject.h"
+#include "controlpushbutton.h"
+#include "preferences/dlgpreferencepage.h"
 
-class MixxxApp;
+class MixxxMainWindow;
 class SoundManager;
 class DlgPrefSound;
 class DlgPrefController;
-class DlgPrefNoControllers;
-class DlgPrefPlaylist;
+class DlgPrefControllers;
+class DlgPrefLibrary;
 class DlgPrefControls;
+class DlgPrefWaveform;
+class DlgPrefAutoDJ;
 class DlgPrefEQ;
+class DlgPrefEffects;
 class DlgPrefCrossfader;
 class DlgPrefRecord;
-class DlgPrefBpm;
+class DlgPrefKey;
 class DlgPrefBeats;
 class DlgPrefVinyl;
 class DlgPrefNoVinyl;
 class DlgPrefShoutcast;
 class DlgPrefReplayGain;
 class ControllerManager;
+class EffectsManager;
 class SkinLoader;
 class PlayerManager;
+class Library;
 class VinylControlManager;
+#ifdef __MODPLUG__
+class DlgPrefModplug;
+#endif
 
-/**
-  *@author Tue & Ken Haste Andersen
-  */
-
-class DlgPreferences : public QDialog, public Ui::DlgPreferencesDlg
-{
+class DlgPreferences : public QDialog, public Ui::DlgPreferencesDlg {
     Q_OBJECT
-public:
-    DlgPreferences(MixxxApp* mixxx, SkinLoader* pSkinLoader, SoundManager* soundman,
-//                    PlayerManager* pPlayerManager, ControllerManager* controllers, MidiDeviceManager* midi,
+  public:
+    DlgPreferences(MixxxMainWindow* mixxx, SkinLoader* pSkinLoader, SoundManager* soundman,
                    PlayerManager* pPlayerManager, ControllerManager* controllers,
-                   VinylControlManager* pVCManager, ConfigObject<ConfigValue>* config);
+                   VinylControlManager* pVCManager, EffectsManager* pEffectsManager,
+                   ConfigObject<ConfigValue>* pConfig, Library *pLibrary);
+    virtual ~DlgPreferences();
 
-    ~DlgPreferences();
-    void createIcons();
-public slots:
-    void slotShow();
-    void slotHide();
-    void rescanControllers();
-    void slotApply();
+    void addPageWidget(DlgPreferencePage* pWidget);
+    void removePageWidget(DlgPreferencePage* pWidget);
+    void expandTreeItem(QTreeWidgetItem* pItem);
+    void switchToPage(DlgPreferencePage* pWidget);
+
+  public slots:
     void changePage(QTreeWidgetItem* current, QTreeWidgetItem* previous);
     void showSoundHardwarePage();
-    void slotHighlightDevice(DlgPrefController* dialog, bool enabled);
-signals:
+    void slotButtonPressed(QAbstractButton* pButton);
+  signals:
     void closeDlg();
     void showDlg();
-protected:
+
+    // Emitted just after the user clicks Apply or OK.
+    void applyPreferences();
+    // Emitted if the user clicks Cancel
+    void cancelPreferences();
+    // Emitted if the user clicks Reset to Defaults.
+    void resetToDefaults();
+
+  protected:
     bool eventFilter(QObject*, QEvent*);
-private:
-    void destroyControllerWidgets();
-    void setupControllerWidgets();
-    int addPageWidget(QWidget* w);
-    QList<DlgPrefController*> m_controllerWindows;
-    
+    void moveEvent(QMoveEvent* e);
+    void resizeEvent(QResizeEvent* e);
+
+  private:
+    DlgPreferencePage* currentPage();
+    void createIcons();
+    void onShow();
+    void onHide();
+    QRect getDefaultGeometry();
+
+    QStringList m_geometry;
+    ConfigObject<ConfigValue>* m_pConfig;
     DlgPrefSound* m_wsound;
-    DlgPrefPlaylist* m_wplaylist;
-    DlgPrefNoControllers *m_wNoControllers;
+    DlgPrefLibrary* m_wlibrary;
+    DlgPrefControllers *m_wcontrollers;
     DlgPrefControls* m_wcontrols;
+    DlgPrefWaveform* m_wwaveform;
+    DlgPrefAutoDJ* m_wautodj;
     DlgPrefEQ* m_weq;
+    DlgPrefEffects* m_weffects;
     DlgPrefCrossfader* m_wcrossfader;
     DlgPrefRecord* m_wrecord;
-    DlgPrefBpm* m_wbpm;
+    DlgPrefKey* m_wkey;
     DlgPrefBeats* m_wbeats;
     DlgPrefVinyl* m_wvinylcontrol;
     DlgPrefNoVinyl* m_wnovinylcontrol;
     DlgPrefShoutcast* m_wshoutcast;
     DlgPrefReplayGain* m_wreplaygain;
-
-    /*
-    QScrollArea* m_sasound;
-    QScrollArea* m_saplaylist;
-//     QScrollArea* m_saNoMidi;
-    QScrollArea* m_sacontrols;
-    QScrollArea* m_saeq;
-    QScrollArea* m_sacrossfader;
-    QScrollArea* m_sarecord;
-    QScrollArea* m_sabpm;
-    QScrollArea* m_savinylcontrol;
-    QScrollArea* m_sanovinylcontrol;
-    QScrollArea* m_sashoutcast;
-    QScrollArea* m_sareplaygain;
-	*/
+#ifdef __MODPLUG__
+    DlgPrefModplug* m_wmodplug;
+#endif
 
     QTreeWidgetItem* m_pSoundButton;
-    QTreeWidgetItem* m_pPlaylistButton;
+    QTreeWidgetItem* m_pLibraryButton;
     QTreeWidgetItem* m_pControlsButton;
+    QTreeWidgetItem* m_pWaveformButton;
+    QTreeWidgetItem* m_pAutoDJButton;
     QTreeWidgetItem* m_pEqButton;
+    QTreeWidgetItem* m_pEffectsButton;
     QTreeWidgetItem* m_pCrossfaderButton;
     QTreeWidgetItem* m_pRecordingButton;
-    QTreeWidgetItem* m_pBPMdetectButton;
-    QTreeWidgetItem* m_pAnalysersButton;
+    QTreeWidgetItem* m_pBeatDetectionButton;
+    QTreeWidgetItem* m_pKeyDetectionButton;
     QTreeWidgetItem* m_pVinylControlButton;
     QTreeWidgetItem* m_pShoutcastButton;
     QTreeWidgetItem* m_pReplayGainButton;
+#ifdef __MODPLUG__
+    QTreeWidgetItem* m_pModplugButton;
+#endif
     QTreeWidgetItem* m_pControllerTreeItem;
-    QList<QTreeWidgetItem*> m_controllerWindowLinks;
 
     QSize m_pageSizeHint;
 
-    ConfigObject<ConfigValue>* config;
-    ControllerManager* m_pControllerManager;
+    ControlPushButton m_preferencesUpdated;
 };
 
 #endif
