@@ -16,19 +16,16 @@
 ***************************************************************************/
 
 
-#include <QtCore>
 #include <QtDebug>
-#include <QtGui>
 #include "dlgprefnovinyl.h"
 
 DlgPrefNoVinyl::DlgPrefNoVinyl(QWidget * parent, SoundManager * soundman,
                                ConfigObject<ConfigValue> * _config)
-        : QWidget(parent) {
+        : DlgPreferencePage(parent) {
     Q_UNUSED(soundman);
     Q_UNUSED(_config);
     setupUi(this);
 }
 
-DlgPrefNoVinyl::~DlgPrefNoVinyl()
-{
+DlgPrefNoVinyl::~DlgPrefNoVinyl() {
 }
