@@ -18,33 +18,35 @@
  ***************************************************************************/
 
 #include <QtDebug>
-#include <QtCore>
-#include <QtGui>
 
 #include "defs_urls.h"
 #include "dlgprefshoutcast.h"
 #include "shoutcast/defs_shoutcast.h"
+#include "controlobjectslave.h"
+
+const char* kDefaultMetadataFormat = "$artist - $title";
 
 DlgPrefShoutcast::DlgPrefShoutcast(QWidget *parent, ConfigObject<ConfigValue> *_config)
-        : QWidget(parent),
+        : DlgPreferencePage(parent),
           m_pConfig(_config) {
-    int tmp_index = 0;  //Used for finding the index of an element by name in a combobox.
-    QString tmp_string;
     setupUi(this);
 
-    m_pUpdateShoutcastFromPrefs = new ControlObjectThreadMain(
-        ControlObject::getControl(ConfigKey(SHOUTCAST_PREF_KEY, "update_from_prefs")));
+    m_pShoutcastEnabled = new ControlObjectSlave(
+            SHOUTCAST_PREF_KEY, "enabled", this);
+    m_pShoutcastEnabled->connectValueChanged(
+            SLOT(shoutcastEnabledChanged(double)));
+
 
     // Enable live broadcasting checkbox
-    enableLiveBroadcasting->setChecked((bool)m_pConfig->getValueString(
-        ConfigKey(SHOUTCAST_PREF_KEY,"enabled")).toInt());
+    enableLiveBroadcasting->setChecked(
+            m_pShoutcastEnabled->toBool());
 
     //Server type combobox
     comboBoxServerType->addItem(tr("Icecast 2"), SHOUTCAST_SERVER_ICECAST2);
-    comboBoxServerType->addItem(tr("Shoutcast"), SHOUTCAST_SERVER_SHOUTCAST);
+    comboBoxServerType->addItem(tr("Shoutcast 1"), SHOUTCAST_SERVER_SHOUTCAST);
     comboBoxServerType->addItem(tr("Icecast 1"), SHOUTCAST_SERVER_ICECAST1);
 
-    tmp_index = comboBoxServerType->findData(m_pConfig->getValueString(
+    int tmp_index = comboBoxServerType->findData(m_pConfig->getValueString(
         ConfigKey(SHOUTCAST_PREF_KEY,"servertype")));
     if (tmp_index < 0) //Set default if invalid.
         tmp_index = 0;
@@ -59,7 +61,7 @@ DlgPrefShoutcast::DlgPrefShoutcast(QWidget *parent, ConfigObject<ConfigValue> *_
         ConfigKey(SHOUTCAST_PREF_KEY,"host")));
 
     // Port
-    tmp_string = m_pConfig->getValueString(
+    QString tmp_string = m_pConfig->getValueString(
         ConfigKey(SHOUTCAST_PREF_KEY,"port"));
     if (tmp_string.isEmpty())
         tmp_string = QString(SHOUTCAST_DEFAULT_PORT);
@@ -85,14 +87,17 @@ DlgPrefShoutcast::DlgPrefShoutcast(QWidget *parent, ConfigObject<ConfigValue> *_
     stream_website->setText(tmp_string);
 
     // Stream description
-    stream_desc->setText(m_pConfig->getValueString(
-        ConfigKey(SHOUTCAST_PREF_KEY,"stream_desc")));
+    tmp_string = m_pConfig->getValueString(
+        ConfigKey(SHOUTCAST_PREF_KEY,"stream_desc"));
+    if (tmp_string.isEmpty())
+        tmp_string = tr("This stream is online for testing purposes!");
+    stream_desc->setText(tmp_string);
 
     // Stream genre
     tmp_string = m_pConfig->getValueString(
         ConfigKey(SHOUTCAST_PREF_KEY,"stream_genre"));
     if (tmp_string.isEmpty())
-        tmp_string = "Live Mix";
+        tmp_string = tr("Live Mix");
     stream_genre->setText(tmp_string);
 
     // Stream "public" checkbox
@@ -167,40 +172,108 @@ DlgPrefShoutcast::DlgPrefShoutcast(QWidget *parent, ConfigObject<ConfigValue> *_
     custom_title->setText(m_pConfig->getValueString(
         ConfigKey(SHOUTCAST_PREF_KEY,"custom_title")));
 
+    //Metadata format
+    tmp_string = m_pConfig->getValueString(
+        ConfigKey(SHOUTCAST_PREF_KEY,"metadata_format"));
+    if (tmp_string.isEmpty())
+        // No tr() here, see https://bugs.launchpad.net/mixxx/+bug/1419500
+        tmp_string = kDefaultMetadataFormat;
+    metadata_format->setText(tmp_string);
+
     slotApply();
 }
 
-DlgPrefShoutcast::~DlgPrefShoutcast()
-{
-    delete m_pUpdateShoutcastFromPrefs;
+DlgPrefShoutcast::~DlgPrefShoutcast() {
 }
 
-void DlgPrefShoutcast::slotUpdate()
-{
-    enableLiveBroadcasting->setChecked((bool)m_pConfig->getValueString(ConfigKey(SHOUTCAST_PREF_KEY,"enabled")).toInt());
+void DlgPrefShoutcast::slotResetToDefaults() {
+    // Make sure to keep these values in sync with the constructor.
+    enableLiveBroadcasting->setChecked(false);
+    comboBoxServerType->setCurrentIndex(0);
+    mountpoint->setText("");
+    host->setText("");
+    port->setText("8000");
+    login->setText("");
+    password->setText("");
+    stream_name->setText("");
+    stream_website->setText(MIXXX_WEBSITE_URL);
+    stream_desc->setText(tr("This stream is online for testing purposes!"));
+    stream_genre->setText(tr("Live Mix"));
+    stream_public->setChecked(false);
+    ogg_dynamicupdate->setChecked(false);
+    comboBoxEncodingBitrate->setCurrentIndex(comboBoxEncodingBitrate->findData(
+            SHOUTCAST_BITRATE_128KBPS));
+    comboBoxEncodingFormat->setCurrentIndex(0);
+    comboBoxEncodingChannels->setCurrentIndex(0);
+    enableUtf8Metadata->setChecked(false);
+    enableCustomMetadata->setChecked(false);
+    // No tr() here, see https://bugs.launchpad.net/mixxx/+bug/1419500
+    metadata_format->setText(kDefaultMetadataFormat);
+    custom_artist->setText("");
+    custom_title->setText("");
+}
+
+void DlgPrefShoutcast::slotUpdate() {
+    enableLiveBroadcasting->setChecked(m_pShoutcastEnabled->toBool());
+
+    // Don't let user modify information if
+    // sending is enabled.
+    if(m_pShoutcastEnabled->toBool()) {
+        this->setEnabled(false);
+    } else {
+        this->setEnabled(true);
+    }
 }
 
 void DlgPrefShoutcast::slotApply()
 {
-    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "enabled"),       ConfigValue(enableLiveBroadcasting->isChecked()));
+    m_pShoutcastEnabled->set(enableLiveBroadcasting->isChecked());
+
+    // Don't let user modify information if
+    // sending is enabled.
+    if(m_pShoutcastEnabled->toBool()) {
+        this->setEnabled(false);
+    } else {
+        this->setEnabled(true);
+    }
 
     // Combo boxes, make sure to load their data not their display strings.
-    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "servertype"),    ConfigValue(comboBoxServerType->itemData(comboBoxServerType->currentIndex()).toString()));
-    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "bitrate"),       ConfigValue(comboBoxEncodingBitrate->itemData(comboBoxEncodingBitrate->currentIndex()).toString()));
-    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "format"),        ConfigValue(comboBoxEncodingFormat->itemData(comboBoxEncodingFormat->currentIndex()).toString()));
-    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "channels"),      ConfigValue(comboBoxEncodingChannels->itemData(comboBoxEncodingChannels->currentIndex()).toString()));
+    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "servertype"),
+            ConfigValue(comboBoxServerType->itemData(
+                            comboBoxServerType->currentIndex()).toString()));
+    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "bitrate"),
+            ConfigValue(comboBoxEncodingBitrate->itemData(
+                            comboBoxEncodingBitrate->currentIndex()).toString()));
+    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "format"),
+            ConfigValue(comboBoxEncodingFormat->itemData(
+                            comboBoxEncodingFormat->currentIndex()).toString()));
+    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "channels"),
+            ConfigValue(comboBoxEncodingChannels->itemData(
+                            comboBoxEncodingChannels->currentIndex()).toString()));
 
-    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "mountpoint"),    ConfigValue(mountpoint->text()));
-    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "host"),          ConfigValue(host->text()));
-    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "port"),          ConfigValue(port->text()));
-    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "login"),         ConfigValue(login->text()));
-    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "password"),      ConfigValue(password->text()));
-    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "stream_name"),   ConfigValue(stream_name->text()));
-    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "stream_website"),ConfigValue(stream_website->text()));
-    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "stream_desc"),   ConfigValue(stream_desc->toPlainText()));
-    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "stream_genre"),  ConfigValue(stream_genre->text()));
-    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "stream_public"), ConfigValue(stream_public->isChecked()));
-    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "ogg_dynamicupdate"), ConfigValue(ogg_dynamicupdate->isChecked()));
+    mountpoint->setText(mountpoint->text().trimmed());
+    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "mountpoint"),
+            ConfigValue(mountpoint->text()));
+    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "host"),
+            ConfigValue(host->text()));
+    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "port"),
+            ConfigValue(port->text()));
+    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "login"),
+            ConfigValue(login->text()));
+    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "password"),
+            ConfigValue(password->text()));
+    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "stream_name"),
+            ConfigValue(stream_name->text()));
+    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "stream_website"),
+            ConfigValue(stream_website->text()));
+    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "stream_desc"),
+            ConfigValue(stream_desc->toPlainText()));
+    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "stream_genre"),
+            ConfigValue(stream_genre->text()));
+    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "stream_public"),
+            ConfigValue(stream_public->isChecked()));
+    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "ogg_dynamicupdate"),
+            ConfigValue(ogg_dynamicupdate->isChecked()));
 
     QString charset = "";
     if (enableUtf8Metadata->isChecked()) {
@@ -220,7 +293,13 @@ void DlgPrefShoutcast::slotApply()
     m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "enable_metadata"),ConfigValue(enableCustomMetadata->isChecked()));
     m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "custom_artist"), ConfigValue(custom_artist->text()));
     m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "custom_title"),  ConfigValue(custom_title->text()));
+    m_pConfig->set(ConfigKey(SHOUTCAST_PREF_KEY, "metadata_format"), ConfigValue(metadata_format->text()));
+}
 
-    //Tell the EngineShoutcast object to update with these values by toggling this control object.
-    m_pUpdateShoutcastFromPrefs->slotSet(1.0f);
+void DlgPrefShoutcast::shoutcastEnabledChanged(double value) {
+    qDebug() << "DlgPrefShoutcast::shoutcastEnabledChanged()" << value;
+    bool enabled = value == 1.0; // 0 and 2 are disabled
+    this->setEnabled(!enabled);
+    enableLiveBroadcasting->setChecked(enabled);
+
 }
