@@ -16,17 +16,17 @@
 #ifndef DLGPREFSOUND_H
 #define DLGPREFSOUND_H
 
-#include <QtCore>
 #include "ui_dlgprefsounddlg.h"
 #include "configobject.h"
 #include "soundmanagerconfig.h"
+#include "preferences/dlgpreferencepage.h"
 
 class SoundManager;
 class PlayerManager;
 class ControlObject;
 class SoundDevice;
 class DlgPrefSoundItem;
-class ControlObjectThreadMain;
+class ControlObjectSlave;
 
 /*
  * TODO(bkgood) (n-decks) establish a signal/slot connection with a signal
@@ -37,53 +37,73 @@ class ControlObjectThreadMain;
 /**
  * Class representing a preferences pane to configure sound devices for Mixxx.
  */
-class DlgPrefSound : public QWidget, public Ui::DlgPrefSoundDlg  {
+class DlgPrefSound : public DlgPreferencePage, public Ui::DlgPrefSoundDlg  {
     Q_OBJECT;
-public:
+  public:
     DlgPrefSound(QWidget *parent, SoundManager *soundManager,
                  PlayerManager* pPlayerManager,
                  ConfigObject<ConfigValue> *config);
-    ~DlgPrefSound();
-signals:
+    virtual ~DlgPrefSound();
+
+  signals:
     void loadPaths(const SoundManagerConfig &config);
     void writePaths(SoundManagerConfig *config);
     void refreshOutputDevices(const QList<SoundDevice*> &devices);
     void refreshInputDevices(const QList<SoundDevice*> &devices);
     void updatingAPI();
     void updatedAPI();
-public slots:
+
+  public slots:
     void slotUpdate(); // called on show
     void slotApply();  // called on ok button
-    void forceApply(); // called by DlgPrefVinyl to make slotApply call setupDevices
+    void slotResetToDefaults();
     void bufferUnderflow(double count);
-private:
-    void initializePaths();
-    void connectSoundItem(DlgPrefSoundItem *item);
-    void loadSettings(const SoundManagerConfig &config);
-    void insertItem(DlgPrefSoundItem *pItem, QVBoxLayout *pLayout);
-    SoundManager *m_pSoundManager;
-    PlayerManager *m_pPlayerManager;
-    ConfigObject<ConfigValue> *m_pConfig;
-    ControlObjectThreadMain* m_pMasterUnderflowCount;
-    QList<SoundDevice*> m_inputDevices;
-    QList<SoundDevice*> m_outputDevices;
-    bool m_settingsModified;
-    SoundManagerConfig m_config;
-    bool m_loading;
-    bool m_forceApply;
-private slots:
+    void masterLatencyChanged(double latency);
+    void headDelayChanged(double value);
+    void masterDelayChanged(double value);
+    void masterMixChanged(int value);
+    void masterEnabledChanged(double value);
+    void masterOutputModeComboBoxChanged(int value);
+    void masterMonoMixdownChanged(double value);
+    void talkoverMixComboBoxChanged(int value);
+    void talkoverMixChanged(double value);
+
+  private slots:
     void addPath(AudioOutput output);
     void addPath(AudioInput input);
     void loadSettings();
     void apiChanged(int index);
     void updateAPIs();
     void sampleRateChanged(int index);
-    void latencyChanged(int index);
-    void updateLatencies(int sampleRateIndex);
+    void audioBufferChanged(int index);
+    void updateAudioBufferSizes(int sampleRateIndex);
+    void syncBuffersChanged(int index);
     void refreshDevices();
     void settingChanged();
     void queryClicked();
-    void resetClicked();
+
+  private:
+    void initializePaths();
+    void connectSoundItem(DlgPrefSoundItem *item);
+    void loadSettings(const SoundManagerConfig &config);
+    void insertItem(DlgPrefSoundItem *pItem, QVBoxLayout *pLayout);
+
+    SoundManager *m_pSoundManager;
+    PlayerManager *m_pPlayerManager;
+    ConfigObject<ConfigValue> *m_pConfig;
+    ControlObjectSlave* m_pMasterAudioLatencyOverloadCount;
+    ControlObjectSlave* m_pMasterLatency;
+    ControlObjectSlave* m_pHeadDelay;
+    ControlObjectSlave* m_pMasterDelay;
+    ControlObjectSlave* m_pKeylockEngine;
+    ControlObjectSlave* m_pMasterEnabled;
+    ControlObjectSlave* m_pMasterMonoMixdown;
+    ControlObjectSlave* m_pMasterTalkoverMix;
+    QList<SoundDevice*> m_inputDevices;
+    QList<SoundDevice*> m_outputDevices;
+    bool m_settingsModified;
+    SoundManagerConfig m_config;
+    bool m_loading;
 };
 
 #endif
