@@ -7,80 +7,95 @@
 
 #include "controlobject.h"
 #include "controlpushbutton.h"
+#include "controlindicator.h"
 #include "trackinfoobject.h"
 #include "library/dao/cue.h"
 #include "cachingreader.h"
-#include "mathstuff.h"
+#include "vinylcontrol/defs_vinylcontrol.h"
 
-CueControl::CueControl(const char * _group,
-                       ConfigObject<ConfigValue> * _config) :
-        EngineControl(_group, _config),
+// TODO: Convert these doubles to a standard enum
+// and convert elseif logic to switch statements
+static const double CUE_MODE_MIXXX = 0.0;
+static const double CUE_MODE_PIONEER = 1.0;
+static const double CUE_MODE_DENON = 2.0;
+static const double CUE_MODE_NUMARK = 3.0;
+static const double CUE_MODE_MIXXX_NO_BLINK = 4.0;
+
+CueControl::CueControl(QString group,
+                       ConfigObject<ConfigValue>* _config) :
+        EngineControl(group, _config),
         m_bPreviewing(false),
-        m_bPreviewingHotcue(false),
-        m_bHotcueCancel(false),
-        m_pPlayButton(ControlObject::getControl(ConfigKey(_group, "play"))),
+        m_pPlayButton(ControlObject::getControl(ConfigKey(group, "play"))),
+        m_pStopButton(ControlObject::getControl(ConfigKey(group, "stop"))),
         m_iCurrentlyPreviewingHotcues(0),
+        m_bypassCueSetByPlay(false),
         m_iNumHotCues(NUM_HOT_CUES),
         m_pLoadedTrack(),
         m_mutex(QMutex::Recursive) {
+    // To silence a compiler warning about CUE_MODE_PIONEER.
+    Q_UNUSED(CUE_MODE_PIONEER);
     createControls();
 
-    m_pTrackSamples = ControlObject::getControl(ConfigKey(_group, "track_samples"));
+    m_pTrackSamples = ControlObject::getControl(ConfigKey(group, "track_samples"));
 
-    m_pQuantizeEnabled = ControlObject::getControl(ConfigKey(_group, "quantize"));
+    m_pQuantizeEnabled = ControlObject::getControl(ConfigKey(group, "quantize"));
 
-    m_pNextBeat = ControlObject::getControl(ConfigKey(_group, "beat_next"));
-    m_pClosestBeat = ControlObject::getControl(ConfigKey(_group, "beat_closest"));
+    m_pNextBeat = ControlObject::getControl(ConfigKey(group, "beat_next"));
+    m_pClosestBeat = ControlObject::getControl(ConfigKey(group, "beat_closest"));
 
-    m_pCuePoint = new ControlObject(ConfigKey(_group, "cue_point"));
-    m_pCueMode = new ControlObject(ConfigKey(_group, "cue_mode"));
-    m_pCuePoint->set(-1);
+    m_pCuePoint = new ControlObject(ConfigKey(group, "cue_point"));
+    m_pCuePoint->set(-1.0);
 
-    m_pCueSet = new ControlPushButton(ConfigKey(_group, "cue_set"));
+    m_pCueMode = new ControlObject(ConfigKey(group, "cue_mode"));
+
+    m_pCueSet = new ControlPushButton(ConfigKey(group, "cue_set"));
+    m_pCueSet->setButtonMode(ControlPushButton::TRIGGER);
     connect(m_pCueSet, SIGNAL(valueChanged(double)),
             this, SLOT(cueSet(double)),
             Qt::DirectConnection);
 
-    m_pCueGoto = new ControlPushButton(ConfigKey(_group, "cue_goto"));
+    m_pCueGoto = new ControlPushButton(ConfigKey(group, "cue_goto"));
     connect(m_pCueGoto, SIGNAL(valueChanged(double)),
             this, SLOT(cueGoto(double)),
             Qt::DirectConnection);
 
     m_pCueGotoAndPlay =
-            new ControlPushButton(ConfigKey(_group, "cue_gotoandplay"));
+            new ControlPushButton(ConfigKey(group, "cue_gotoandplay"));
     connect(m_pCueGotoAndPlay, SIGNAL(valueChanged(double)),
             this, SLOT(cueGotoAndPlay(double)),
             Qt::DirectConnection);
 
     m_pCueGotoAndStop =
-            new ControlPushButton(ConfigKey(_group, "cue_gotoandstop"));
+            new ControlPushButton(ConfigKey(group, "cue_gotoandstop"));
     connect(m_pCueGotoAndStop, SIGNAL(valueChanged(double)),
             this, SLOT(cueGotoAndStop(double)),
             Qt::DirectConnection);
 
-    m_pCueSimple = new ControlPushButton(ConfigKey(_group, "cue_simple"));
-    connect(m_pCueSimple, SIGNAL(valueChanged(double)),
-            this, SLOT(cueSimple(double)),
-            Qt::DirectConnection);
-
-    m_pCuePreview = new ControlPushButton(ConfigKey(_group, "cue_preview"));
+    m_pCuePreview = new ControlPushButton(ConfigKey(group, "cue_preview"));
     connect(m_pCuePreview, SIGNAL(valueChanged(double)),
             this, SLOT(cuePreview(double)),
             Qt::DirectConnection);
 
-    m_pCueCDJ = new ControlPushButton(ConfigKey(_group, "cue_cdj"));
+    m_pCueCDJ = new ControlPushButton(ConfigKey(group, "cue_cdj"));
     connect(m_pCueCDJ, SIGNAL(valueChanged(double)),
             this, SLOT(cueCDJ(double)),
             Qt::DirectConnection);
 
-    m_pCueDefault = new ControlPushButton(ConfigKey(_group, "cue_default"));
+    m_pCueDefault = new ControlPushButton(ConfigKey(group, "cue_default"));
     connect(m_pCueDefault, SIGNAL(valueChanged(double)),
             this, SLOT(cueDefault(double)),
             Qt::DirectConnection);
 
-    connect(m_pPlayButton, SIGNAL(valueChanged(double)),
-            this, SLOT(playFromCuePreview(double)),
+    m_pPlayStutter = new ControlPushButton(ConfigKey(group, "play_stutter"));
+    connect(m_pPlayStutter, SIGNAL(valueChanged(double)),
+            this, SLOT(playStutter(double)),
             Qt::DirectConnection);
+
+    m_pCueIndicator = new ControlIndicator(ConfigKey(group, "cue_indicator"));
+    m_pPlayIndicator = new ControlIndicator(ConfigKey(group, "play_indicator"));
+
+    m_pVinylControlEnabled = new ControlObjectSlave(group, "vinylcontrol_enabled");
+    m_pVinylControlMode = new ControlObjectSlave(group, "vinylcontrol_mode");
 }
 
 CueControl::~CueControl() {
@@ -90,14 +105,15 @@ CueControl::~CueControl() {
     delete m_pCueGoto;
     delete m_pCueGotoAndPlay;
     delete m_pCueGotoAndStop;
-    delete m_pCueSimple;
     delete m_pCuePreview;
     delete m_pCueCDJ;
     delete m_pCueDefault;
-    while (m_hotcueControl.size() > 0) {
-        HotcueControl* pControl = m_hotcueControl.takeLast();
-        delete pControl;
-    }
+    delete m_pPlayStutter;
+    delete m_pCueIndicator;
+    delete m_pPlayIndicator;
+    delete m_pVinylControlEnabled;
+    delete m_pVinylControlMode;
+    qDeleteAll(m_hotcueControl);
 }
 
 void CueControl::createControls() {
@@ -134,33 +150,38 @@ void CueControl::createControls() {
 }
 
 void CueControl::attachCue(Cue* pCue, int hotCue) {
-    if (hotCue < 0 || hotCue >= m_iNumHotCues) {
+    HotcueControl* pControl = m_hotcueControl.value(hotCue, NULL);
+    if (pControl == NULL) {
         return;
     }
-    HotcueControl* pControl = m_hotcueControl[hotCue];
     if (pControl->getCue() != NULL) {
         detachCue(pControl->getHotcueNumber());
     }
-    pControl->setCue(pCue);
     connect(pCue, SIGNAL(updated()),
             this, SLOT(cueUpdated()),
             Qt::DirectConnection);
 
     pControl->getPosition()->set(pCue->getPosition());
     pControl->getEnabled()->set(pCue->getPosition() == -1 ? 0.0 : 1.0);
+    // set pCue only if all other data is in place
+    // because we have a null check for valid data else where  in the code
+    pControl->setCue(pCue);
+
 }
 
 void CueControl::detachCue(int hotCue) {
-    if (hotCue < 0 || hotCue >= m_iNumHotCues) {
+    HotcueControl* pControl = m_hotcueControl.value(hotCue, NULL);
+    if (pControl == NULL) {
         return;
     }
-    HotcueControl* pControl = m_hotcueControl[hotCue];
     Cue* pCue = pControl->getCue();
     if (!pCue)
         return;
     disconnect(pCue, 0, this, 0);
+    // clear pCue first because we have a null check for valid data else where
+    // in the code
     pControl->setCue(NULL);
-    pControl->getPosition()->set(-1);
+    pControl->getPosition()->set(-1); // invalidate position for hintReader()
     pControl->getEnabled()->set(0);
 }
 
@@ -193,23 +214,34 @@ void CueControl::trackLoaded(TrackPointer pTrack) {
             attachCue(pCue, hotcue);
     }
 
+    double loadCuePoint = 0.0;
+    // If cue recall is ON in the prefs, then we're supposed to seek to the cue
+    // point on song load. Note that [Controls],cueRecall == 0 corresponds to "ON", not OFF.
+    bool cueRecall = (getConfig()->getValueString(
+                ConfigKey("[Controls]","CueRecall"), "0").toInt() == 0);
     if (loadCue != NULL) {
         m_pCuePoint->set(loadCue->getPosition());
+        if (cueRecall) {
+            loadCuePoint = loadCue->getPosition();
+        }
     } else {
-        m_pCuePoint->set(0.0f);
+        // If no cue point is stored, set one at track start
+        m_pCuePoint->set(0.0);
     }
 
-    int cueRecall = getConfig()->getValueString(
-        ConfigKey("[Controls]","CueRecall")).toInt();
-    //If cue recall is ON in the prefs, then we're supposed to seek to the cue
-    //point on song load. Note that cueRecall == 0 corresponds to "ON", not OFF.
-    double loadCuePoint = 0;
-    if (loadCue && cueRecall == 0) {
-        loadCuePoint = loadCue->getPosition();
-    }
     // Need to unlock before emitting any signals to prevent deadlock.
     lock.unlock();
-    emit(seekAbs(loadCuePoint));
+    // If cueRecall is on, seek to it even if we didn't find a cue value (we'll
+    // seek to 0.
+    if (cueRecall) {
+        seekExact(loadCuePoint);
+    } else if (!(m_pVinylControlEnabled->get() &&
+            m_pVinylControlMode->get() == MIXXX_VCMODE_ABSOLUTE)) {
+        // If cuerecall is off, seek to zero unless
+        // vinylcontrol is on and set to absolute.  This allows users to
+        // load tracks and have the needle-drop be maintained.
+        seekExact(0.0);
+    }
 }
 
 void CueControl::trackUnloaded(TrackPointer pTrack) {
@@ -222,7 +254,7 @@ void CueControl::trackUnloaded(TrackPointer pTrack) {
     // Store the cue point in a load cue.
     double cuePoint = m_pCuePoint->get();
 
-    if (cuePoint != -1 && cuePoint != 0.0f) {
+    if (cuePoint != -1 && cuePoint != 0.0) {
         Cue* loadCue = NULL;
         const QList<Cue*>& cuePoints = pTrack->getCuePoints();
         QListIterator<Cue*> it(cuePoints);
@@ -241,6 +273,8 @@ void CueControl::trackUnloaded(TrackPointer pTrack) {
         loadCue->setPosition(cuePoint);
     }
 
+    m_pCueIndicator->setBlinkValue(ControlIndicator::OFF);
+    m_pCuePoint->set(-1.0);
     m_pLoadedTrack.clear();
 }
 
@@ -266,7 +300,13 @@ void CueControl::trackCuesUpdated() {
 
         int hotcue = pCue->getHotCue();
         if (hotcue != -1) {
-            HotcueControl* pControl = m_hotcueControl[hotcue];
+            HotcueControl* pControl = m_hotcueControl.value(hotcue, NULL);
+
+            // Cue's hotcue doesn't have a hotcue control.
+            if (pControl == NULL) {
+                continue;
+            }
+
             Cue* pOldCue = pControl->getCue();
 
             // If the old hotcue is different than this one.
@@ -315,8 +355,8 @@ void CueControl::hotcueSet(HotcueControl* pControl, double v) {
     Cue* pCue = m_pLoadedTrack->addCue();
     double cuePosition =
             (m_pQuantizeEnabled->get() > 0.0 && m_pClosestBeat->get() != -1) ?
-            floorf(m_pClosestBeat->get()) : floorf(getCurrentSample());
-    if (!even(cuePosition))
+            floor(m_pClosestBeat->get()) : floor(getCurrentSample());
+    if (!even(static_cast<int>(cuePosition)))
         cuePosition--;
     pCue->setPosition(cuePosition);
     pCue->setHotCue(hotcue);
@@ -328,10 +368,11 @@ void CueControl::hotcueSet(HotcueControl* pControl, double v) {
     // If quantize is enabled and we are not playing, jump to the cue point
     // since it's not necessarily where we currently are. TODO(XXX) is this
     // potentially invalid for vinyl control?
-    bool playing = m_pPlayButton->get() > 0;
+    bool playing = m_pPlayButton->toBool();
     if (!playing && m_pQuantizeEnabled->get() > 0.0) {
         lock.unlock();  // prevent deadlock.
-        emit(seekAbs(cuePosition));
+        // Enginebuffer will quantize more exactly than we can.
+        seekAbs(cuePosition);
     }
 }
 
@@ -340,8 +381,9 @@ void CueControl::hotcueGoto(HotcueControl* pControl, double v) {
         return;
 
     QMutexLocker lock(&m_mutex);
-    if (!m_pLoadedTrack)
+    if (!m_pLoadedTrack) {
         return;
+    }
 
     Cue* pCue = pControl->getCue();
 
@@ -351,7 +393,7 @@ void CueControl::hotcueGoto(HotcueControl* pControl, double v) {
     if (pCue) {
         int position = pCue->getPosition();
         if (position != -1) {
-            emit(seekAbs(position));
+            seekAbs(position);
         }
     }
 }
@@ -373,7 +415,7 @@ void CueControl::hotcueGotoAndStop(HotcueControl* pControl, double v) {
         int position = pCue->getPosition();
         if (position != -1) {
             m_pPlayButton->set(0.0);
-            emit(seekAbs(position));
+            seekExact(position);
         }
     }
 }
@@ -383,8 +425,9 @@ void CueControl::hotcueGotoAndPlay(HotcueControl* pControl, double v) {
         return;
 
     QMutexLocker lock(&m_mutex);
-    if (!m_pLoadedTrack)
+    if (!m_pLoadedTrack) {
         return;
+    }
 
     Cue* pCue = pControl->getCue();
 
@@ -394,7 +437,9 @@ void CueControl::hotcueGotoAndPlay(HotcueControl* pControl, double v) {
     if (pCue) {
         int position = pCue->getPosition();
         if (position != -1) {
-            emit(seekAbs(position));
+            seekAbs(position);
+            // don't move the cue point to the hot cue point in DENON mode
+            m_bypassCueSetByPlay = true;
             m_pPlayButton->set(1.0);
         }
     }
@@ -405,8 +450,9 @@ void CueControl::hotcueActivate(HotcueControl* pControl, double v) {
 
     QMutexLocker lock(&m_mutex);
 
-    if (!m_pLoadedTrack)
+    if (!m_pLoadedTrack) {
         return;
+    }
 
     Cue* pCue = pControl->getCue();
 
@@ -414,11 +460,12 @@ void CueControl::hotcueActivate(HotcueControl* pControl, double v) {
 
     if (pCue) {
         if (v) {
-            m_bHotcueCancel = false;
             if (pCue->getPosition() == -1) {
                 hotcueSet(pControl, v);
             } else {
-                if (!m_bPreviewingHotcue && m_pPlayButton->get() == 1.0f) {
+                if (!m_iCurrentlyPreviewingHotcues &&
+                        !m_bPreviewing &&
+                        m_pPlayButton->toBool()) {
                     hotcueGoto(pControl, v);
                 } else {
                     hotcueActivatePreview(pControl, v);
@@ -432,9 +479,8 @@ void CueControl::hotcueActivate(HotcueControl* pControl, double v) {
     } else {
         if (v) {
             // just in case
-            m_bHotcueCancel = false;
             hotcueSet(pControl, v);
-        } else if (m_bPreviewingHotcue) {
+        } else if (m_iCurrentlyPreviewingHotcues) {
             // The cue is non-existent, yet we got a release for it and are
             // currently previewing a hotcue. This is indicative of a corner
             // case where the cue was detached while we were pressing it. Let
@@ -446,25 +492,26 @@ void CueControl::hotcueActivate(HotcueControl* pControl, double v) {
 
 void CueControl::hotcueActivatePreview(HotcueControl* pControl, double v) {
     QMutexLocker lock(&m_mutex);
-    if (!m_pLoadedTrack)
+    if (!m_pLoadedTrack) {
         return;
+    }
     Cue* pCue = pControl->getCue();
 
     if (v) {
         if (pCue && pCue->getPosition() != -1) {
             m_iCurrentlyPreviewingHotcues++;
             int iPosition = pCue->getPosition();
+            m_bypassCueSetByPlay = true;
             m_pPlayButton->set(1.0);
-            m_bPreviewingHotcue = true;
             pControl->setPreviewing(true);
             pControl->setPreviewingPosition(iPosition);
 
             // Need to unlock before emitting any signals to prevent deadlock.
             lock.unlock();
 
-            emit(seekAbs(iPosition));
+            seekAbs(iPosition);
         }
-    } else if (m_bPreviewingHotcue) {
+    } else if (m_iCurrentlyPreviewingHotcues) {
         // This is a activate release and we are previewing at least one
         // hotcue. If this hotcue is previewing:
         if (pControl->isPreviewing()) {
@@ -474,23 +521,11 @@ void CueControl::hotcueActivatePreview(HotcueControl* pControl, double v) {
             pControl->setPreviewingPosition(-1);
 
             // If this is the last hotcue to leave preview.
-            if (--m_iCurrentlyPreviewingHotcues == 0) {
-                bool bHotcueCancel = m_bHotcueCancel;
-                m_bPreviewingHotcue = false;
-                m_bHotcueCancel = false;
-                // If hotcue cancel is marked then do not snap back to the
-                // hotcue and stop. Otherwise, seek back to the start point and
-                // stop.
-                if (bHotcueCancel) {
-                    // Re-trigger the play button value so controllers get the correct one
-                    // after.
-                    m_pPlayButton->set(m_pPlayButton->get());
-                } else {
-                    m_pPlayButton->set(0.0);
-                    // Need to unlock before emitting any signals to prevent deadlock.
-                    lock.unlock();
-                    emit(seekAbs(iPosition));
-                }
+            if (--m_iCurrentlyPreviewingHotcues == 0 && !m_bPreviewing) {
+                m_pPlayButton->set(0.0);
+                // Need to unlock before emitting any signals to prevent deadlock.
+                lock.unlock();
+                seekExact(iPosition);
             }
         }
     }
@@ -501,8 +536,9 @@ void CueControl::hotcueClear(HotcueControl* pControl, double v) {
         return;
 
     QMutexLocker lock(&m_mutex);
-    if (!m_pLoadedTrack)
+    if (!m_pLoadedTrack) {
         return;
+    }
 
     Cue* pCue = pControl->getCue();
     if (pCue) {
@@ -534,31 +570,30 @@ void CueControl::hotcuePositionChanged(HotcueControl* pControl, double newPositi
     }
 }
 
-void CueControl::hintReader(QList<Hint>& hintList) {
-    QMutexLocker lock(&m_mutex);
-
+void CueControl::hintReader(HintVector* pHintList) {
     Hint cue_hint;
     double cuePoint = m_pCuePoint->get();
     if (cuePoint >= 0) {
         cue_hint.sample = m_pCuePoint->get();
         cue_hint.length = 0;
         cue_hint.priority = 10;
-        hintList.append(cue_hint);
+        pHintList->append(cue_hint);
     }
 
-    for (int i = 0; i < m_iNumHotCues; ++i) {
-        HotcueControl* pControl = m_hotcueControl[i];
-        Cue *pCue = pControl->getCue();
-        if (pCue != NULL) {
-            double position = pControl->getPosition()->get();
-            if (position != -1) {
-                cue_hint.sample = position;
-                if (cue_hint.sample % 2 != 0)
-                    cue_hint.sample--;
-                cue_hint.length = 0;
-                cue_hint.priority = 10;
-                hintList.push_back(cue_hint);
-            }
+    // this is called from the engine thread
+    // it is no locking required, because m_hotcueControl is filled during the
+    // constructor and getPosition()->get() is a ControlObject
+    for (QList<HotcueControl*>::const_iterator it = m_hotcueControl.constBegin();
+         it != m_hotcueControl.constEnd(); ++it) {
+        HotcueControl* pControl = *it;
+        double position = pControl->getPosition()->get();
+        if (position != -1) {
+            cue_hint.sample = position;
+            if (cue_hint.sample % 2 != 0)
+                cue_hint.sample--;
+            cue_hint.length = 0;
+            cue_hint.priority = 10;
+            pHintList->append(cue_hint);
         }
     }
 }
@@ -569,14 +604,16 @@ void CueControl::saveCuePoint(double cuePoint) {
     }
 }
 
+// Moves the cue point to current position or to closest beat in case
+// quantize is enabled
 void CueControl::cueSet(double v) {
     if (!v)
         return;
 
     QMutexLocker lock(&m_mutex);
     double cue = (m_pQuantizeEnabled->get() > 0.0 && m_pClosestBeat->get() != -1) ?
-            floorf(m_pClosestBeat->get()) : floorf(getCurrentSample());
-    if (!even(cue))
+            floor(m_pClosestBeat->get()) : floor(getCurrentSample());
+    if (!even(static_cast<int>(cue)))
         cue--;
     m_pCuePoint->set(cue);
     saveCuePoint(cue);
@@ -594,7 +631,7 @@ void CueControl::cueGoto(double v)
     // Need to unlock before emitting any signals to prevent deadlock.
     lock.unlock();
 
-    emit(seekAbs(cuePoint));
+    seekAbs(cuePoint);
 }
 
 void CueControl::cueGotoAndPlay(double v)
@@ -604,7 +641,10 @@ void CueControl::cueGotoAndPlay(double v)
     cueGoto(v);
     QMutexLocker lock(&m_mutex);
     // Start playing if not already
-    if (m_pPlayButton->get()==0.) {
+    if (!m_pPlayButton->toBool()) {
+        // cueGoto is processed asynchrony.
+        // avoid a wrong cue set if seek by cueGoto is still pending
+        m_bypassCueSetByPlay = true;
         m_pPlayButton->set(1.0);
     }
 }
@@ -621,7 +661,7 @@ void CueControl::cueGotoAndStop(double v)
     // Need to unlock before emitting any signals to prevent deadlock.
     lock.unlock();
 
-    emit(seekAbs(cuePoint));
+    seekExact(cuePoint);
 }
 
 void CueControl::cuePreview(double v)
@@ -629,134 +669,319 @@ void CueControl::cuePreview(double v)
     QMutexLocker lock(&m_mutex);
 
     if (v) {
-        m_pPlayButton->set(1.0);
         m_bPreviewing = true;
+        m_bypassCueSetByPlay = true;
+        m_pPlayButton->set(1.0);
     } else if (!v && m_bPreviewing) {
-        m_pPlayButton->set(0.0);
         m_bPreviewing = false;
-    }
-
-    double cuePoint = m_pCuePoint->get();
-
-    // Need to unlock before emitting any signals to prevent deadlock.
-    lock.unlock();
-
-    emit(seekAbs(cuePoint));
-}
-
-void CueControl::cueSimple(double v) {
-    if (!v)
+        if (!m_iCurrentlyPreviewingHotcues) {
+            m_pPlayButton->set(0.0);
+        } else {
+            return;
+        }
+    } else {
         return;
-
-    QMutexLocker lock(&m_mutex);
-    // Simple cueing is if the player is not playing, set the cue point --
-    // otherwise seek to the cue point.
-    if (m_pPlayButton->get() == 0.0f) {
-        return cueSet(v);
     }
 
-    double cuePoint = m_pCuePoint->get();
-
     // Need to unlock before emitting any signals to prevent deadlock.
     lock.unlock();
 
-    emit(seekAbs(cuePoint));
+    seekAbs(m_pCuePoint->get());
 }
 
 void CueControl::cueCDJ(double v) {
-    /* This is how CDJ cue buttons work:
-     * If pressed while playing, stop playback and go to cue.
-     * If pressed while stopped and at cue, play while pressed.
-     * If pressed while stopped and not at cue, set new cue point.
-     * If play is pressed while holding cue, the deck is now playing. (Handled in playFromCuePreview().)
-     */
+    // This is how Pioneer cue buttons work:
+    // If pressed while playing, stop playback and go to cue.
+    // If pressed while stopped and at cue, play while pressed.
+    // If pressed while stopped and not at cue, set new cue point.
+    // If play is pressed while holding cue, the deck is now playing. (Handled in playFromCuePreview().)
 
     QMutexLocker lock(&m_mutex);
-    bool playing = (m_pPlayButton->get() == 1.0);
-    double cuePoint = m_pCuePoint->get();
+    bool playing = m_pPlayButton->toBool();
 
     if (v) {
-        if (playing) {
-            m_pPlayButton->set(0.0);
+        if (m_iCurrentlyPreviewingHotcues) {
+            // we are already previewing by hotcues
+            // just jump to cue point and continue previewing
+            m_bPreviewing = true;
+            lock.unlock();
+            seekAbs(m_pCuePoint->get());
+        } else if (playing || atEndPosition()) {
+            // Jump to cue when playing or when at end position
 
             // Just in case.
             m_bPreviewing = false;
+            m_pPlayButton->set(0.0);
 
             // Need to unlock before emitting any signals to prevent deadlock.
             lock.unlock();
 
-            emit(seekAbs(cuePoint));
+            seekAbs(m_pCuePoint->get());
+        } else if (isTrackAtCue()) {
+            // pause at cue point
+            m_bPreviewing = true;
+            m_pPlayButton->set(1.0);
         } else {
-            if (fabs(getCurrentSample() - m_pCuePoint->get()) < 1.0f) {
-                m_pPlayButton->set(1.0);
-                m_bPreviewing = true;
-            } else {
-                cueSet(v);
-                // Just in case.
-                m_bPreviewing = false;
+            // Pause not at cue point and not at end position
+            cueSet(v);
+            // Just in case.
+            m_bPreviewing = false;
 
-                // If quantize is enabled, jump to the cue point since it's not
-                // necessarily where we currently are
-                if (m_pQuantizeEnabled->get() > 0.0) {
-                    lock.unlock();  // prevent deadlock.
-                    emit(seekAbs(m_pCuePoint->get()));
-                }
+            // If quantize is enabled, jump to the cue point since it's not
+            // necessarily where we currently are
+            if (m_pQuantizeEnabled->get() > 0.0) {
+                lock.unlock();  // prevent deadlock.
+                // Enginebuffer will quantize more exactly than we can.
+                seekAbs(m_pCuePoint->get());
             }
         }
     } else if (m_bPreviewing) {
-        m_pPlayButton->set(0.0);
         m_bPreviewing = false;
+        if (!m_iCurrentlyPreviewingHotcues) {
+            m_pPlayButton->set(0.0);
 
-        // Need to unlock before emitting any signals to prevent deadlock.
-        lock.unlock();
+            // Need to unlock before emitting any signals to prevent deadlock.
+            lock.unlock();
 
-        emit(seekAbs(cuePoint));
+            seekAbs(m_pCuePoint->get());
+        }
     }
-    else {
-        // Re-trigger the play button value so controllers get the correct one
-        // after playFromCuePreview() changes it.
-        m_pPlayButton->set(m_pPlayButton->get());
+    // indicator may flash because the delayed adoption of seekAbs
+    // Correct the Indicator set via play
+    if (m_pLoadedTrack && !playing) {
+        m_pCueIndicator->setBlinkValue(ControlIndicator::ON);
+    } else {
+        m_pCueIndicator->setBlinkValue(ControlIndicator::OFF);
     }
 }
 
-void CueControl::playFromCuePreview(double v) {
-    Q_UNUSED(v);
+void CueControl::cueDenon(double v) {
+    // This is how Denon DN-S 3700 cue buttons work:
+    // If pressed go to cue and stop.
+    // If pressed while stopped and at cue, play while pressed.
+    // Cue Point is moved by play from pause
 
     QMutexLocker lock(&m_mutex);
+    bool playing = (m_pPlayButton->toBool());
 
-    if (m_bPreviewing) {
-    // we're previewing? Then stop previewing and go into normal play mode.
-        m_pPlayButton->set(1.0);
+    if (v) {
+        if (m_iCurrentlyPreviewingHotcues) {
+            // we are already previewing by hotcues
+            // just jump to cue point and continue previewing
+            m_bPreviewing = true;
+            lock.unlock();
+            seekAbs(m_pCuePoint->get());
+        } else if (!playing && isTrackAtCue()) {
+            // pause at cue point
+            m_bPreviewing = true;
+            m_pPlayButton->set(1.0);
+        } else {
+            // Just in case.
+            m_bPreviewing = false;
+            m_pPlayButton->set(0.0);
+
+            // Need to unlock before emitting any signals to prevent deadlock.
+            lock.unlock();
+
+            seekAbs(m_pCuePoint->get());
+        }
+    } else if (m_bPreviewing) {
         m_bPreviewing = false;
-    }
+        if (!m_iCurrentlyPreviewingHotcues) {
+            m_pPlayButton->set(0.0);
 
-    if (m_bPreviewingHotcue) {
-        m_pPlayButton->set(1.0);
-        m_bHotcueCancel = true;
-    }
+            // Need to unlock before emitting any signals to prevent deadlock.
+            lock.unlock();
 
-    lock.unlock();
+            seekAbs(m_pCuePoint->get());
+        }
+    }
 }
 
 void CueControl::cueDefault(double v) {
+    double cueMode = m_pCueMode->get();
     // Decide which cue implementation to call based on the user preference
-    if (m_pCueMode->get() == 0.0f) {
-        cueCDJ(v);
+    if (cueMode == CUE_MODE_DENON || cueMode == CUE_MODE_NUMARK) {
+        cueDenon(v);
     } else {
-        cueSimple(v);
+        // The modes CUE_MODE_PIONEER and CUE_MODE_MIXXX are similar
+        // are handled inside cueCDJ(v)
+        // default to Pioneer mode
+        cueCDJ(v);
     }
 }
 
+void CueControl::pause(double v) {
+    QMutexLocker lock(&m_mutex);
+    //qDebug() << "CueControl::pause()" << v;
+    if (v != 0.0) {
+        m_pPlayButton->set(0.0);
+    }
+}
+
+void CueControl::playStutter(double v) {
+    QMutexLocker lock(&m_mutex);
+    //qDebug() << "playStutter" << v;
+    if (v != 0.0) {
+        if (m_pPlayButton->toBool()) {
+            cueGoto(1.0);
+        } else {
+            m_pPlayButton->set(1.0);
+        }
+    }
+}
+
+bool CueControl::updateIndicatorsAndModifyPlay(bool newPlay, bool playPossible) {
+    //qDebug() << "updateIndicatorsAndModifyPlay" << newPlay << playPossible
+    //        << m_iCurrentlyPreviewingHotcues << m_bPreviewing;
+    QMutexLocker lock(&m_mutex);
+    double cueMode = m_pCueMode->get();
+    if ((cueMode == CUE_MODE_DENON || cueMode == CUE_MODE_NUMARK) &&
+        newPlay && playPossible &&
+        !m_pPlayButton->toBool() &&
+        !m_bypassCueSetByPlay) {
+        // in Denon mode each play from pause moves the cue point
+        // if not previewing
+        cueSet(1.0);
+    }
+    m_bypassCueSetByPlay = false;
+
+    // when previewing, "play" was set by cue button, a following toggle request
+    // (play = 0.0) is used for latching play.
+    bool previewing = false;
+    if (m_bPreviewing || m_iCurrentlyPreviewingHotcues) {
+        if (!newPlay) {
+            // play latch request: stop previewing and go into normal play mode.
+            m_bPreviewing = false;
+            m_iCurrentlyPreviewingHotcues = 0;
+            newPlay = true;
+        } else {
+            previewing = true;
+        }
+    }
+
+    if (!playPossible) {
+        // play not possible
+        newPlay = false;
+        m_pPlayIndicator->setBlinkValue(ControlIndicator::OFF);
+        m_pStopButton->set(0.0);
+    } else if (newPlay && !previewing) {
+        // Play: Indicates a latched Play
+        m_pPlayIndicator->setBlinkValue(ControlIndicator::ON);
+        m_pStopButton->set(0.0);
+    } else {
+        // Pause:
+        m_pStopButton->set(1.0);
+        if (cueMode == CUE_MODE_DENON) {
+            if (isTrackAtCue() || previewing) {
+                m_pPlayIndicator->setBlinkValue(ControlIndicator::OFF);
+            } else {
+                // Flashing indicates that a following play would move cue point
+                m_pPlayIndicator->setBlinkValue(ControlIndicator::RATIO1TO1_500MS);
+            }
+        } else if (cueMode == CUE_MODE_MIXXX || cueMode == CUE_MODE_MIXXX_NO_BLINK ||
+                   cueMode == CUE_MODE_NUMARK) {
+            m_pPlayIndicator->setBlinkValue(ControlIndicator::OFF);
+        } else {
+            // Flashing indicates that play is possible in Pioneer mode
+            m_pPlayIndicator->setBlinkValue(ControlIndicator::RATIO1TO1_500MS);
+        }
+    }
+
+    if (cueMode != CUE_MODE_DENON && cueMode != CUE_MODE_NUMARK) {
+        if (m_pCuePoint->get() != -1) {
+            if (newPlay == 0.0 && !isTrackAtCue() &&
+                    !atEndPosition()) {
+                if (cueMode == CUE_MODE_MIXXX) {
+                    // in Mixxx mode Cue Button is flashing slow if CUE will move Cue point
+                    m_pCueIndicator->setBlinkValue(ControlIndicator::RATIO1TO1_500MS);
+                } else if (cueMode == CUE_MODE_MIXXX_NO_BLINK) {
+                    m_pCueIndicator->setBlinkValue(ControlIndicator::OFF);
+                } else {
+                    // in Pioneer mode Cue Button is flashing fast if CUE will move Cue point
+                    m_pCueIndicator->setBlinkValue(ControlIndicator::RATIO1TO1_250MS);
+                }
+            } else {
+                m_pCueIndicator->setBlinkValue(ControlIndicator::OFF);
+            }
+        } else {
+            m_pCueIndicator->setBlinkValue(ControlIndicator::OFF);
+        }
+    }
+    m_pPlayStutter->set(newPlay ? 1.0 : 0.0);
+
+    return newPlay;
+}
+
+void CueControl::updateIndicators() {
+    // No need for mutex lock because we are only touching COs.
+    double cueMode = m_pCueMode->get();
+
+    if (cueMode == CUE_MODE_DENON || cueMode == CUE_MODE_NUMARK) {
+        // Cue button is only lit at cue point
+        bool playing = m_pPlayButton->toBool();
+        if (isTrackAtCue()) {
+            // at cue point
+            if (!playing) {
+                m_pCueIndicator->setBlinkValue(ControlIndicator::ON);
+                m_pPlayIndicator->setBlinkValue(ControlIndicator::OFF);
+            }
+        } else {
+            m_pCueIndicator->setBlinkValue(ControlIndicator::OFF);
+            if (!playing) {
+                if (!atEndPosition() && cueMode != CUE_MODE_NUMARK) {
+                    // Play will move cue point
+                    m_pPlayIndicator->setBlinkValue(ControlIndicator::RATIO1TO1_500MS);
+                } else {
+                    // At track end
+                    m_pPlayIndicator->setBlinkValue(ControlIndicator::OFF);
+                }
+            }
+        }
+    } else {
+        // Here we have CUE_MODE_PIONEER or CUE_MODE_MIXXX
+        // default to Pioneer mode
+        if (!m_bPreviewing) {
+            bool playing = m_pPlayButton->toBool();
+            if (!playing) {
+                if (!isTrackAtCue()) {
+                    if (!atEndPosition()) {
+                        if (cueMode == CUE_MODE_MIXXX) {
+                            // in Mixxx mode Cue Button is flashing slow if CUE will move Cue point
+                            m_pCueIndicator->setBlinkValue(ControlIndicator::RATIO1TO1_500MS);
+                        } else if (cueMode == CUE_MODE_MIXXX_NO_BLINK) {
+                            m_pCueIndicator->setBlinkValue(ControlIndicator::OFF);
+                        } else {
+                            // in Pioneer mode Cue Button is flashing fast if CUE will move Cue point
+                            m_pCueIndicator->setBlinkValue(ControlIndicator::RATIO1TO1_250MS);
+                        }
+                    } else {
+                        // At track end
+                        m_pCueIndicator->setBlinkValue(ControlIndicator::OFF);
+                    }
+                } else if (m_pCuePoint->get() != -1) {
+                    // Next Press is preview
+                    m_pCueIndicator->setBlinkValue(ControlIndicator::ON);
+                }
+            }
+        }
+    }
+}
+
+bool CueControl::isTrackAtCue() {
+    return (fabs(getCurrentSample() - m_pCuePoint->get()) < 1.0f);
+}
+
 ConfigKey HotcueControl::keyForControl(int hotcue, QString name) {
     ConfigKey key;
-    key.group = m_pGroup;
+    key.group = m_group;
     // Add one to hotcue so that we dont have a hotcue_0
     key.item = QString("hotcue_%1_%2").arg(QString::number(hotcue+1), name);
     return key;
 }
 
-HotcueControl::HotcueControl(const char* pGroup, int i)
-        : m_pGroup(pGroup),
+HotcueControl::HotcueControl(QString group, int i)
+        : m_group(group),
           m_iHotcueNumber(i),
           m_pCue(NULL),
           m_bPreviewing(false),
