@@ -9,6 +9,7 @@
 
 #include "engine/enginecontrol.h"
 #include "configobject.h"
+#include "controlobjectslave.h"
 #include "trackinfoobject.h"
 
 #define NUM_HOT_CUES 37
@@ -16,11 +17,12 @@
 class ControlObject;
 class ControlPushButton;
 class Cue;
+class ControlIndicator;
 
 class HotcueControl : public QObject {
     Q_OBJECT
   public:
-    HotcueControl(const char* pGroup, int hotcueNumber);
+    HotcueControl(QString group, int hotcueNumber);
     virtual ~HotcueControl();
 
     inline int getHotcueNumber() { return m_iHotcueNumber; }
@@ -59,7 +61,7 @@ class HotcueControl : public QObject {
   private:
     ConfigKey keyForControl(int hotcue, QString name);
 
-    const char* m_pGroup;
+    QString m_group;
     int m_iHotcueNumber;
     Cue* m_pCue;
 
@@ -82,11 +84,15 @@ class HotcueControl : public QObject {
 class CueControl : public EngineControl {
     Q_OBJECT
   public:
-    CueControl(const char * _group,
-               ConfigObject<ConfigValue> * _config);
+    CueControl(QString group,
+               ConfigObject<ConfigValue>* _config);
     virtual ~CueControl();
 
-    virtual void hintReader(QList<Hint>& hintList);
+    virtual void hintReader(HintVector* pHintList);
+    bool updateIndicatorsAndModifyPlay(bool newPlay, bool playPossible);
+    void updateIndicators();
+    bool isTrackAtCue();
+    bool getPlayFlashingAtPause();
 
   public slots:
     void trackLoaded(TrackPointer pTrack);
@@ -108,11 +114,12 @@ class CueControl : public EngineControl {
     void cueGoto(double v);
     void cueGotoAndPlay(double v);
     void cueGotoAndStop(double v);
-    void cueSimple(double v);
     void cuePreview(double v);
     void cueCDJ(double v);
+    void cueDenon(double v);
     void cueDefault(double v);
-    void playFromCuePreview(double v);
+    void pause(double v);
+    void playStutter(double v);
 
   private:
     // These methods are not thread safe, only call them when the lock is held.
@@ -121,14 +128,14 @@ class CueControl : public EngineControl {
     void detachCue(int hotcueNumber);
     void saveCuePoint(double cuePoint);
 
-    bool m_bHotcueCancel;
     bool m_bPreviewing;
-    bool m_bPreviewingHotcue;
     ControlObject* m_pPlayButton;
+    ControlObject* m_pStopButton;
     int m_iCurrentlyPreviewingHotcues;
     ControlObject* m_pQuantizeEnabled;
     ControlObject* m_pNextBeat;
     ControlObject* m_pClosestBeat;
+    bool m_bypassCueSetByPlay;
 
     const int m_iNumHotCues;
     QList<HotcueControl*> m_hotcueControl;
@@ -137,13 +144,17 @@ class CueControl : public EngineControl {
     ControlObject* m_pCuePoint;
     ControlObject* m_pCueMode;
     ControlPushButton* m_pCueSet;
-    ControlPushButton* m_pCueSimple;
     ControlPushButton* m_pCueCDJ;
     ControlPushButton* m_pCueDefault;
+    ControlPushButton* m_pPlayStutter;
+    ControlIndicator* m_pCueIndicator;
+    ControlIndicator* m_pPlayIndicator;
     ControlPushButton* m_pCueGoto;
     ControlPushButton* m_pCueGotoAndPlay;
     ControlPushButton* m_pCueGotoAndStop;
     ControlPushButton* m_pCuePreview;
+    ControlObjectSlave* m_pVinylControlEnabled;
+    ControlObjectSlave* m_pVinylControlMode;
 
     TrackPointer m_pLoadedTrack;
 
