@@ -18,35 +18,25 @@
 #ifndef ENGINEBUFFERSCALEDUMMY_H
 #define ENGINEBUFFERSCALEDUMMY_H
 
-#include "defs.h"
 #include <QObject>
+
+#include "util/types.h"
 #include "engine/enginebufferscale.h"
 
 class ReadAheadManager;
 
-class EngineBufferScaleDummy : public EngineBufferScale
-{
-public:
+class EngineBufferScaleDummy : public EngineBufferScale {
+  public:
     EngineBufferScaleDummy(ReadAheadManager* pReadAheadManager);
-    ~EngineBufferScaleDummy();
+    virtual ~EngineBufferScaleDummy();
 
-    /** Set base tempo, ie. normal playback speed. */
-    void setBaseRate(double dBaseRate);
-    /** Set tempo */
-    double setTempo(double dTempo);
-    /** Get new playpos after call to scale() */
-    double getNewPlaypos();
     /** Called from EngineBuffer when seeking, to ensure the buffers are flushed */
     void clear();
     /** Scale buffer */
-    CSAMPLE* scale(double playpos,
-                   unsigned long buf_size,
-                   CSAMPLE* pBase,
-                   unsigned long iBaseLength);
+    CSAMPLE* getScaled(unsigned long buf_size);
 
-private:
+  private:
     ReadAheadManager* m_pReadAheadManager;
-
 };
 
 #endif
