@@ -4,15 +4,19 @@
 #ifndef ENGINEWORKER_H
 #define ENGINEWORKER_H
 
+#include <QAtomicInt>
 #include <QObject>
-#include <QRunnable>
+#include <QSemaphore>
+#include <QThread>
 
 // EngineWorker is an interface for running background processing work when the
 // audio callback is not active. While the audio callback is active, an
 // EngineWorker can emit its workReady signal, and an EngineWorkerManager will
 // schedule it for running after the audio callback has completed.
 
-class EngineWorker : public QObject, public QRunnable {
+class EngineWorkerScheduler;
+
+class EngineWorker : public QThread {
     Q_OBJECT
   public:
     EngineWorker();
@@ -20,11 +24,17 @@ class EngineWorker : public QObject, public QRunnable {
 
     virtual void run();
 
-  signals:
-    void workReady();
-    void workStarting();
-    void workDone();
+    void setScheduler(EngineWorkerScheduler* pScheduler);
+    bool workReady();
+    void wake() {
+        m_semaRun.release();
+    }
+
+  protected:
+    QSemaphore m_semaRun;
+
+  private:
+    EngineWorkerScheduler* m_pScheduler;
 };
 
-
 #endif /* ENGINEWORKER_H */
