@@ -3,82 +3,102 @@
 // Author: asantoni, rryan
 
 #include <QtDebug>
-#include <QObject>
 
 #include "controlobject.h"
 #include "configobject.h"
 #include "controlpushbutton.h"
 #include "cachingreader.h"
 #include "engine/loopingcontrol.h"
+#include "engine/bpmcontrol.h"
 #include "engine/enginecontrol.h"
-#include "mathstuff.h"
+#include "util/math.h"
 
 #include "trackinfoobject.h"
 #include "track/beats.h"
 
-double LoopingControl::s_dBeatSizes[] = { 0.03125, 0.0625, 0.125, 0.25, 0.5, 1, 2, 4, 8, 16, 32, 64, };
+double LoopingControl::s_dBeatSizes[] = { 0.03125, 0.0625, 0.125, 0.25, 0.5,
+                                          1, 2, 4, 8, 16, 32, 64 };
 
-LoopingControl::LoopingControl(const char * _group,
-                               ConfigObject<ConfigValue> * _config)
-        : EngineControl(_group, _config) {
+// Used to generate the beatloop_%SIZE, beatjump_%SIZE, and loop_move_%SIZE CO
+// ConfigKeys.
+ConfigKey keyForControl(QString group, QString ctrlName, double num) {
+    ConfigKey key;
+    key.group = group;
+    key.item = ctrlName.arg(num);
+    return key;
+}
+
+// static
+QList<double> LoopingControl::getBeatSizes() {
+    QList<double> result;
+    for (unsigned int i = 0; i < (sizeof(s_dBeatSizes) / sizeof(s_dBeatSizes[0])); ++i) {
+        result.append(s_dBeatSizes[i]);
+    }
+    return result;
+}
+
+LoopingControl::LoopingControl(QString group,
+                               ConfigObject<ConfigValue>* _config)
+        : EngineControl(group, _config) {
     m_bLoopingEnabled = false;
+    m_bLoopRollActive = false;
     m_iLoopStartSample = kNoTrigger;
     m_iLoopEndSample = kNoTrigger;
     m_iCurrentSample = 0.;
     m_pActiveBeatLoop = NULL;
 
     //Create loop-in, loop-out, loop-exit, and reloop/exit ControlObjects
-    m_pLoopInButton = new ControlPushButton(ConfigKey(_group, "loop_in"));
+    m_pLoopInButton = new ControlPushButton(ConfigKey(group, "loop_in"));
     connect(m_pLoopInButton, SIGNAL(valueChanged(double)),
             this, SLOT(slotLoopIn(double)),
             Qt::DirectConnection);
     m_pLoopInButton->set(0);
 
-    m_pLoopOutButton = new ControlPushButton(ConfigKey(_group, "loop_out"));
+    m_pLoopOutButton = new ControlPushButton(ConfigKey(group, "loop_out"));
     connect(m_pLoopOutButton, SIGNAL(valueChanged(double)),
             this, SLOT(slotLoopOut(double)),
             Qt::DirectConnection);
     m_pLoopOutButton->set(0);
 
-    m_pLoopExitButton = new ControlPushButton(ConfigKey(_group, "loop_exit"));
+    m_pLoopExitButton = new ControlPushButton(ConfigKey(group, "loop_exit"));
     connect(m_pLoopExitButton, SIGNAL(valueChanged(double)),
             this, SLOT(slotLoopExit(double)),
             Qt::DirectConnection);
     m_pLoopExitButton->set(0);
 
-    m_pReloopExitButton = new ControlPushButton(ConfigKey(_group, "reloop_exit"));
+    m_pReloopExitButton = new ControlPushButton(ConfigKey(group, "reloop_exit"));
     connect(m_pReloopExitButton, SIGNAL(valueChanged(double)),
             this, SLOT(slotReloopExit(double)),
             Qt::DirectConnection);
     m_pReloopExitButton->set(0);
 
-    m_pCOLoopEnabled = new ControlObject(ConfigKey(_group, "loop_enabled"));
-    m_pCOLoopEnabled->set(0.0f);
+    m_pCOLoopEnabled = new ControlObject(ConfigKey(group, "loop_enabled"));
+    m_pCOLoopEnabled->set(0.0);
 
     m_pCOLoopStartPosition =
-            new ControlObject(ConfigKey(_group, "loop_start_position"));
+            new ControlObject(ConfigKey(group, "loop_start_position"));
     m_pCOLoopStartPosition->set(kNoTrigger);
     connect(m_pCOLoopStartPosition, SIGNAL(valueChanged(double)),
             this, SLOT(slotLoopStartPos(double)),
             Qt::DirectConnection);
 
     m_pCOLoopEndPosition =
-            new ControlObject(ConfigKey(_group, "loop_end_position"));
+            new ControlObject(ConfigKey(group, "loop_end_position"));
     m_pCOLoopEndPosition->set(kNoTrigger);
     connect(m_pCOLoopEndPosition, SIGNAL(valueChanged(double)),
             this, SLOT(slotLoopEndPos(double)),
             Qt::DirectConnection);
 
-    m_pQuantizeEnabled = ControlObject::getControl(ConfigKey(_group, "quantize"));
-    m_pNextBeat = ControlObject::getControl(ConfigKey(_group, "beat_next"));
-    m_pClosestBeat = ControlObject::getControl(ConfigKey(_group, "beat_closest"));
-    m_pTrackSamples = ControlObject::getControl(ConfigKey(_group,"track_samples"));
-    m_pSlipEnabled = ControlObject::getControl(ConfigKey(_group,"slip_enabled"));
+    m_pQuantizeEnabled = ControlObject::getControl(ConfigKey(group, "quantize"));
+    m_pNextBeat = ControlObject::getControl(ConfigKey(group, "beat_next"));
+    m_pClosestBeat = ControlObject::getControl(ConfigKey(group, "beat_closest"));
+    m_pTrackSamples = ControlObject::getControl(ConfigKey(group, "track_samples"));
+    m_pSlipEnabled = ControlObject::getControl(ConfigKey(group, "slip_enabled"));
 
     // Connect beatloop, which can flexibly handle different values.
     // Using this CO directly is meant to be used internally and by scripts,
     // or anything else that can pass in arbitrary values.
-    m_pCOBeatLoop = new ControlPushButton(ConfigKey(_group, "beatloop"));
+    m_pCOBeatLoop = new ControlObject(ConfigKey(group, "beatloop"), false);
     connect(m_pCOBeatLoop, SIGNAL(valueChanged(double)), this,
             SLOT(slotBeatLoop(double)), Qt::DirectConnection);
 
@@ -86,7 +106,7 @@ LoopingControl::LoopingControl(const char * _group,
     // Here we create corresponding beatloop_(SIZE) CO's which all call the same
     // BeatControl, but with a set value.
     for (unsigned int i = 0; i < (sizeof(s_dBeatSizes) / sizeof(s_dBeatSizes[0])); ++i) {
-        BeatLoopingControl* pBeatLoop = new BeatLoopingControl(_group, s_dBeatSizes[i]);
+        BeatLoopingControl* pBeatLoop = new BeatLoopingControl(group, s_dBeatSizes[i]);
         connect(pBeatLoop, SIGNAL(activateBeatLoop(BeatLoopingControl*)),
                 this, SLOT(slotBeatLoopActivate(BeatLoopingControl*)),
                 Qt::DirectConnection);
@@ -102,13 +122,41 @@ LoopingControl::LoopingControl(const char * _group,
         m_beatLoops.append(pBeatLoop);
     }
 
-    m_pCOLoopScale = new ControlObject(ConfigKey(_group, "loop_scale"));
+    m_pCOBeatJump = new ControlObject(ConfigKey(group, "beatjump"), false);
+    connect(m_pCOBeatJump, SIGNAL(valueChanged(double)),
+            this, SLOT(slotBeatJump(double)), Qt::DirectConnection);
+
+    // Create beatjump_(SIZE) CO's which all call beatjump, but with a set
+    // value.
+    for (unsigned int i = 0; i < (sizeof(s_dBeatSizes) / sizeof(s_dBeatSizes[0])); ++i) {
+        BeatJumpControl* pBeatJump = new BeatJumpControl(group, s_dBeatSizes[i]);
+        connect(pBeatJump, SIGNAL(beatJump(double)),
+                this, SLOT(slotBeatJump(double)),
+                Qt::DirectConnection);
+        m_beatJumps.append(pBeatJump);
+    }
+
+    m_pCOLoopMove = new ControlObject(ConfigKey(group, "loop_move"), false);
+    connect(m_pCOLoopMove, SIGNAL(valueChanged(double)),
+            this, SLOT(slotLoopMove(double)), Qt::DirectConnection);
+
+    // Create loop_move_(SIZE) CO's which all call loop_move, but with a set
+    // value.
+    for (unsigned int i = 0; i < (sizeof(s_dBeatSizes) / sizeof(s_dBeatSizes[0])); ++i) {
+        LoopMoveControl* pLoopMove = new LoopMoveControl(group, s_dBeatSizes[i]);
+        connect(pLoopMove, SIGNAL(loopMove(double)),
+                this, SLOT(slotLoopMove(double)),
+                Qt::DirectConnection);
+        m_loopMoves.append(pLoopMove);
+    }
+
+    m_pCOLoopScale = new ControlObject(ConfigKey(group, "loop_scale"), false);
     connect(m_pCOLoopScale, SIGNAL(valueChanged(double)),
             this, SLOT(slotLoopScale(double)));
-    m_pLoopHalveButton = new ControlPushButton(ConfigKey(_group, "loop_halve"));
+    m_pLoopHalveButton = new ControlPushButton(ConfigKey(group, "loop_halve"));
     connect(m_pLoopHalveButton, SIGNAL(valueChanged(double)),
             this, SLOT(slotLoopHalve(double)));
-    m_pLoopDoubleButton = new ControlPushButton(ConfigKey(_group, "loop_double"));
+    m_pLoopDoubleButton = new ControlPushButton(ConfigKey(group, "loop_double"));
     connect(m_pLoopDoubleButton, SIGNAL(valueChanged(double)),
             this, SLOT(slotLoopDouble(double)));
 }
@@ -124,16 +172,32 @@ LoopingControl::~LoopingControl() {
     delete m_pCOLoopScale;
     delete m_pLoopHalveButton;
     delete m_pLoopDoubleButton;
-    delete m_pCOBeatLoop;
 
-    while (m_beatLoops.size() > 0) {
+    delete m_pCOBeatLoop;
+    while (!m_beatLoops.isEmpty()) {
         BeatLoopingControl* pBeatLoop = m_beatLoops.takeLast();
         delete pBeatLoop;
     }
+
+    delete m_pCOBeatJump;
+    while (!m_beatJumps.isEmpty()) {
+        BeatJumpControl* pBeatJump = m_beatJumps.takeLast();
+        delete pBeatJump;
+    }
+
+    delete m_pCOLoopMove;
+    while (!m_loopMoves.isEmpty()) {
+        LoopMoveControl* pLoopMove = m_loopMoves.takeLast();
+        delete pLoopMove;
+    }
 }
 
 void LoopingControl::slotLoopScale(double scale) {
+    if (m_iLoopStartSample == kNoTrigger || m_iLoopEndSample == kNoTrigger) {
+        return;
+    }
     int loop_length = m_iLoopEndSample - m_iLoopStartSample;
+    int old_loop_end = m_iLoopEndSample;
     int samples = m_pTrackSamples->get();
     loop_length *= scale;
 
@@ -162,14 +226,25 @@ void LoopingControl::slotLoopScale(double scale) {
             m_iLoopEndSample += 2;
     }
     // Do not allow loops to go past the end of the song
-    else if (m_iLoopEndSample > samples)
+    else if (m_iLoopEndSample > samples) {
         m_iLoopEndSample = samples;
+    }
 
     // Update CO for loop end marker
     m_pCOLoopEndPosition->set(m_iLoopEndSample);
+
+    // Reseek if the loop shrank out from under the playposition.
+    if (m_bLoopingEnabled && scale < 1.0) {
+        seekInsideAdjustedLoop(
+                m_iLoopStartSample, old_loop_end,
+                m_iLoopStartSample, m_iLoopEndSample);
+    }
 }
 
 void LoopingControl::slotLoopHalve(double v) {
+    if (m_iLoopStartSample == kNoTrigger || m_iLoopEndSample == kNoTrigger) {
+        return;
+    }
     if (v > 0.0) {
         // If a beatloop is active then halve should deactive the current
         // beatloop and activate the previous one.
@@ -177,7 +252,15 @@ void LoopingControl::slotLoopHalve(double v) {
             int active_index = m_beatLoops.indexOf(m_pActiveBeatLoop);
             if (active_index - 1 >= 0) {
                 if (m_bLoopingEnabled) {
+                    // If the current position is outside the range of the new loop,
+                    // take the current position and subtract the length of the new loop until
+                    // it fits.
+                    int old_loop_in = m_iLoopStartSample;
+                    int old_loop_out = m_iLoopEndSample;
                     slotBeatLoopActivate(m_beatLoops[active_index - 1]);
+                    seekInsideAdjustedLoop(
+                            old_loop_in, old_loop_out,
+                            m_iLoopStartSample, m_iLoopEndSample);
                 } else {
                     // Calling scale clears the active beatloop.
                     slotLoopScale(0.5);
@@ -191,7 +274,10 @@ void LoopingControl::slotLoopHalve(double v) {
 }
 
 void LoopingControl::slotLoopDouble(double v) {
-    if (v > 0.0f) {
+    if (m_iLoopStartSample == kNoTrigger || m_iLoopEndSample == kNoTrigger) {
+        return;
+    }
+    if (v > 0.0) {
         // If a beatloop is active then double should deactive the current
         // beatloop and activate the next one.
         if (m_pActiveBeatLoop != NULL) {
@@ -224,11 +310,10 @@ double LoopingControl::process(const double dRate,
     bool reverse = dRate < 0;
 
     double retval = kNoTrigger;
-    if(m_bLoopingEnabled &&
-       m_iLoopStartSample != kNoTrigger &&
-       m_iLoopEndSample != kNoTrigger) {
+    if (m_bLoopingEnabled && m_iLoopStartSample != kNoTrigger &&
+            m_iLoopEndSample != kNoTrigger) {
         bool outsideLoop = currentSample >= m_iLoopEndSample ||
-                currentSample <= m_iLoopStartSample;
+                           currentSample <= m_iLoopStartSample;
         if (outsideLoop) {
             retval = reverse ? m_iLoopEndSample : m_iLoopStartSample;
         }
@@ -246,7 +331,7 @@ double LoopingControl::nextTrigger(const double dRate,
     Q_UNUSED(iBufferSize);
     bool bReverse = dRate < 0;
 
-    if(m_bLoopingEnabled) {
+    if (m_bLoopingEnabled) {
         if (bReverse)
             return m_iLoopStartSample;
         else
@@ -264,7 +349,7 @@ double LoopingControl::getTrigger(const double dRate,
     Q_UNUSED(iBufferSize);
     bool bReverse = dRate < 0;
 
-    if(m_bLoopingEnabled) {
+    if (m_bLoopingEnabled) {
         if (bReverse)
             return m_iLoopEndSample;
         else
@@ -273,7 +358,7 @@ double LoopingControl::getTrigger(const double dRate,
     return kNoTrigger;
 }
 
-void LoopingControl::hintReader(QList<Hint>& hintList) {
+void LoopingControl::hintReader(HintVector* pHintList) {
     Hint loop_hint;
     // If the loop is enabled, then this is high priority because we will loop
     // sometime potentially very soon! The current audio itself is priority 1,
@@ -287,20 +372,20 @@ void LoopingControl::hintReader(QList<Hint>& hintList) {
             loop_hint.priority = 2;
             loop_hint.sample = m_iLoopStartSample;
             loop_hint.length = 0; // Let it issue the default length
-            hintList.append(loop_hint);
+            pHintList->append(loop_hint);
         }
         if (m_iLoopEndSample >= 0) {
             loop_hint.priority = 10;
             loop_hint.sample = m_iLoopEndSample;
             loop_hint.length = -1; // Let it issue the default (backwards) length
-            hintList.append(loop_hint);
+            pHintList->append(loop_hint);
         }
     } else {
         if (m_iLoopStartSample >= 0) {
             loop_hint.priority = 10;
             loop_hint.sample = m_iLoopStartSample;
             loop_hint.length = 0; // Let it issue the default length
-            hintList.append(loop_hint);
+            pHintList->append(loop_hint);
         }
     }
 }
@@ -315,7 +400,7 @@ void LoopingControl::slotLoopIn(double val) {
         // set loop-in position
         int pos =
                 (m_pQuantizeEnabled->get() > 0.0 && m_pClosestBeat->get() != -1) ?
-                static_cast<int>(floorf(m_pClosestBeat->get())) : m_iCurrentSample;
+                static_cast<int>(floor(m_pClosestBeat->get())) : m_iCurrentSample;
 
         // If we're looping and the loop-in and out points are now so close
         //  that the loop would be inaudible (which can happen easily with
@@ -358,11 +443,11 @@ void LoopingControl::slotLoopOut(double val) {
     if (val) {
         int pos =
                 (m_pQuantizeEnabled->get() > 0.0 && m_pClosestBeat->get() != -1) ?
-                static_cast<int>(floorf(m_pClosestBeat->get())) : m_iCurrentSample;
+                static_cast<int>(floor(m_pClosestBeat->get())) : m_iCurrentSample;
 
         // If the user is trying to set a loop-out before the loop in or without
         // having a loop-in, then ignore it.
-        if (m_iLoopStartSample == -1 || pos < m_iLoopStartSample) {
+        if (m_iLoopStartSample == kNoTrigger || pos < m_iLoopStartSample) {
             return;
         }
 
@@ -408,7 +493,7 @@ void LoopingControl::slotLoopExit(double val) {
         // If we're looping, stop looping
         if (m_bLoopingEnabled) {
             setLoopingEnabled(false);
-        } 
+        }
     }
 }
 
@@ -419,6 +504,11 @@ void LoopingControl::slotReloopExit(double val) {
     if (val) {
         // If we're looping, stop looping
         if (m_bLoopingEnabled) {
+            // If loop roll was active, also disable slip.
+            if (m_bLoopRollActive) {
+                m_pSlipEnabled->set(0);
+                m_bLoopRollActive = false;
+            }
             setLoopingEnabled(false);
             //qDebug() << "reloop_exit looping off";
         } else {
@@ -442,9 +532,15 @@ void LoopingControl::slotLoopStartPos(double pos) {
         newpos--;
     }
 
+
+    if (m_iLoopStartSample == newpos) {
+        //nothing to do
+        return;
+    }
+
     clearActiveBeatLoop();
 
-    if (pos == -1.0f) {
+    if (pos == -1.0) {
         setLoopingEnabled(false);
     }
 
@@ -469,6 +565,11 @@ void LoopingControl::slotLoopEndPos(double pos) {
         newpos--;
     }
 
+    if (m_iLoopEndSample == newpos) {
+        //nothing to do
+        return;
+    }
+
     // Reject if the loop-in is not set, or if the new position is before the
     // start point (but not -1).
     if (m_iLoopStartSample == -1 ||
@@ -479,7 +580,7 @@ void LoopingControl::slotLoopEndPos(double pos) {
 
     clearActiveBeatLoop();
 
-    if (pos == -1.0f) {
+    if (pos == -1.0) {
         setLoopingEnabled(false);
     }
     m_iLoopEndSample = newpos;
@@ -544,12 +645,11 @@ void LoopingControl::slotBeatLoopActivate(BeatLoopingControl* pBeatLoopControl)
         return;
     }
 
-    // Maintain the current start point if there is an active beat loop and we
-    // are currently looping. slotBeatLoop will update m_pActiveBeatLoop if
-    // applicable
-    bool beatLoopAlreadyActive = m_pActiveBeatLoop != NULL;
-    slotBeatLoop(pBeatLoopControl->getSize(),
-                 beatLoopAlreadyActive && m_bLoopingEnabled);
+    // Maintain the current start point if there is an active loop currently
+    // looping. slotBeatLoop will update m_pActiveBeatLoop if applicable. Note,
+    // this used to only maintain the current start point if a beatloop was
+    // enabled. See Bug #1159243.
+    slotBeatLoop(pBeatLoopControl->getSize(), m_bLoopingEnabled);
 }
 
 void LoopingControl::slotBeatLoopActivateRoll(BeatLoopingControl* pBeatLoopControl) {
@@ -557,9 +657,10 @@ void LoopingControl::slotBeatLoopActivateRoll(BeatLoopingControl* pBeatLoopContr
          return;
      }
 
-    //Disregard existing loops
+    // Disregard existing loops.
     m_pSlipEnabled->set(1);
     slotBeatLoop(pBeatLoopControl->getSize(), false);
+    m_bLoopRollActive = true;
 }
 
 void LoopingControl::slotBeatLoopDeactivate(BeatLoopingControl* pBeatLoopControl) {
@@ -571,6 +672,7 @@ void LoopingControl::slotBeatLoopDeactivateRoll(BeatLoopingControl* pBeatLoopCon
     Q_UNUSED(pBeatLoopControl);
     setLoopingEnabled(false);
     m_pSlipEnabled->set(0);
+    m_bLoopRollActive = false;
 }
 
 void LoopingControl::clearActiveBeatLoop() {
@@ -583,26 +685,10 @@ void LoopingControl::clearActiveBeatLoop() {
 void LoopingControl::slotBeatLoop(double beats, bool keepStartPoint) {
     int samples = m_pTrackSamples->get();
     if (!m_pTrack || samples == 0) {
+        clearActiveBeatLoop();
         return;
     }
 
-    // O(n) search, but there are only ~10-ish beatloop controls so this is
-    // fine.
-    foreach (BeatLoopingControl* pBeatLoopControl, m_beatLoops) {
-        if (pBeatLoopControl->getSize() == beats) {
-            if (m_pActiveBeatLoop &&
-                m_pActiveBeatLoop != pBeatLoopControl) {
-                m_pActiveBeatLoop->deactivate();
-            }
-            m_pActiveBeatLoop = pBeatLoopControl;
-            pBeatLoopControl->activate();
-        }
-    }
-
-    // give loop_in and loop_out defaults so we can detect problems
-    int loop_in = -1;
-    int loop_out = -1;
-
     if (!m_pBeats) {
         clearActiveBeatLoop();
         return;
@@ -614,23 +700,70 @@ void LoopingControl::slotBeatLoop(double beats, bool keepStartPoint) {
         return;
     }
 
+    // O(n) search, but there are only ~10-ish beatloop controls so this is
+    // fine.
+    foreach (BeatLoopingControl* pBeatLoopControl, m_beatLoops) {
+        if (pBeatLoopControl->getSize() == beats) {
+            if (m_pActiveBeatLoop != pBeatLoopControl) {
+                if (m_pActiveBeatLoop) {
+                    m_pActiveBeatLoop->deactivate();
+                }
+                m_pActiveBeatLoop = pBeatLoopControl;
+            }
+            pBeatLoopControl->activate();
+            break;
+        }
+    }
+
+    // give loop_in and loop_out defaults so we can detect problems
+    int loop_in = -1;
+    int loop_out = -1;
+
     // For positive numbers we start from the current position/closest beat and
     // create the loop around X beats from there.
     if (beats > 0) {
         if (keepStartPoint) {
             loop_in = m_iLoopStartSample;
         } else {
-            // loop_in is set to the closest beat if quantize is on
-            double currentClosestBeat =
-                    floorf(m_pBeats->findClosestBeat(getCurrentSample()));
-            loop_in = (m_pQuantizeEnabled->get() > 0.0 && currentClosestBeat != -1) ?
-                    currentClosestBeat : floorf(getCurrentSample());
+            // loop_in is set to the previous beat if quantize is on.  The
+            // closest beat might be ahead of play position which would cause a seek.
+            // TODO: If in reverse, should probably choose nextBeat.
+            double cur_pos = getCurrentSample();
+            double prevBeat;
+            double nextBeat;
+            m_pBeats->findPrevNextBeats(cur_pos, &prevBeat, &nextBeat);
+
+            if (m_pQuantizeEnabled->get() > 0.0 && prevBeat != -1) {
+                if (beats >= 1.0) {
+                    loop_in = prevBeat;
+                } else {
+                    // In case of beat length less then 1 beat:
+                    // (| - beats, ^ - current track's position):
+                    //
+                    // ...|...................^........|...
+                    //
+                    // If we press 1/2 beatloop we want loop from 50% to 100%,
+                    // If I press 1/4 beatloop, we want loop from 50% to 75% etc
+                    double beat_len = nextBeat - prevBeat;
+                    double loops_per_beat = 1.0 / beats;
+                    double beat_pos = cur_pos - prevBeat;
+                    int beat_frac =
+                            static_cast<int>(floor((beat_pos / beat_len) *
+                                                   loops_per_beat));
+                    loop_in = prevBeat + beat_len / loops_per_beat * beat_frac;
+                }
+
+            } else {
+                loop_in = floor(cur_pos);
+            }
+
+
             if (!even(loop_in)) {
                 loop_in--;
             }
         }
 
-        int fullbeats = static_cast<int>(floorf(beats));
+        int fullbeats = static_cast<int>(beats);
         double fracbeats = beats - static_cast<double>(fullbeats);
 
         // Now we need to calculate the length of the beatloop. We do this by
@@ -641,6 +774,7 @@ void LoopingControl::slotBeatLoop(double beats, bool keepStartPoint) {
         if (fullbeats > 0) {
             // Add the length between this beat and the fullbeats'th beat to the
             // loop_out position;
+            // TODO: figure out how to convert this to a findPrevNext call.
             double this_beat = m_pBeats->findNthBeat(loop_in, 1);
             double nth_beat = m_pBeats->findNthBeat(loop_in, 1 + fullbeats);
             loop_out += (nth_beat - this_beat);
@@ -649,13 +783,14 @@ void LoopingControl::slotBeatLoop(double beats, bool keepStartPoint) {
         if (fracbeats > 0) {
             // Add the fraction of the beat following the current loop_out
             // position to loop out.
+            // TODO: figure out how to convert this to a findPrevNext call.
             double loop_out_beat = m_pBeats->findNthBeat(loop_out, 1);
             double loop_out_next_beat = m_pBeats->findNthBeat(loop_out, 2);
             loop_out += (loop_out_next_beat - loop_out_beat) * fracbeats;
         }
     }
 
-    if ((loop_in == -1) || ( loop_out == -1))
+    if ((loop_in == -1) || (loop_out == -1))
         return;
 
     if (!even(loop_in))
@@ -674,6 +809,11 @@ void LoopingControl::slotBeatLoop(double beats, bool keepStartPoint) {
         loop_out = samples;
     }
 
+    if (keepStartPoint) {
+        seekInsideAdjustedLoop(m_iLoopStartSample, m_iLoopEndSample,
+                               loop_in, loop_out);
+    }
+
     m_iLoopStartSample = loop_in;
     m_pCOLoopStartPosition->set(loop_in);
     m_iLoopEndSample = loop_out;
@@ -681,41 +821,193 @@ void LoopingControl::slotBeatLoop(double beats, bool keepStartPoint) {
     setLoopingEnabled(true);
 }
 
-BeatLoopingControl::BeatLoopingControl(const char* pGroup, double size)
+void LoopingControl::slotBeatJump(double beats) {
+    if (!m_pTrack || !m_pBeats) {
+        return;
+    }
+
+    double dPosition = getCurrentSample();
+    double dBeatLength;
+    if (BpmControl::getBeatContext(m_pBeats, dPosition,
+                                   NULL, NULL, &dBeatLength, NULL)) {
+        seekAbs(dPosition + beats * dBeatLength);
+    }
+}
+
+void LoopingControl::slotLoopMove(double beats) {
+    if (!m_pTrack || !m_pBeats) {
+        return;
+    }
+    if (m_iLoopStartSample == kNoTrigger || m_iLoopEndSample == kNoTrigger) {
+        return;
+    }
+
+    double dPosition = getCurrentSample();
+    double dBeatLength;
+    if (BpmControl::getBeatContext(m_pBeats, dPosition,
+                                   NULL, NULL, &dBeatLength, NULL)) {
+        int old_loop_in = m_iLoopStartSample;
+        int old_loop_out = m_iLoopEndSample;
+        int new_loop_in = old_loop_in + (beats * dBeatLength);
+        int new_loop_out = old_loop_out + (beats * dBeatLength);
+        if (!even(new_loop_in)) {
+            --new_loop_in;
+        }
+        if (!even(new_loop_out)) {
+            --new_loop_out;
+        }
+
+        m_iLoopStartSample = new_loop_in;
+        m_pCOLoopStartPosition->set(new_loop_in);
+        m_iLoopEndSample = new_loop_out;
+        m_pCOLoopEndPosition->set(new_loop_out);
+
+        // If we are looping make sure that the play head does not leave the
+        // loop as a result of our adjustment.
+        if (m_bLoopingEnabled) {
+            seekInsideAdjustedLoop(old_loop_in, old_loop_out,
+                                   new_loop_in, new_loop_out);
+        }
+    }
+}
+
+void LoopingControl::seekInsideAdjustedLoop(int old_loop_in, int old_loop_out,
+                                            int new_loop_in, int new_loop_out) {
+    if (m_iCurrentSample >= new_loop_in && m_iCurrentSample <= new_loop_out) {
+        return;
+    }
+
+    int new_loop_size = new_loop_out - new_loop_in;
+    if (!even(new_loop_size)) {
+        --new_loop_size;
+    }
+    if (new_loop_size > old_loop_out - old_loop_in) {
+        // Could this happen if the user grows a loop and then also shifts it?
+        qWarning() << "seekInsideAdjustedLoop called for loop that got larger -- ignoring";
+        return;
+    }
+
+    int adjusted_position = m_iCurrentSample;
+    while (adjusted_position > new_loop_out) {
+        adjusted_position -= new_loop_size;
+        if (adjusted_position < new_loop_in) {
+            // I'm not even sure this is possible.  The new loop would have to be bigger than the
+            // old loop, and the playhead was somehow outside the old loop.
+            qWarning() << "SHOULDN'T HAPPEN: seekInsideAdjustedLoop couldn't find a new position --"
+                       << " seeking to in point";
+            adjusted_position = new_loop_in;
+        }
+    }
+    while (adjusted_position < new_loop_in) {
+        adjusted_position += new_loop_size;
+        if (adjusted_position > new_loop_out) {
+            qWarning() << "SHOULDN'T HAPPEN: seekInsideAdjustedLoop couldn't find a new position --"
+                       << " seeking to in point";
+            adjusted_position = new_loop_in;
+        }
+    }
+    if (adjusted_position != m_iCurrentSample) {
+        m_iCurrentSample = adjusted_position;
+        seekAbs(static_cast<double>(adjusted_position));
+    }
+}
+
+BeatJumpControl::BeatJumpControl(QString group, double size)
+        : m_dBeatJumpSize(size) {
+    m_pJumpForward = new ControlPushButton(
+            keyForControl(group, "beatjump_%1_forward", size));
+    connect(m_pJumpForward, SIGNAL(valueChanged(double)),
+            this, SLOT(slotJumpForward(double)),
+            Qt::DirectConnection);
+    m_pJumpBackward = new ControlPushButton(
+            keyForControl(group, "beatjump_%1_backward", size));
+    connect(m_pJumpBackward, SIGNAL(valueChanged(double)),
+            this, SLOT(slotJumpBackward(double)),
+            Qt::DirectConnection);
+}
+
+BeatJumpControl::~BeatJumpControl() {
+    delete m_pJumpForward;
+    delete m_pJumpBackward;
+}
+
+void BeatJumpControl::slotJumpBackward(double v) {
+    if (v > 0) {
+        emit(beatJump(-m_dBeatJumpSize));
+    }
+}
+
+void BeatJumpControl::slotJumpForward(double v) {
+    if (v > 0) {
+        emit(beatJump(m_dBeatJumpSize));
+    }
+}
+
+LoopMoveControl::LoopMoveControl(QString group, double size)
+        : m_dLoopMoveSize(size) {
+    m_pMoveForward = new ControlPushButton(
+            keyForControl(group, "loop_move_%1_forward", size));
+    connect(m_pMoveForward, SIGNAL(valueChanged(double)),
+            this, SLOT(slotMoveForward(double)),
+            Qt::DirectConnection);
+    m_pMoveBackward = new ControlPushButton(
+            keyForControl(group, "loop_move_%1_backward", size));
+    connect(m_pMoveBackward, SIGNAL(valueChanged(double)),
+            this, SLOT(slotMoveBackward(double)),
+            Qt::DirectConnection);
+}
+
+LoopMoveControl::~LoopMoveControl() {
+    delete m_pMoveForward;
+    delete m_pMoveBackward;
+}
+
+void LoopMoveControl::slotMoveBackward(double v) {
+    if (v > 0) {
+        emit(loopMove(-m_dLoopMoveSize));
+    }
+}
+
+void LoopMoveControl::slotMoveForward(double v) {
+    if (v > 0) {
+        emit(loopMove(m_dLoopMoveSize));
+    }
+}
+
+BeatLoopingControl::BeatLoopingControl(QString group, double size)
         : m_dBeatLoopSize(size),
           m_bActive(false) {
     // This is the original beatloop control which is now deprecated. Its value
     // is the state of the beatloop control (1 for enabled, 0 for disabled).
     m_pLegacy = new ControlPushButton(
-        keyForControl(pGroup, "beatloop_%1", size));
-    m_pLegacy->setStates(2);
+            keyForControl(group, "beatloop_%1", size));
     m_pLegacy->setButtonMode(ControlPushButton::TOGGLE);
     connect(m_pLegacy, SIGNAL(valueChanged(double)),
             this, SLOT(slotLegacy(double)),
             Qt::DirectConnection);
     // A push-button which activates the beatloop.
     m_pActivate = new ControlPushButton(
-        keyForControl(pGroup, "beatloop_%1_activate", size));
+            keyForControl(group, "beatloop_%1_activate", size));
     connect(m_pActivate, SIGNAL(valueChanged(double)),
             this, SLOT(slotActivate(double)),
             Qt::DirectConnection);
     // A push-button which toggles the beatloop as active or inactive.
     m_pToggle = new ControlPushButton(
-        keyForControl(pGroup, "beatloop_%1_toggle", size));
+            keyForControl(group, "beatloop_%1_toggle", size));
     connect(m_pToggle, SIGNAL(valueChanged(double)),
             this, SLOT(slotToggle(double)),
             Qt::DirectConnection);
 
     // A push-button which activates rolling beatloops
     m_pActivateRoll = new ControlPushButton(
-        keyForControl(pGroup, "beatlooproll_%1_activate", size));
+            keyForControl(group, "beatlooproll_%1_activate", size));
     connect(m_pActivateRoll, SIGNAL(valueChanged(double)),
             this, SLOT(slotActivateRoll(double)),
             Qt::DirectConnection);
 
     // An indicator control which is 1 if the beatloop is enabled and 0 if not.
     m_pEnabled = new ControlObject(
-        keyForControl(pGroup, "beatloop_%1_enabled", size));
+            keyForControl(group, "beatloop_%1_enabled", size));
 }
 
 BeatLoopingControl::~BeatLoopingControl() {
@@ -727,15 +1019,19 @@ BeatLoopingControl::~BeatLoopingControl() {
 }
 
 void BeatLoopingControl::deactivate() {
-    m_bActive = false;
-    m_pEnabled->set(0);
-    m_pLegacy->set(0);
+    if (m_bActive) {
+        m_bActive = false;
+        m_pEnabled->set(0);
+        m_pLegacy->set(0);
+    }
 }
 
 void BeatLoopingControl::activate() {
-    m_bActive = true;
-    m_pEnabled->set(1);
-    m_pLegacy->set(1);
+    if (!m_bActive) {
+        m_bActive = true;
+        m_pEnabled->set(1);
+        m_pLegacy->set(1);
+    }
 }
 
 void BeatLoopingControl::slotLegacy(double v) {
@@ -775,11 +1071,3 @@ void BeatLoopingControl::slotToggle(double v) {
         emit(activateBeatLoop(this));
     }
 }
-
-ConfigKey BeatLoopingControl::keyForControl(const char* pGroup,
-                                            QString ctrlName, double num) {
-    ConfigKey key;
-    key.group = pGroup;
-    key.item = ctrlName.arg(num);
-    return key;
-}
