@@ -5,111 +5,130 @@
 #include "controlpushbutton.h"
 #include "controlpotmeter.h"
 #include "controlttrotary.h"
+#include "controlobjectslave.h"
 #include "rotary.h"
-#include "mathstuff.h"
+#include "util/math.h"
+#include "vinylcontrol/defs_vinylcontrol.h"
 
+#include "engine/bpmcontrol.h"
 #include "engine/enginecontrol.h"
 #include "engine/ratecontrol.h"
 #include "engine/positionscratchcontroller.h"
 
-#include <QDebug>
+#include <QtDebug>
 
-// Static default values for rate buttons (percents)
+// Static default values for rate buttons (percents). Note that these are not
+// actually used -- the preferences code sets the values that are stored in the
+// user's configuration. These are just fail safe defaults.
 double RateControl::m_dTemp = 4.00; //(eg. 4.00%)
-double RateControl::m_dTempSmall = 1.00;
+double RateControl::m_dTempSmall = 2.00;
 double RateControl::m_dPerm = 0.50;
 double RateControl::m_dPermSmall = 0.05;
 
 int RateControl::m_iRateRampSensitivity = 250;
+const double RateControl::kWheelMultiplier = 40.0;
+const double RateControl::kPausedJogMultiplier = 18.0;
 enum RateControl::RATERAMP_MODE RateControl::m_eRateRampMode = RateControl::RATERAMP_STEP;
 
-RateControl::RateControl(const char* _group,
+RateControl::RateControl(QString group,
                          ConfigObject<ConfigValue>* _config)
-    : EngineControl(_group, _config),
-      m_bVinylControlEnabled(false),
-      m_bVinylControlScratching(false),
+    : EngineControl(group, _config),
+      m_pBpmControl(NULL),
       m_ePbCurrent(0),
       m_ePbPressed(0),
       m_bTempStarted(false),
       m_dTempRateChange(0.0),
       m_dRateTemp(0.0),
       m_eRampBackMode(RATERAMP_RAMPBACK_NONE),
-      m_dRateTempRampbackChange(0.0),
-      m_dOldRate(0.0f) {
-    m_pScratchController = new PositionScratchController(_group);
+      m_dRateTempRampbackChange(0.0) {
+    m_pScratchController = new PositionScratchController(group);
 
-    m_pRateDir = new ControlObject(ConfigKey(_group, "rate_dir"));
-    m_pRateRange = new ControlObject(ConfigKey(_group, "rateRange"));
-    m_pRateSlider = new ControlPotmeter(ConfigKey(_group, "rate"), -1.f, 1.f);
+    m_pRateDir = new ControlObject(ConfigKey(group, "rate_dir"));
+    m_pRateRange = new ControlObject(ConfigKey(group, "rateRange"));
+    // Allow rate slider to go out of bounds so that master sync rate
+    // adjustments are not capped.
+    m_pRateSlider = new ControlPotmeter(ConfigKey(group, "rate"),
+                                        -1.0, 1.0, true);
 
     // Search rate. Rate used when searching in sound. This overrules the
     // playback rate
-    m_pRateSearch = new ControlPotmeter(ConfigKey(_group, "rateSearch"), -300., 300.);
+    m_pRateSearch = new ControlPotmeter(ConfigKey(group, "rateSearch"), -300., 300.);
 
     // Reverse button
-    m_pReverseButton = new ControlPushButton(ConfigKey(_group, "reverse"));
+    m_pReverseButton = new ControlPushButton(ConfigKey(group, "reverse"));
     m_pReverseButton->set(0);
 
     // Forward button
-    m_pForwardButton = new ControlPushButton(ConfigKey(_group, "fwd"));
+    m_pForwardButton = new ControlPushButton(ConfigKey(group, "fwd"));
     connect(m_pForwardButton, SIGNAL(valueChanged(double)),
             this, SLOT(slotControlFastForward(double)),
             Qt::DirectConnection);
     m_pForwardButton->set(0);
 
     // Back button
-    m_pBackButton = new ControlPushButton(ConfigKey(_group, "back"));
+    m_pBackButton = new ControlPushButton(ConfigKey(group, "back"));
     connect(m_pBackButton, SIGNAL(valueChanged(double)),
             this, SLOT(slotControlFastBack(double)),
             Qt::DirectConnection);
     m_pBackButton->set(0);
 
+    m_pReverseRollButton = new ControlPushButton(ConfigKey(group, "reverseroll"));
+    connect(m_pReverseRollButton, SIGNAL(valueChanged(double)),
+            this, SLOT(slotReverseRollActivate(double)),
+            Qt::DirectConnection);
+
+    m_pSlipEnabled = new ControlObjectSlave(group, "slip_enabled", this);
+
+    m_pVCEnabled = ControlObject::getControl(ConfigKey(getGroup(), "vinylcontrol_enabled"));
+    m_pVCScratching = ControlObject::getControl(ConfigKey(getGroup(), "vinylcontrol_scratching"));
+    m_pVCMode = ControlObject::getControl(ConfigKey(getGroup(), "vinylcontrol_mode"));
+
     // Permanent rate-change buttons
     buttonRatePermDown =
-        new ControlPushButton(ConfigKey(_group,"rate_perm_down"));
+        new ControlPushButton(ConfigKey(group,"rate_perm_down"));
     connect(buttonRatePermDown, SIGNAL(valueChanged(double)),
             this, SLOT(slotControlRatePermDown(double)),
             Qt::DirectConnection);
 
     buttonRatePermDownSmall =
-        new ControlPushButton(ConfigKey(_group,"rate_perm_down_small"));
+        new ControlPushButton(ConfigKey(group,"rate_perm_down_small"));
     connect(buttonRatePermDownSmall, SIGNAL(valueChanged(double)),
             this, SLOT(slotControlRatePermDownSmall(double)),
             Qt::DirectConnection);
 
     buttonRatePermUp =
-        new ControlPushButton(ConfigKey(_group,"rate_perm_up"));
+        new ControlPushButton(ConfigKey(group,"rate_perm_up"));
     connect(buttonRatePermUp, SIGNAL(valueChanged(double)),
             this, SLOT(slotControlRatePermUp(double)),
             Qt::DirectConnection);
 
     buttonRatePermUpSmall =
-        new ControlPushButton(ConfigKey(_group,"rate_perm_up_small"));
+        new ControlPushButton(ConfigKey(group,"rate_perm_up_small"));
     connect(buttonRatePermUpSmall, SIGNAL(valueChanged(double)),
             this, SLOT(slotControlRatePermUpSmall(double)),
             Qt::DirectConnection);
 
     // Temporary rate-change buttons
     buttonRateTempDown =
-        new ControlPushButton(ConfigKey(_group,"rate_temp_down"));
+        new ControlPushButton(ConfigKey(group,"rate_temp_down"));
     connect(buttonRateTempDown, SIGNAL(valueChanged(double)),
             this, SLOT(slotControlRateTempDown(double)),
             Qt::DirectConnection);
 
     buttonRateTempDownSmall =
-        new ControlPushButton(ConfigKey(_group,"rate_temp_down_small"));
+        new ControlPushButton(ConfigKey(group,"rate_temp_down_small"));
     connect(buttonRateTempDownSmall, SIGNAL(valueChanged(double)),
             this, SLOT(slotControlRateTempDownSmall(double)),
             Qt::DirectConnection);
 
     buttonRateTempUp =
-        new ControlPushButton(ConfigKey(_group,"rate_temp_up"));
+        new ControlPushButton(ConfigKey(group,"rate_temp_up"));
     connect(buttonRateTempUp, SIGNAL(valueChanged(double)),
             this, SLOT(slotControlRateTempUp(double)),
             Qt::DirectConnection);
 
     buttonRateTempUpSmall =
-        new ControlPushButton(ConfigKey(_group,"rate_temp_up_small"));
+        new ControlPushButton(ConfigKey(group,"rate_temp_up_small"));
     connect(buttonRateTempUpSmall, SIGNAL(valueChanged(double)),
             this, SLOT(slotControlRateTempUpSmall(double)),
             Qt::DirectConnection);
@@ -119,19 +138,25 @@ RateControl::RateControl(const char* _group,
     m_pSampleRate = ControlObject::getControl(ConfigKey("[Master]","samplerate"));
 
     // Wheel to control playback position/speed
-    m_pWheel = new ControlTTRotary(ConfigKey(_group, "wheel"));
+    m_pWheel = new ControlTTRotary(ConfigKey(group, "wheel"));
 
     // Scratch controller, this is an accumulator which is useful for
     // controllers that return individiual +1 or -1s, these get added up and
     // cleared when we read
-    m_pScratch = new ControlTTRotary(ConfigKey(_group, "scratch2"));
-    m_pOldScratch = new ControlTTRotary(ConfigKey(_group, "scratch"));  // Deprecated
+    m_pScratch2 = new ControlObject(ConfigKey(group, "scratch2"));
 
     // Scratch enable toggle
-    m_pScratchToggle = new ControlPushButton(ConfigKey(_group, "scratch2_enable"));
-    m_pScratchToggle->set(0);
+    m_pScratch2Enable = new ControlPushButton(ConfigKey(group, "scratch2_enable"));
+    m_pScratch2Enable->set(0);
 
-    m_pJog = new ControlObject(ConfigKey(_group, "jog"));
+    m_pScratch2Scratching = new ControlPushButton(ConfigKey(group,
+                                                            "scratch2_indicates_scratching"));
+    // Enable by default, because it was always scratching before introducing
+    // this control.
+    m_pScratch2Scratching->set(1.0);
+
+
+    m_pJog = new ControlObject(ConfigKey(group, "jog"));
     m_pJogFilter = new Rotary();
     // FIXME: This should be dependent on sample rate/block size or something
     m_pJogFilter->setFilterLength(25);
@@ -145,39 +170,19 @@ RateControl::RateControl(const char* _group,
     m_iRateRampSensitivity =
             getConfig()->getValueString(ConfigKey("[Controls]","RateRampSensitivity")).toInt();
 
-#ifdef __VINYLCONTROL__
-    ControlObject* pVCEnabled = ControlObject::getControl(ConfigKey(_group, "vinylcontrol_enabled"));
-    // Throw a hissy fit if somebody moved us such that the vinylcontrol_enabled
-    // control doesn't exist yet. This will blow up immediately, won't go unnoticed.
-    Q_ASSERT(pVCEnabled);
-    connect(pVCEnabled, SIGNAL(valueChanged(double)),
-            this, SLOT(slotControlVinyl(double)),
-            Qt::DirectConnection);
-    connect(pVCEnabled, SIGNAL(valueChangedFromEngine(double)),
-            this, SLOT(slotControlVinyl(double)),
-            Qt::DirectConnection);
-
-    ControlObject* pVCScratching = ControlObject::getControl(ConfigKey(_group, "vinylcontrol_scratching"));
-    // Throw a hissy fit if somebody moved us such that the vinylcontrol_enabled
-    // control doesn't exist yet. This will blow up immediately, won't go unnoticed.
-    Q_ASSERT(pVCScratching);
-    connect(pVCScratching, SIGNAL(valueChanged(double)),
-            this, SLOT(slotControlVinylScratching(double)),
-            Qt::DirectConnection);
-    connect(pVCScratching, SIGNAL(valueChangedFromEngine(double)),
-            this, SLOT(slotControlVinylScratching(double)),
-            Qt::DirectConnection);
-#endif
+    m_pSyncMode = new ControlObjectSlave(group, "sync_mode", this);
 }
 
 RateControl::~RateControl() {
     delete m_pRateSlider;
     delete m_pRateRange;
     delete m_pRateDir;
+    delete m_pSyncMode;
 
     delete m_pRateSearch;
 
     delete m_pReverseButton;
+    delete m_pReverseRollButton;
     delete m_pForwardButton;
     delete m_pBackButton;
 
@@ -191,20 +196,22 @@ RateControl::~RateControl() {
     delete buttonRatePermUpSmall;
 
     delete m_pWheel;
-    delete m_pScratch;
-    delete m_pOldScratch;
-    delete m_pScratchToggle;
+    delete m_pScratch2;
+    delete m_pScratch2Scratching;
+    delete m_pScratch2Enable;
     delete m_pJog;
     delete m_pJogFilter;
     delete m_pScratchController;
 }
 
+void RateControl::setBpmControl(BpmControl* bpmcontrol) {
+    m_pBpmControl = bpmcontrol;
+}
+
 void RateControl::setRateRamp(bool linearMode)
 {
-    if ( linearMode )
-        m_eRateRampMode = RateControl::RATERAMP_LINEAR;
-    else
-        m_eRateRampMode = RateControl::RATERAMP_STEP;
+    m_eRateRampMode = linearMode ?
+            RateControl::RATERAMP_LINEAR : RateControl::RATERAMP_STEP;
 }
 
 void RateControl::setRateRampSensitivity(int sense)
@@ -212,12 +219,13 @@ void RateControl::setRateRampSensitivity(int sense)
     // Reverse the actual sensitivity value passed.
     // That way the gui works in an intuitive manner.
     sense = RATE_SENSITIVITY_MAX - sense + RATE_SENSITIVITY_MIN;
-    if ( sense < RATE_SENSITIVITY_MIN )
+    if (sense < RATE_SENSITIVITY_MIN) {
         m_iRateRampSensitivity = RATE_SENSITIVITY_MIN;
-    else if ( sense > RATE_SENSITIVITY_MAX )
+    } else if (sense > RATE_SENSITIVITY_MAX) {
         m_iRateRampSensitivity = RATE_SENSITIVITY_MAX;
-    else
+    } else {
         m_iRateRampSensitivity = sense;
+    }
 }
 
 void RateControl::setTemp(double v) {
@@ -236,6 +244,16 @@ void RateControl::setPermSmall(double v) {
     m_dPermSmall = v;
 }
 
+void RateControl::slotReverseRollActivate(double v) {
+    if (v > 0.0) {
+        m_pSlipEnabled->set(1);
+        m_pReverseButton->set(1);
+    } else {
+        m_pReverseButton->set(0);
+        m_pSlipEnabled->set(0);
+    }
+}
+
 void RateControl::slotControlFastForward(double v)
 {
     //qDebug() << "slotControlFastForward(" << v << ")";
@@ -258,28 +276,33 @@ void RateControl::slotControlRatePermDown(double)
 {
     // Adjusts temp rate down if button pressed
     if (buttonRatePermDown->get())
-        m_pRateSlider->sub(m_pRateDir->get() * m_dPerm / (100. * m_pRateRange->get()));
+        m_pRateSlider->set(m_pRateSlider->get() -
+                           m_pRateDir->get() * m_dPerm / (100. * m_pRateRange->get()));
 }
 
 void RateControl::slotControlRatePermDownSmall(double)
 {
     // Adjusts temp rate down if button pressed
     if (buttonRatePermDownSmall->get())
-        m_pRateSlider->sub(m_pRateDir->get() * m_dPermSmall / (100. * m_pRateRange->get()));
+        m_pRateSlider->set(m_pRateSlider->get() -
+                           m_pRateDir->get() * m_dPermSmall / (100. * m_pRateRange->get()));
 }
 
 void RateControl::slotControlRatePermUp(double)
 {
     // Adjusts temp rate up if button pressed
-    if (buttonRatePermUp->get())
-        m_pRateSlider->add(m_pRateDir->get() * m_dPerm / (100. * m_pRateRange->get()));
+    if (buttonRatePermUp->get()) {
+        m_pRateSlider->set(m_pRateSlider->get() +
+                           m_pRateDir->get() * m_dPerm / (100. * m_pRateRange->get()));
+    }
 }
 
 void RateControl::slotControlRatePermUpSmall(double)
 {
     // Adjusts temp rate up if button pressed
     if (buttonRatePermUpSmall->get())
-        m_pRateSlider->add(m_pRateDir->get() * m_dPermSmall / (100. * m_pRateRange->get()));
+        m_pRateSlider->set(m_pRateSlider->get() +
+                           m_pRateDir->get() * m_dPermSmall / (100. * m_pRateRange->get()));
 }
 
 void RateControl::slotControlRateTempDown(double)
@@ -342,17 +365,29 @@ void RateControl::slotControlRateTempUpSmall(double)
     }
 }
 
-double RateControl::getRawRate() {
+void RateControl::trackLoaded(TrackPointer pTrack) {
+    if (m_pTrack) {
+        trackUnloaded(m_pTrack);
+    }
+    m_pTrack = pTrack;
+}
+
+void RateControl::trackUnloaded(TrackPointer pTrack) {
+    Q_UNUSED(pTrack);
+    m_pTrack.clear();
+}
+
+double RateControl::getRawRate() const {
     return m_pRateSlider->get() *
         m_pRateRange->get() *
         m_pRateDir->get();
 }
 
-double RateControl::getWheelFactor() {
+double RateControl::getWheelFactor() const {
     return m_pWheel->get();
 }
 
-double RateControl::getJogFactor() {
+double RateControl::getJogFactor() const {
     // FIXME: Sensitivity should be configurable separately?
     const double jogSensitivity = 0.1;  // Nudges during playback
     double jogValue = m_pJog->get();
@@ -365,85 +400,76 @@ double RateControl::getJogFactor() {
     double jogFactor = jogValueFiltered * jogSensitivity;
 
     if (isnan(jogValue) || isnan(jogFactor)) {
-        jogFactor = 0.0f;
+        jogFactor = 0.0;
     }
 
     return jogFactor;
 }
 
-double RateControl::calculateRate(double baserate, bool paused, int iSamplesPerBuffer,
-                                  bool* isScratching) {
-    double rate = (paused ? 0 : 1.0);
+SyncMode RateControl::getSyncMode() const {
+    return syncModeFromDouble(m_pSyncMode->get());
+}
 
+double RateControl::calculateSpeed(double baserate, double speed, bool paused,
+                                   int iSamplesPerBuffer,
+                                   bool* pReportScratching,
+                                   bool* pReportReverse) {
+    *pReportScratching = false;
+    *pReportReverse = false;
+    double rate = (paused ? 0 : 1.0);
     double searching = m_pRateSearch->get();
     if (searching) {
         // If searching is in progress, it overrides everything else
         rate = searching;
     } else {
-
-
         double wheelFactor = getWheelFactor();
         double jogFactor = getJogFactor();
-        bool scratchEnable = m_pScratchToggle->get() != 0 || m_bVinylControlEnabled;
+        bool bVinylControlEnabled = m_pVCEnabled && m_pVCEnabled->toBool();
+        bool useScratch2Value = m_pScratch2Enable->get() != 0;
 
-
-        double scratchFactor = m_pScratch->get();
-        // Don't trust values from m_pScratch
-        if (isnan(scratchFactor)) {
-            scratchFactor = 0.0;
+        // By default scratch2_enable is enough to determine if the user is
+        // scratching or not. Moving platter controllers have to disable
+        // "scratch2_indicates_scratching" if they are not scratching,
+        // to allow things like key-lock.
+        if (useScratch2Value && m_pScratch2Scratching->get()) {
+            *pReportScratching = true;
         }
 
-        // Old Scratch works without scratchEnable
-        double oldScratchFactor = m_pOldScratch->get(); // Deprecated
-        // Don't trust values from m_pScratch
-        if (isnan(oldScratchFactor)) {
-            oldScratchFactor = 0.0;
-        }
-
-        // If vinyl control is enabled and scratching then also set isScratching
-        if (m_bVinylControlEnabled && m_bVinylControlScratching) {
-            *isScratching = true;
-        }
-
-        if (paused) {
-            // Stopped. Wheel, jog and scratch controller all scrub through audio.
-            // New scratch behavior overrides old
-            if (scratchEnable) {
-                rate = scratchFactor + jogFactor + wheelFactor * 40.0;
-            } else {
-                // Just remove oldScratchFactor in future
-                rate = oldScratchFactor + jogFactor * 18 + wheelFactor;
+        if (bVinylControlEnabled) {
+            if (m_pVCScratching->toBool()) {
+                *pReportScratching = true;
             }
+            rate = speed;
         } else {
-            // The buffer is playing, so calculate the buffer rate.
-
-            // There are four rate effects we apply: wheel, scratch, jog and temp.
-            // Wheel: a linear additive effect (no spring-back)
-            // Scratch: a rate multiplier
-            // Jog: a linear additive effect whose value is filtered (springs back)
-            // Temp: pitch bend
-
-            // New scratch behavior - overrides playback speed (and old behavior)
-            if (scratchEnable) {
-                rate = scratchFactor;
-            } else {
-
-                rate = 1. + getRawRate() + getTempRate();
-                rate += wheelFactor;
-
-                // Deprecated old scratch behavior
-                if (oldScratchFactor < 0.) {
-                    rate *= (oldScratchFactor - 1.);
-                } else if (oldScratchFactor > 0.) {
-                    rate *= (oldScratchFactor + 1.);
-                }
+            double scratchFactor = m_pScratch2->get();
+            // Don't trust values from m_pScratch2
+            if (isnan(scratchFactor)) {
+                scratchFactor = 0.0;
             }
+            if (paused) {
+                // Stopped. Wheel, jog and scratch controller all scrub through audio.
+                if (useScratch2Value) {
+                    rate = scratchFactor + jogFactor + wheelFactor * kWheelMultiplier;
+                } else {
+                    rate = jogFactor * kPausedJogMultiplier + wheelFactor;
+                }
+            } else {
+                // The buffer is playing, so calculate the buffer rate.
 
-            rate += jogFactor;
+                // There are four rate effects we apply: wheel, scratch, jog and temp.
+                // Wheel: a linear additive effect (no spring-back)
+                // Scratch: a rate multiplier
+                // Jog: a linear additive effect whose value is filtered (springs back)
+                // Temp: pitch bend
 
-            // If we are reversing (and not scratching,) flip the rate.
-            if (!scratchEnable && m_pReverseButton->get()) {
-                rate = -rate;
+                // New scratch behavior - overrides playback speed (and old behavior)
+                if (useScratch2Value) {
+                    rate = scratchFactor;
+                } else {
+                    rate = speed + getTempRate();
+                    rate += wheelFactor;
+                }
+                rate += jogFactor;
             }
         }
 
@@ -453,13 +479,36 @@ double RateControl::calculateRate(double baserate, bool paused, int iSamplesPerB
         // If waveform scratch is enabled, override all other controls
         if (m_pScratchController->isEnabled()) {
             rate = m_pScratchController->getRate();
-            *isScratching = true;
+            *pReportScratching = true;
+        } else {
+            // If master sync is on, respond to it -- but vinyl and scratch mode always override.
+            if (getSyncMode() == SYNC_FOLLOWER && !paused &&
+                    !bVinylControlEnabled && !useScratch2Value) {
+                if (m_pBpmControl == NULL) {
+                    qDebug() << "ERROR: calculateRate m_pBpmControl is null during master sync";
+                    return 1.0;
+                }
+
+                double userTweak = 0.0;
+                if (!*pReportScratching) {
+                    // Only report user tweak if the user is not scratching.
+                    userTweak = getTempRate() + wheelFactor + jogFactor;
+                }
+                rate = m_pBpmControl->calcSyncedRate(userTweak);
+            }
+            // If we are reversing (and not scratching,) flip the rate.  This is ok even when syncing.
+            // Reverse with vinyl is only ok if absolute mode isn't on.
+            int vcmode = m_pVCMode ? m_pVCMode->get() : MIXXX_VCMODE_ABSOLUTE;
+            // TODO(owen): Instead of just ignoring reverse mode, should we
+            // disable absolute mode instead?
+            if (m_pReverseButton->get()
+                    && !m_pScratch2Enable->get()
+                    && (!bVinylControlEnabled || vcmode != MIXXX_VCMODE_ABSOLUTE)) {
+                rate = -rate;
+                *pReportReverse = true;
+            }
         }
     }
-
-    // Scale the rate by the engine samplerate
-    rate *= baserate;
-
     return rate;
 }
 
@@ -474,8 +523,8 @@ double RateControl::process(const double rate,
     /*
      * Code to handle temporary rate change buttons.
      *
-     * We support two behaviours, the standard ramped pitch bending
-     * and pitch shift stepping, which is the old behaviour.
+     * We support two behaviors, the standard ramped pitch bending
+     * and pitch shift stepping, which is the old behavior.
      */
 
     /*
@@ -488,14 +537,11 @@ double RateControl::process(const double rate,
     double latrate = ((double)bufferSamples / (double)m_pSampleRate->get());
 
 
-    if ((m_ePbPressed) && (!m_bTempStarted))
-    {
+    if ((m_ePbPressed) && (!m_bTempStarted)) {
         m_bTempStarted = true;
 
-
-        if ( m_eRateRampMode == RATERAMP_STEP )
-        {
-            // old temporary pitch shift behaviour
+        if (m_eRateRampMode == RATERAMP_STEP) {
+            // old temporary pitch shift behavior
             double range = m_pRateRange->get();
 
             // Avoid Division by Zero
@@ -517,9 +563,8 @@ double RateControl::process(const double rate,
                 addRateTemp(csmall);
             else if (buttonRateTempDownSmall->get())
                 subRateTemp(csmall);
-        }
-        else
-        {
+        } else {
+            // m_eRateRampMode == RATERAMP_LINEAR
             m_dTempRateChange = ((double)latrate / ((double)m_iRateRampSensitivity / 100.));
 
             if (m_eRampBackMode == RATERAMP_RAMPBACK_PERIOD)
@@ -529,48 +574,38 @@ double RateControl::process(const double rate,
     }
 
     if (m_eRateRampMode == RATERAMP_LINEAR) {
-
-        if (m_ePbCurrent)
-        {
+        if (m_ePbCurrent) {
             // apply ramped pitchbending
-            if ( m_ePbCurrent == RateControl::RATERAMP_UP )
+            if (m_ePbCurrent == RateControl::RATERAMP_UP) {
                 addRateTemp(m_dTempRateChange);
-            else if ( m_ePbCurrent == RateControl::RATERAMP_DOWN )
+            } else if (m_ePbCurrent == RateControl::RATERAMP_DOWN) {
                 subRateTemp(m_dTempRateChange);
-        }
-        else if ((m_bTempStarted) || ((m_eRampBackMode != RATERAMP_RAMPBACK_NONE) && (m_dRateTemp != 0.0)))
-        {
+            }
+        } else if ((m_bTempStarted)
+                || ((m_eRampBackMode != RATERAMP_RAMPBACK_NONE)
+                        && (m_dRateTemp != 0.0))) {
             // No buttons pressed, so time to deinitialize
             m_bTempStarted = false;
 
-
-            if ((m_eRampBackMode == RATERAMP_RAMPBACK_PERIOD) &&  (m_dRateTempRampbackChange == 0.0))
-            {
+            if ((m_eRampBackMode == RATERAMP_RAMPBACK_PERIOD)
+                    && (m_dRateTempRampbackChange == 0.0)) {
                 int period = 2;
-                if (period)
-                    m_dRateTempRampbackChange = fabs(m_dRateTemp / (double)period);
-                else {
+                m_dRateTempRampbackChange = fabs(
+                        m_dRateTemp / period);
+            } else if ((m_eRampBackMode != RATERAMP_RAMPBACK_NONE)
+                    && (m_dRateTempRampbackChange == 0.0)) {
+                if (fabs(m_dRateTemp) < m_dRateTempRampbackChange) {
                     resetRateTemp();
-                    return kNoTrigger;
-                }
-
-            }
-            else if ((m_eRampBackMode != RATERAMP_RAMPBACK_NONE) && (m_dRateTempRampbackChange == 0.0))
-            {
-
-                if ( fabs(m_dRateTemp) < m_dRateTempRampbackChange)
-                    resetRateTemp();
-                else if ( m_dRateTemp > 0 )
+                } else if (m_dRateTemp > 0) {
                     subRateTemp(m_dRateTempRampbackChange);
-                else
+                } else {
                     addRateTemp(m_dRateTempRampbackChange);
-            }
-            else
+                }
+            } else {
                 resetRateTemp();
+            }
         }
-    }
-    else if ((m_eRateRampMode == RATERAMP_STEP) && (m_bTempStarted))
-    {
+    } else if ((m_eRateRampMode == RATERAMP_STEP) && (m_bTempStarted)) {
         if (!m_ePbCurrent) {
             m_bTempStarted = false;
             resetRateTemp();
@@ -587,16 +622,17 @@ double RateControl::getTempRate() {
 void RateControl::setRateTemp(double v)
 {
     // Do not go backwards
-    if (( 1. + getRawRate() + v ) < 0)
+    if ((1. + getRawRate() + v) < 0)
         return;
 
     m_dRateTemp = v;
-    if ( m_dRateTemp < -1.0 )
+    if (m_dRateTemp < -1.0) {
         m_dRateTemp = -1.0;
-    else if ( m_dRateTemp > 1.0 )
+    } else if (m_dRateTemp > 1.0) {
         m_dRateTemp = 1.0;
-    else if ( isnan(m_dRateTemp))
+    } else if (isnan(m_dRateTemp)) {
         m_dRateTemp = 0;
+    }
 }
 
 void RateControl::addRateTemp(double v)
@@ -614,14 +650,6 @@ void RateControl::resetRateTemp(void)
     setRateTemp(0.0);
 }
 
-void RateControl::slotControlVinyl(double toggle) {
-    m_bVinylControlEnabled = (bool)toggle;
-}
-
-void RateControl::slotControlVinylScratching(double toggle) {
-    m_bVinylControlScratching = (bool)toggle;
-}
-
 void RateControl::notifySeek(double playPos) {
     m_pScratchController->notifySeek(playPos);
 }
