@@ -1,25 +1,37 @@
 // readaheadmanager.cpp
 // Created 8/2/2009 by RJ Ryan (rryan@mit.edu)
 
-#include <QMutexLocker>
-
 #include "engine/readaheadmanager.h"
 #include "sampleutil.h"
-
-#include "mathstuff.h"
-#include "engine/enginecontrol.h"
+#include "util/math.h"
+#include "util/defs.h"
+#include "engine/loopingcontrol.h"
+#include "engine/ratecontrol.h"
 #include "cachingreader.h"
 
-ReadAheadManager::ReadAheadManager(CachingReader* pReader) :
-    m_iCurrentPosition(0),
-    m_pReader(pReader),
-    m_pCrossFadeBuffer(new CSAMPLE[MAX_BUFFER_LEN]) {
-    // zero out crossfade buffer
-    SampleUtil::applyGain(m_pCrossFadeBuffer, 0.0, MAX_BUFFER_LEN);
+ReadAheadManager::ReadAheadManager()
+        : m_pLoopingControl(NULL),
+          m_pRateControl(NULL),
+          m_iCurrentPosition(0),
+          m_pReader(NULL),
+          m_pCrossFadeBuffer(SampleUtil::alloc(MAX_BUFFER_LEN)) {
+    // For testing only: ReadAheadManagerMock
+}
+
+ReadAheadManager::ReadAheadManager(CachingReader* pReader, 
+                                   LoopingControl* pLoopingControl) 
+        : m_pLoopingControl(pLoopingControl),
+          m_pRateControl(NULL),
+          m_iCurrentPosition(0),
+          m_pReader(pReader),
+          m_pCrossFadeBuffer(SampleUtil::alloc(MAX_BUFFER_LEN)) {
+    DEBUG_ASSERT(m_pLoopingControl != NULL);
+    DEBUG_ASSERT(m_pReader != NULL);
+    SampleUtil::clear(m_pCrossFadeBuffer, MAX_BUFFER_LEN);
 }
 
 ReadAheadManager::~ReadAheadManager() {
-    delete [] m_pCrossFadeBuffer;
+    SampleUtil::free(m_pCrossFadeBuffer);
 }
 
 int ReadAheadManager::getNextSamples(double dRate, CSAMPLE* buffer,
@@ -36,27 +48,20 @@ int ReadAheadManager::getNextSamples(double dRate, CSAMPLE* buffer,
 
     // A loop will only limit the amount we can read in one shot.
 
-    QPair<int, double> next_loop;
-    next_loop.first = 0;
-    next_loop.second = m_sEngineControls[0]->nextTrigger(dRate,
-                                                         m_iCurrentPosition,
-                                                         0, 0);
+    const double loop_trigger = m_pLoopingControl->nextTrigger(
+            dRate, m_iCurrentPosition, 0, 0);
+    bool loop_active = loop_trigger != kNoTrigger;
     int preloop_samples = 0;
 
-    if (next_loop.second != kNoTrigger) {
-        int samples_available;
-        if (in_reverse) {
-            samples_available = m_iCurrentPosition - next_loop.second;
+    if (loop_active) {
+        int samples_available = in_reverse ?
+                m_iCurrentPosition - loop_trigger :
+                loop_trigger - m_iCurrentPosition;
+        if (samples_available < 0) {
+            samples_needed = 0;
         } else {
-            samples_available = next_loop.second - m_iCurrentPosition;
-        }
-        samples_needed = math_max(0, math_min(samples_needed,
-                                              samples_available));
-
-        if (in_reverse) {
-            preloop_samples = m_iCurrentPosition - next_loop.second;
-        } else {
-            preloop_samples = next_loop.second - m_iCurrentPosition;
+            preloop_samples = samples_available;
+            samples_needed = math_clamp(samples_needed, 0, samples_available);
         }
     }
 
@@ -77,13 +82,6 @@ int ReadAheadManager::getNextSamples(double dRate, CSAMPLE* buffer,
         qDebug() << "didn't get what we wanted" << samples_read << samples_needed;
     }
 
-    // Sometimes our read length reduces to 0. In this case, just return 0 and
-    // make sure not to record a 0-length ReadLogEntry because that will be
-    // interpreted as a seek.
-    if (samples_read == 0) {
-        return 0;
-    }
-
     // Increment or decrement current read-ahead position
     if (in_reverse) {
         addReadLogEntry(m_iCurrentPosition, m_iCurrentPosition - samples_read);
@@ -94,78 +92,47 @@ int ReadAheadManager::getNextSamples(double dRate, CSAMPLE* buffer,
     }
 
     // Activate on this trigger if necessary
-    if (next_loop.second != kNoTrigger) {
-        double loop_trigger = next_loop.second;
-        double loop_target = m_sEngineControls[next_loop.first]->
-            getTrigger(dRate,
-                       m_iCurrentPosition,
-                       0, 0);
-
-        if (loop_target != kNoTrigger &&
-            ((in_reverse && m_iCurrentPosition <= loop_trigger) ||
-            (!in_reverse && m_iCurrentPosition >= loop_trigger))) {
+    if (loop_active) {
+        // LoopingControl makes the decision about whether we should loop or
+        // not.
+        const double loop_target = m_pLoopingControl->
+                process(dRate, m_iCurrentPosition, 0, 0);
 
+        if (loop_target != kNoTrigger) {
             m_iCurrentPosition = loop_target;
 
-            int loop_read_position = m_iCurrentPosition;
-            if (in_reverse) {
-                loop_read_position += preloop_samples;
-            } else {
-                loop_read_position -= preloop_samples;
-            }
+            int loop_read_position = m_iCurrentPosition +
+                    (in_reverse ? preloop_samples : -preloop_samples);
 
             int looping_samples_read = m_pReader->read(
-                loop_read_position, samples_read, m_pCrossFadeBuffer);
+                    loop_read_position, samples_read, m_pCrossFadeBuffer);
 
             if (looping_samples_read != samples_read) {
                 qDebug() << "ERROR: Couldn't get all needed samples for crossfade.";
             }
 
             // do crossfade from the current buffer into the new loop beginning
-            double mix_amount = 0.0;
-            double mix_inc = 2.0 / static_cast<double>(samples_read);
-            for (int i = 0; i < samples_read; i += 2) {
-                base_buffer[i] = base_buffer[i] * (1.0 - mix_amount) + m_pCrossFadeBuffer[i] * mix_amount;
-                base_buffer[i+1] = base_buffer[i+1] * (1.0 - mix_amount) + m_pCrossFadeBuffer[i+1] * mix_amount;
-                mix_amount += mix_inc;
+            if (samples_read != 0) { // avoid division by zero
+                SampleUtil::linearCrossfadeBuffers(base_buffer, base_buffer, m_pCrossFadeBuffer, samples_read);
             }
         }
     }
 
     // Reverse the samples in-place
     if (in_reverse) {
-        // TODO(rryan) pull this into MixxxUtil or something
-        CSAMPLE temp1, temp2;
-        for (int j = 0; j < samples_read/2; j += 2) {
-            const int endpos = samples_read-1-j-1;
-            temp1 = base_buffer[j];
-            temp2 = base_buffer[j+1];
-            base_buffer[j] = base_buffer[endpos];
-            base_buffer[j+1] = base_buffer[endpos+1];
-            base_buffer[endpos] = temp1;
-            base_buffer[endpos+1] = temp2;
-        }
+        SampleUtil::reverse(base_buffer, samples_read);
     }
 
     //qDebug() << "read" << m_iCurrentPosition << samples_read;
     return samples_read;
 }
 
-void ReadAheadManager::addEngineControl(EngineControl* pControl) {
-    if (pControl == NULL) {
-        return;
-    }
-    m_sEngineControls.append(pControl);
-}
-
-void ReadAheadManager::setNewPlaypos(int iNewPlaypos) {
-    QMutexLocker locker(&m_mutex);
-    m_iCurrentPosition = iNewPlaypos;
-    m_readAheadLog.clear();
+void ReadAheadManager::addRateControl(RateControl* pRateControl) {
+    m_pRateControl = pRateControl;
 }
 
+// Not thread-save, call from engine thread only
 void ReadAheadManager::notifySeek(int iSeekPosition) {
-    QMutexLocker locker(&m_mutex);
     m_iCurrentPosition = iSeekPosition;
     m_readAheadLog.clear();
 
@@ -178,13 +145,13 @@ void ReadAheadManager::notifySeek(int iSeekPosition) {
     // }
 }
 
-void ReadAheadManager::hintReader(double dRate, QList<Hint>& hintList) {
+void ReadAheadManager::hintReader(double dRate, HintVector* pHintList) {
     bool in_reverse = dRate < 0;
     Hint current_position;
 
     // SoundTouch can read up to 2 chunks ahead. Always keep 2 chunks ahead in
     // cache.
-    int length_to_cache = 2*CachingReader::kSamplesPerChunk;
+    int length_to_cache = 2 * CachingReaderWorker::kSamplesPerChunk;
 
     current_position.length = length_to_cache;
     current_position.sample = in_reverse ?
@@ -199,12 +166,12 @@ void ReadAheadManager::hintReader(double dRate, QList<Hint>& hintList) {
 
     // top priority, we need to read this data immediately
     current_position.priority = 1;
-    hintList.append(current_position);
+    pHintList->append(current_position);
 }
 
+// Not thread-save, call from engine thread only
 void ReadAheadManager::addReadLogEntry(double virtualPlaypositionStart,
                                        double virtualPlaypositionEndNonInclusive) {
-    QMutexLocker locker(&m_mutex);
     ReadLogEntry newEntry(virtualPlaypositionStart,
                           virtualPlaypositionEndNonInclusive);
     if (m_readAheadLog.size() > 0) {
@@ -216,13 +183,13 @@ void ReadAheadManager::addReadLogEntry(double virtualPlaypositionStart,
     m_readAheadLog.append(newEntry);
 }
 
+// Not thread-save, call from engine thread only
 int ReadAheadManager::getEffectiveVirtualPlaypositionFromLog(double currentVirtualPlayposition,
                                                              double numConsumedSamples) {
     if (numConsumedSamples == 0) {
         return currentVirtualPlayposition;
     }
 
-    QMutexLocker locker(&m_mutex);
     if (m_readAheadLog.size() == 0) {
         // No log entries to read from.
         qDebug() << this << "No read ahead log entries to read from. Case not currently handled.";
@@ -239,8 +206,9 @@ int ReadAheadManager::getEffectiveVirtualPlaypositionFromLog(double currentVirtu
 
         // Notify EngineControls that we have taken a seek.
         if (shouldNotifySeek) {
-            foreach (EngineControl* pControl, m_sEngineControls) {
-                pControl->notifySeek(entry.virtualPlaypositionStart);
+            m_pLoopingControl->notifySeek(entry.virtualPlaypositionStart);
+            if (m_pRateControl) {
+                m_pRateControl->notifySeek(entry.virtualPlaypositionStart);
             }
         }
 
