@@ -20,7 +20,15 @@ class BaseExternalLibraryFeature : public LibraryFeature {
     virtual void onRightClickChild(const QPoint& globalPos, QModelIndex index);
 
   protected:
-    virtual BaseSqlTableModel* getPlaylistModelForPlaylist(QString playlist) = 0;
+    // Must be implemented by external Libraries copied to Mixxx DB
+    virtual BaseSqlTableModel* getPlaylistModelForPlaylist(QString playlist) {
+        Q_UNUSED(playlist);
+        return NULL;
+    }
+    // Must be implemented by external Libraries not copied to Mixxx DB
+    virtual void appendTrackIdsFromRightClickIndex(QList<int>* trackIds, QString* pPlaylist);
+
+    QModelIndex m_lastRightClickedIndex;
 
   private slots:
     void slotAddToAutoDJ();
@@ -31,10 +39,9 @@ class BaseExternalLibraryFeature : public LibraryFeature {
     void addToAutoDJ(bool bTop);
 
     TrackCollection* m_pTrackCollection;
-    QModelIndex m_lastRightClickedIndex;
     QAction* m_pAddToAutoDJAction;
     QAction* m_pAddToAutoDJTopAction;
     QAction* m_pImportAsMixxxPlaylistAction;
 };
 
-#endif /* BASEEXTERNALLIBRARYFEATURE_H */
+#endif // BASEEXTERNALLIBRARYFEATURE_H
