@@ -3,7 +3,9 @@
 
 #include <QtSql>
 #include <QItemDelegate>
-#include <QtCore>
+#include <QString>
+#include <QObject>
+#include <QModelIndex>
 
 #include "library/trackmodel.h"
 #include "library/basesqltablemodel.h"
@@ -15,31 +17,22 @@ class BaseExternalPlaylistModel : public BaseSqlTableModel {
     Q_OBJECT
   public:
     BaseExternalPlaylistModel(QObject* pParent, TrackCollection* pTrackCollection,
-                              QString settingsNamespace, QString playlistsTable,
-                              QString playlistTracksTable, QString trackSource);
+                              const char* settingsNamespace, const QString& playlistsTable,
+                              const QString& playlistTracksTable, QSharedPointer<BaseTrackCache> trackSource);
+
     virtual ~BaseExternalPlaylistModel();
 
     virtual TrackPointer getTrack(const QModelIndex& index) const;
-    virtual void search(const QString& searchText);
     virtual bool isColumnInternal(int column);
-    virtual bool isColumnHiddenByDefault(int column);
-    virtual Qt::ItemFlags flags(const QModelIndex &index) const;
-    virtual void setPlaylist(QString path_name);
+    Qt::ItemFlags flags(const QModelIndex &index) const;
+    void setPlaylist(QString path_name);
+    virtual void trackLoaded(QString group, TrackPointer pTrack);
     virtual TrackModel::CapabilitiesFlags getCapabilities() const;
 
-  private slots:
-    void slotSearch(const QString& searchText);
-
-  signals:
-    void doSearch(const QString& searchText);
-
   private:
     QString m_playlistsTable;
     QString m_playlistTracksTable;
-    QString m_trackSource;
-
-    TrackCollection* m_pTrackCollection;
-    QSqlDatabase& m_database;
+    QSharedPointer<BaseTrackCache> m_trackSource;
 };
 
 #endif /* BASEEXTERNALPLAYLISTMODEL_H */
