@@ -1,5 +1,10 @@
 #include "library/baseplaylistfeature.h"
 
+#include <QInputDialog>
+#include <QFileDialog>
+#include <QFileInfo>
+#include <QDesktopServices>
+
 #include "library/parser.h"
 #include "library/parserm3u.h"
 #include "library/parserpls.h"
@@ -10,11 +15,12 @@
 #include "mixxxkeyboard.h"
 #include "widget/wlibrary.h"
 #include "widget/wlibrarytextbrowser.h"
+#include "util/assert.h"
 
-BasePlaylistFeature::BasePlaylistFeature(
-    QObject* parent, ConfigObject<ConfigValue>* pConfig,
-    TrackCollection* pTrackCollection,
-    QString rootViewName)
+BasePlaylistFeature::BasePlaylistFeature(QObject* parent,
+                                         ConfigObject<ConfigValue>* pConfig,
+                                         TrackCollection* pTrackCollection,
+                                         QString rootViewName)
         : LibraryFeature(parent),
           m_pConfig(pConfig),
           m_pTrackCollection(pTrackCollection),
@@ -22,7 +28,7 @@ BasePlaylistFeature::BasePlaylistFeature(
           m_trackDao(pTrackCollection->getTrackDAO()),
           m_pPlaylistTableModel(NULL),
           m_rootViewName(rootViewName) {
-    m_pCreatePlaylistAction = new QAction(tr("New Playlist"),this);
+    m_pCreatePlaylistAction = new QAction(tr("Create New Playlist"),this);
     connect(m_pCreatePlaylistAction, SIGNAL(triggered()),
             this, SLOT(slotCreatePlaylist()));
 
@@ -46,26 +52,33 @@ BasePlaylistFeature::BasePlaylistFeature(
     connect(m_pLockPlaylistAction, SIGNAL(triggered()),
             this, SLOT(slotTogglePlaylistLock()));
 
+    m_pDuplicatePlaylistAction = new QAction(tr("Duplicate"), this);
+    connect(m_pDuplicatePlaylistAction, SIGNAL(triggered()),
+            this, SLOT(slotDuplicatePlaylist()));
+
     m_pImportPlaylistAction = new QAction(tr("Import Playlist"),this);
     connect(m_pImportPlaylistAction, SIGNAL(triggered()),
             this, SLOT(slotImportPlaylist()));
 
-    m_pDuplicatePlaylistAction = new QAction(tr("Duplicate Playlist"), this);
-    connect(m_pDuplicatePlaylistAction, SIGNAL(triggered()),
-            this, SLOT(slotDuplicatePlaylist()));
-
     m_pExportPlaylistAction = new QAction(tr("Export Playlist"), this);
     connect(m_pExportPlaylistAction, SIGNAL(triggered()),
             this, SLOT(slotExportPlaylist()));
 
+    m_pAnalyzePlaylistAction = new QAction(tr("Analyze entire Playlist"), this);
+    connect(m_pAnalyzePlaylistAction, SIGNAL(triggered()),
+            this, SLOT(slotAnalyzePlaylist()));
+
     connect(&m_playlistDao, SIGNAL(added(int)),
             this, SLOT(slotPlaylistTableChanged(int)));
 
     connect(&m_playlistDao, SIGNAL(deleted(int)),
             this, SLOT(slotPlaylistTableChanged(int)));
 
-    connect(&m_playlistDao, SIGNAL(renamed(int)),
-            this, SLOT(slotPlaylistTableChanged(int)));
+    connect(&m_playlistDao, SIGNAL(renamed(int,QString)),
+            this, SLOT(slotPlaylistTableRenamed(int,QString)));
+
+    connect(&m_playlistDao, SIGNAL(changed(int)),
+            this, SLOT(slotPlaylistContentChanged(int)));
 
     connect(&m_playlistDao, SIGNAL(lockChanged(int)),
             this, SLOT(slotPlaylistTableChanged(int)));
@@ -82,28 +95,59 @@ BasePlaylistFeature::~BasePlaylistFeature() {
     delete m_pAddToAutoDJTopAction;
     delete m_pRenamePlaylistAction;
     delete m_pLockPlaylistAction;
+    delete m_pAnalyzePlaylistAction;
+}
+
+int BasePlaylistFeature::playlistIdFromIndex(QModelIndex index) {
+    TreeItem* item = static_cast<TreeItem*>(index.internalPointer());
+    if (item == NULL) {
+        return -1;
+    }
+
+    QString dataPath = item->dataPath().toString();
+    bool ok = false;
+    int playlistId = dataPath.toInt(&ok);
+    if (!ok) {
+        return -1;
+    }
+    return playlistId;
 }
 
 void BasePlaylistFeature::activate() {
     emit(switchToView(m_rootViewName));
     emit(restoreSearch(QString())); // Null String disables search box
+    emit(enableCoverArtDisplay(true));
 }
 
 void BasePlaylistFeature::activateChild(const QModelIndex& index) {
     //qDebug() << "BasePlaylistFeature::activateChild()" << index;
-
-    // Switch the playlist table model's playlist.
-    QString playlistName = index.data().toString();
-    int playlistId = m_playlistDao.getPlaylistIdFromName(playlistName);
-    if (m_pPlaylistTableModel) {
-        m_pPlaylistTableModel->setPlaylist(playlistId);
+    int playlistId = playlistIdFromIndex(index);
+    if (playlistId != -1 && m_pPlaylistTableModel) {
+        m_pPlaylistTableModel->setTableModel(playlistId);
         emit(showTrackModel(m_pPlaylistTableModel));
+        emit(enableCoverArtDisplay(true));
+    }
+}
+
+void BasePlaylistFeature::activatePlaylist(int playlistId) {
+    //qDebug() << "BasePlaylistFeature::activatePlaylist()" << playlistId;
+    QModelIndex index = indexFromPlaylistId(playlistId);
+    if (playlistId != -1 && index.isValid() && m_pPlaylistTableModel) {
+        m_pPlaylistTableModel->setTableModel(playlistId);
+        emit(showTrackModel(m_pPlaylistTableModel));
+        emit(enableCoverArtDisplay(true));
+        // Update selection
+        emit(featureSelect(this, m_lastRightClickedIndex));
+        activateChild(m_lastRightClickedIndex);
     }
 }
 
 void BasePlaylistFeature::slotRenamePlaylist() {
-    QString oldName = m_lastRightClickedIndex.data().toString();
-    int playlistId = m_playlistDao.getPlaylistIdFromName(oldName);
+    int playlistId = playlistIdFromIndex(m_lastRightClickedIndex);
+    if (playlistId == -1) {
+        return;
+    }
+    QString oldName = m_playlistDao.getPlaylistName(playlistId);
     bool locked = m_playlistDao.isPlaylistLocked(playlistId);
 
     if (locked) {
@@ -111,19 +155,17 @@ void BasePlaylistFeature::slotRenamePlaylist() {
                  << "is locked.";
         return;
     }
-
     QString newName;
     bool validNameGiven = false;
 
-    do {
+    while (!validNameGiven) {
         bool ok = false;
         newName = QInputDialog::getText(NULL,
                                         tr("Rename Playlist"),
-                                        tr("New playlist name:"),
+                                        tr("Enter new name for playlist:"),
                                         QLineEdit::Normal,
                                         oldName,
                                         &ok).trimmed();
-
         if (!ok || oldName == newName) {
             return;
         }
@@ -134,38 +176,38 @@ void BasePlaylistFeature::slotRenamePlaylist() {
             QMessageBox::warning(NULL,
                                 tr("Renaming Playlist Failed"),
                                 tr("A playlist by that name already exists."));
-        }
-        else if (newName.isEmpty()) {
+        } else if (newName.isEmpty()) {
             QMessageBox::warning(NULL,
                                 tr("Renaming Playlist Failed"),
                                 tr("A playlist cannot have a blank name."));
-        }
-        else {
+        } else {
             validNameGiven = true;
         }
-    } while (!validNameGiven);
+    }
 
     m_playlistDao.renamePlaylist(playlistId, newName);
 }
 
 void BasePlaylistFeature::slotDuplicatePlaylist() {
-    QString oldName = m_lastRightClickedIndex.data().toString();
-    int oldPlaylistId = m_playlistDao.getPlaylistIdFromName(oldName);
+    int oldPlaylistId = playlistIdFromIndex(m_lastRightClickedIndex);
+    if (oldPlaylistId == -1) {
+        return;
+    }
 
+    QString oldName = m_playlistDao.getPlaylistName(oldPlaylistId);
 
     QString name;
     bool validNameGiven = false;
 
-    do {
+    while (!validNameGiven) {
         bool ok = false;
         name = QInputDialog::getText(NULL,
-                                        tr("Duplicate Playlist"),
-                                        tr("Playlist name:"),
-                                        QLineEdit::Normal,
-                                        //: Appendix to default name when duplicating a playlist
-                                        oldName + tr("_copy" , "[noun]"),
-                                        &ok).trimmed();
-
+                                     tr("Duplicate Playlist"),
+                                     tr("Enter name for new playlist:"),
+                                     QLineEdit::Normal,
+                                     //: Appendix to default name when duplicating a playlist
+                                     oldName + tr("_copy" , "[noun]"),
+                                     &ok).trimmed();
         if (!ok || oldName == name) {
             return;
         }
@@ -174,30 +216,30 @@ void BasePlaylistFeature::slotDuplicatePlaylist() {
 
         if (existingId != -1) {
             QMessageBox::warning(NULL,
-                                tr("Playlist Creation Failed"),
-                                tr("A playlist by that name already exists."));
-        }
-        else if (name.isEmpty()) {
+                                 tr("Playlist Creation Failed"),
+                                 tr("A playlist by that name already exists."));
+        } else if (name.isEmpty()) {
             QMessageBox::warning(NULL,
-                                tr("Playlist Creation Failed"),
-                                tr("A playlist cannot have a blank name."));
-        }
-        else {
+                                 tr("Playlist Creation Failed"),
+                                 tr("A playlist cannot have a blank name."));
+        } else {
             validNameGiven = true;
         }
-    } while (!validNameGiven);
+    }
 
     int newPlaylistId = m_playlistDao.createPlaylist(name);
-    m_playlistDao.copyPlaylistTracks(oldPlaylistId, newPlaylistId);
 
-    if (newPlaylistId != -1) {
-        emit(showTrackModel(m_pPlaylistTableModel));
+    if (newPlaylistId != -1 &&
+        m_playlistDao.copyPlaylistTracks(oldPlaylistId, newPlaylistId)) {
+        activatePlaylist(newPlaylistId);
     }
 }
 
 void BasePlaylistFeature::slotTogglePlaylistLock() {
-    QString playlistName = m_lastRightClickedIndex.data().toString();
-    int playlistId = m_playlistDao.getPlaylistIdFromName(playlistName);
+    int playlistId = playlistIdFromIndex(m_lastRightClickedIndex);
+    if (playlistId == -1) {
+        return;
+    }
     bool locked = !m_playlistDao.isPlaylistLocked(playlistId);
 
     if (!m_playlistDao.setPlaylistLocked(playlistId, locked)) {
@@ -213,15 +255,14 @@ void BasePlaylistFeature::slotCreatePlaylist() {
     QString name;
     bool validNameGiven = false;
 
-    do {
+    while (!validNameGiven) {
         bool ok = false;
         name = QInputDialog::getText(NULL,
-                                     tr("New Playlist"),
-                                     tr("Playlist name:"),
+                                     tr("Create New Playlist"),
+                                     tr("Enter name for new playlist:"),
                                      QLineEdit::Normal,
                                      tr("New Playlist"),
                                      &ok).trimmed();
-
         if (!ok)
             return;
 
@@ -238,15 +279,13 @@ void BasePlaylistFeature::slotCreatePlaylist() {
         } else {
             validNameGiven = true;
         }
-
-    } while (!validNameGiven);
+    }
 
     int playlistId = m_playlistDao.createPlaylist(name);
 
     if (playlistId != -1) {
-        emit(showTrackModel(m_pPlaylistTableModel));
-    }
-    else {
+        activatePlaylist(playlistId);
+    } else {
         QMessageBox::warning(NULL,
                              tr("Playlist Creation Failed"),
                              tr("An unknown error occurred while creating playlist: ")
@@ -254,19 +293,23 @@ void BasePlaylistFeature::slotCreatePlaylist() {
     }
 }
 
-
 void BasePlaylistFeature::slotDeletePlaylist() {
     //qDebug() << "slotDeletePlaylist() row:" << m_lastRightClickedIndex.data();
-    int playlistId = m_playlistDao.getPlaylistIdFromName(m_lastRightClickedIndex.data().toString());
-    bool locked = m_playlistDao.isPlaylistLocked(playlistId);
+    int playlistId = playlistIdFromIndex(m_lastRightClickedIndex);
+    if (playlistId == -1) {
+        return;
+    }
 
+    bool locked = m_playlistDao.isPlaylistLocked(playlistId);
     if (locked) {
         qDebug() << "Skipping playlist deletion because playlist" << playlistId << "is locked.";
         return;
     }
 
     if (m_lastRightClickedIndex.isValid()) {
-        Q_ASSERT(playlistId >= 0);
+        DEBUG_ASSERT_AND_HANDLE(playlistId >= 0) {
+            return;
+        }
 
         m_playlistDao.deletePlaylist(playlistId);
         activate();
@@ -275,26 +318,40 @@ void BasePlaylistFeature::slotDeletePlaylist() {
 
 
 void BasePlaylistFeature::slotImportPlaylist() {
-    qDebug() << "slotImportPlaylist() row:" ; //<< m_lastRightClickedIndex.data();
+    //qDebug() << "slotImportPlaylist() row:" << m_lastRightClickedIndex.data();
 
     if (!m_pPlaylistTableModel) {
         return;
     }
 
+    QString lastPlaylistDirectory = m_pConfig->getValueString(
+            ConfigKey("[Library]", "LastImportExportPlaylistDirectory"),
+            QDesktopServices::storageLocation(QDesktopServices::MusicLocation));
+
     QString playlist_file = QFileDialog::getOpenFileName(
-        NULL,
-        tr("Import Playlist"),
-        QDesktopServices::storageLocation(QDesktopServices::MusicLocation),
-        tr("Playlist Files (*.m3u *.m3u8 *.pls *.csv)"));
+            NULL,
+            tr("Import Playlist"),
+            lastPlaylistDirectory,
+            tr("Playlist Files (*.m3u *.m3u8 *.pls *.csv)"));
     // Exit method if user cancelled the open dialog.
     if (playlist_file.isNull() || playlist_file.isEmpty()) {
         return;
     }
 
+    // Update the import/export playlist directory
+    QFileInfo fileName(playlist_file);
+    m_pConfig->set(ConfigKey("[Library]","LastImportExportPlaylistDirectory"),
+                ConfigValue(fileName.dir().absolutePath()));
+
+    // The user has picked a new directory via a file dialog. This means the
+    // system sandboxer (if we are sandboxed) has granted us permission to this
+    // folder. We don't need access to this file on a regular basis so we do not
+    // register a security bookmark.
+
     Parser* playlist_parser = NULL;
 
     if (playlist_file.endsWith(".m3u", Qt::CaseInsensitive) ||
-        playlist_file.endsWith(".m3u8", Qt::CaseInsensitive)) {
+            playlist_file.endsWith(".m3u8", Qt::CaseInsensitive)) {
         playlist_parser = new ParserM3u();
     } else if (playlist_file.endsWith(".pls", Qt::CaseInsensitive)) {
         playlist_parser = new ParserPls();
@@ -303,14 +360,16 @@ void BasePlaylistFeature::slotImportPlaylist() {
     } else {
         return;
     }
-    QList<QString> entries = playlist_parser->parse(playlist_file);
 
-    // Iterate over the List that holds URLs of playlist entires
-    m_pPlaylistTableModel->addTracks(QModelIndex(), entries);
-
-    // delete the parser object
     if (playlist_parser) {
-        delete playlist_parser;
+      QStringList entries = playlist_parser->parse(playlist_file);
+
+      // Iterate over the List that holds URLs of playlist entires
+      m_pPlaylistTableModel->addTracks(QModelIndex(), entries);
+      activateChild(m_lastRightClickedIndex);
+
+      // delete the parser object
+      delete playlist_parser;
     }
 }
 
@@ -318,31 +377,60 @@ void BasePlaylistFeature::slotExportPlaylist() {
     if (!m_pPlaylistTableModel) {
         return;
     }
+    int playlistId = playlistIdFromIndex(m_lastRightClickedIndex);
+    if (playlistId == -1) {
+        return;
+    }
+    QString playlistName = m_playlistDao.getPlaylistName(playlistId);
+    qDebug() << "Export playlist" << playlistName;
+
+    QString lastPlaylistDirectory = m_pConfig->getValueString(
+                ConfigKey("[Library]", "LastImportExportPlaylistDirectory"),
+                QDesktopServices::storageLocation(QDesktopServices::MusicLocation));
 
-    qDebug() << "Export playlist" << m_lastRightClickedIndex.data();
     // Open a dialog to let the user choose the file location for playlist export.
-    // By default, the directory is set to the OS's Music directory and the file
+    // The location is set to the last used directory for import/export and the file
     // name to the playlist name.
-    QString playlist_filename = m_lastRightClickedIndex.data().toString();
-    QString music_directory = QDesktopServices::storageLocation(QDesktopServices::MusicLocation);
+    QString filefilter = tr("M3U Playlist (*.m3u)");
     QString file_location = QFileDialog::getSaveFileName(
-        NULL,
-        tr("Export Playlist"),
-        music_directory.append("/").append(playlist_filename),
-        tr("M3U Playlist (*.m3u);;M3U8 Playlist (*.m3u8);;"
-           "PLS Playlist (*.pls);;Text CSV (*.csv);;Readable Text (*.txt)"));
+            NULL,
+            tr("Export Playlist"),
+            lastPlaylistDirectory.append("/").append(playlistName),
+            tr("M3U Playlist (*.m3u);;M3U8 Playlist (*.m3u8);;"
+            "PLS Playlist (*.pls);;Text CSV (*.csv);;Readable Text (*.txt)"),
+            &filefilter);
     // Exit method if user cancelled the open dialog.
     if (file_location.isNull() || file_location.isEmpty()) {
         return;
     }
+    // Manually add extension due to bug in QFileDialog
+    // via https://bugreports.qt-project.org/browse/QTBUG-27186
+    // Can be removed after switch to Qt5
+    QFileInfo fileName(file_location);
+    if (fileName.suffix().isNull() || fileName.suffix().isEmpty()) {
+        QString ext = filefilter.section(".",1,1);
+        ext.chop(1);
+        file_location.append(".").append(ext);
+    }
+    // Update the import/export playlist directory
+    //QFileInfo fileName(file_location);
+    m_pConfig->set(ConfigKey("[Library]","LastImportExportPlaylistDirectory"),
+                ConfigValue(fileName.dir().absolutePath()));
+
+    // The user has picked a new directory via a file dialog. This means the
+    // system sandboxer (if we are sandboxed) has granted us permission to this
+    // folder. We don't need access to this file on a regular basis so we do not
+    // register a security bookmark.
 
     // Create a new table model since the main one might have an active search.
+    // This will only export songs that we think exist on default
     QScopedPointer<PlaylistTableModel> pPlaylistTableModel(
         new PlaylistTableModel(this, m_pTrackCollection,
                                "mixxx.db.model.playlist_export"));
 
-    pPlaylistTableModel->setPlaylist(m_pPlaylistTableModel->getPlaylist());
-    pPlaylistTableModel->setSort(pPlaylistTableModel->fieldIndex(PLAYLISTTRACKSTABLE_POSITION), Qt::AscendingOrder);
+    pPlaylistTableModel->setTableModel(m_pPlaylistTableModel->getPlaylist());
+    pPlaylistTableModel->setSort(pPlaylistTableModel->fieldIndex(
+            ColumnCache::COLUMN_PLAYLISTTRACKSTABLE_POSITION), Qt::AscendingOrder);
     pPlaylistTableModel->select();
 
     // check config if relative paths are desired
@@ -395,17 +483,24 @@ void BasePlaylistFeature::slotAddToAutoDJTop() {
 
 void BasePlaylistFeature::addToAutoDJ(bool bTop) {
     //qDebug() << "slotAddToAutoDJ() row:" << m_lastRightClickedIndex.data();
-
     if (m_lastRightClickedIndex.isValid()) {
-        int playlistId = m_playlistDao.getPlaylistIdFromName(
-            m_lastRightClickedIndex.data().toString());
+        int playlistId = playlistIdFromIndex(m_lastRightClickedIndex);
         if (playlistId >= 0) {
             // Insert this playlist
-            m_playlistDao.addToAutoDJQueue(playlistId, bTop);
+            m_playlistDao.addPlaylistToAutoDJQueue(playlistId, bTop);
         }
     }
 }
 
+void BasePlaylistFeature::slotAnalyzePlaylist() {
+    if (m_lastRightClickedIndex.isValid()) {
+        int playlistId = playlistIdFromIndex(m_lastRightClickedIndex);
+        if (playlistId >= 0) {
+            QList<int> ids = m_playlistDao.getTrackIds(playlistId);
+            emit(analyzeTracks(ids));
+        }
+    }
+}
 
 TreeItemModel* BasePlaylistFeature::getChildModel() {
     return &m_childModel;
@@ -417,14 +512,13 @@ void BasePlaylistFeature::bindWidget(WLibrary* libraryWidget,
     WLibraryTextBrowser* edit = new WLibraryTextBrowser(libraryWidget);
     edit->setHtml(getRootViewHtml());
     edit->setOpenLinks(false);
-    connect(edit,SIGNAL(anchorClicked(const QUrl)),
-        this,SLOT(htmlLinkClicked(const QUrl))
-    );
+    connect(edit, SIGNAL(anchorClicked(const QUrl)),
+            this, SLOT(htmlLinkClicked(const QUrl)));
     libraryWidget->registerView(m_rootViewName, edit);
 }
 
-void BasePlaylistFeature::htmlLinkClicked(const QUrl & link) {
-    if (QString(link.path())=="create") {
+void BasePlaylistFeature::htmlLinkClicked(const QUrl& link) {
+    if (QString(link.path()) == "create") {
         slotCreatePlaylist();
     } else {
         qDebug() << "Unknonw playlist link clicked" << link.path();
@@ -436,8 +530,7 @@ void BasePlaylistFeature::htmlLinkClicked(const QUrl & link) {
   * we require the sidebar model not to reset.
   * This method queries the database and does dynamic insertion
 */
-QModelIndex BasePlaylistFeature::constructChildModel(int selected_id)
-{
+QModelIndex BasePlaylistFeature::constructChildModel(int selected_id) {
     buildPlaylistList();
     QList<TreeItem*> data_list;
     int selected_row = -1;
@@ -457,7 +550,7 @@ QModelIndex BasePlaylistFeature::constructChildModel(int selected_id)
         }
 
         // Create the TreeItem whose parent is the invisible root item
-        TreeItem* item = new TreeItem(playlist_name, playlist_name, this, root);
+        TreeItem* item = new TreeItem(playlist_name, QString::number(playlist_id), this, root);
         decorateChild(item, playlist_id);
         data_list.append(item);
     }
@@ -470,6 +563,26 @@ QModelIndex BasePlaylistFeature::constructChildModel(int selected_id)
     return m_childModel.index(selected_row, 0);
 }
 
+void BasePlaylistFeature::updateChildModel(int selected_id) {
+    buildPlaylistList();
+
+    int row = 0;
+    for (QList<QPair<int, QString> >::const_iterator it = m_playlistList.begin();
+         it != m_playlistList.end(); ++it, ++row) {
+        int playlist_id = it->first;
+        QString playlist_name = it->second;
+
+        if (selected_id == playlist_id) {
+            TreeItem* item = m_childModel.getItem(indexFromPlaylistId(playlist_id));
+            item->setData(playlist_name, QString::number(playlist_id));
+            decorateChild(item, playlist_id);
+        }
+
+    }
+}
+
+
+
 /**
   * Clears the child model dynamically, but the invisible root item remains
   */
@@ -477,3 +590,16 @@ void BasePlaylistFeature::clearChildModel() {
     m_childModel.removeRows(0, m_playlistList.size());
 }
 
+QModelIndex BasePlaylistFeature::indexFromPlaylistId(int playlistId) {
+    int row = 0;
+    for (QList<QPair<int, QString> >::const_iterator it = m_playlistList.begin();
+         it != m_playlistList.end(); ++it, ++row) {
+        int current_id = it->first;
+        QString playlist_name = it->second;
+
+        if (playlistId == current_id) {
+            return m_childModel.index(row, 0);
+        }
+    }
+    return QModelIndex();
+}
