@@ -1,9 +1,11 @@
 // cratefeature.cpp
 // Created 10/22/2009 by RJ Ryan (rryan@mit.edu)
 
+#include <QFileDialog>
 #include <QInputDialog>
-#include <QMenu>
 #include <QLineEdit>
+#include <QMenu>
+#include <QDesktopServices>
 
 #include "library/cratefeature.h"
 #include "library/parser.h"
@@ -13,20 +15,24 @@
 
 #include "library/cratetablemodel.h"
 #include "library/trackcollection.h"
+#include "library/queryutil.h"
 #include "widget/wlibrarytextbrowser.h"
 #include "widget/wlibrary.h"
 #include "mixxxkeyboard.h"
 #include "treeitem.h"
 #include "soundsourceproxy.h"
+#include "util/dnd.h"
+#include "util/time.h"
 
 CrateFeature::CrateFeature(QObject* parent,
-                           TrackCollection* pTrackCollection, ConfigObject<ConfigValue>* pConfig)
+                           TrackCollection* pTrackCollection,
+                           ConfigObject<ConfigValue>* pConfig)
         : m_pTrackCollection(pTrackCollection),
           m_crateDao(pTrackCollection->getCrateDAO()),
           m_crateTableModel(this, pTrackCollection),
           m_pConfig(pConfig) {
     Q_UNUSED(parent);
-    m_pCreateCrateAction = new QAction(tr("New Crate"),this);
+    m_pCreateCrateAction = new QAction(tr("Create New Crate"),this);
     connect(m_pCreateCrateAction, SIGNAL(triggered()),
             this, SLOT(slotCreateCrate()));
 
@@ -45,22 +51,39 @@ CrateFeature::CrateFeature(QObject* parent,
     m_pImportPlaylistAction = new QAction(tr("Import Crate"),this);
     connect(m_pImportPlaylistAction, SIGNAL(triggered()),
             this, SLOT(slotImportPlaylist()));
+
     m_pExportPlaylistAction = new QAction(tr("Export Crate"), this);
     connect(m_pExportPlaylistAction, SIGNAL(triggered()),
             this, SLOT(slotExportPlaylist()));
 
-    m_pDuplicateCrateAction = new QAction(tr("Duplicate Crate"),this);
+    m_pDuplicateCrateAction = new QAction(tr("Duplicate"),this);
     connect(m_pDuplicateCrateAction, SIGNAL(triggered()),
             this, SLOT(slotDuplicateCrate()));
 
+    m_pAnalyzeCrateAction = new QAction(tr("Analyze entire Crate"),this);
+    connect(m_pAnalyzeCrateAction, SIGNAL(triggered()),
+            this, SLOT(slotAnalyzeCrate()));
+
+#ifdef __AUTODJCRATES__
+
+    m_pAutoDjTrackSource = new QAction(tr("Auto DJ Track Source"),this);
+    m_pAutoDjTrackSource->setCheckable(true);
+    connect(m_pAutoDjTrackSource, SIGNAL(changed()),
+            this, SLOT(slotAutoDjTrackSourceChanged()));
+
+#endif // __AUTODJCRATES__
+
     connect(&m_crateDao, SIGNAL(added(int)),
             this, SLOT(slotCrateTableChanged(int)));
 
     connect(&m_crateDao, SIGNAL(deleted(int)),
             this, SLOT(slotCrateTableChanged(int)));
 
-    connect(&m_crateDao, SIGNAL(renamed(int)),
-            this, SLOT(slotCrateTableChanged(int)));
+    connect(&m_crateDao, SIGNAL(changed(int)),
+            this, SLOT(slotCrateContentChanged(int)));
+
+    connect(&m_crateDao, SIGNAL(renamed(int,QString)),
+            this, SLOT(slotCrateTableRenamed(int,QString)));
 
     connect(&m_crateDao, SIGNAL(lockChanged(int)),
             this, SLOT(slotCrateTableChanged(int)));
@@ -79,6 +102,10 @@ CrateFeature::~CrateFeature() {
     delete m_pDuplicateCrateAction;
     delete m_pLockCrateAction;
     delete m_pImportPlaylistAction;
+    delete m_pAnalyzeCrateAction;
+#ifdef __AUTODJCRATES__
+    delete m_pAutoDjTrackSource;
+#endif // __AUTODJCRATES__
 }
 
 QVariant CrateFeature::title() {
@@ -89,20 +116,32 @@ QIcon CrateFeature::getIcon() {
     return QIcon(":/images/library/ic_library_crates.png");
 }
 
-bool CrateFeature::dropAcceptChild(const QModelIndex& index, QList<QUrl> urls,
-                                   QWidget *pSource) {
-    QString crateName = index.data().toString();
-    int crateId = m_crateDao.getCrateIdByName(crateName);
-    QList<QFileInfo> files;
-    foreach (QUrl url, urls) {
-        //XXX: See the comment in PlaylistFeature::dropAcceptChild() about
-        //     QUrl::toLocalFile() vs. QUrl::toString() usage.
-        files.append(url.toLocalFile());
+int CrateFeature::crateIdFromIndex(QModelIndex index) {
+    TreeItem* item = static_cast<TreeItem*>(index.internalPointer());
+    if (item == NULL) {
+        return -1;
     }
 
+    QString dataPath = item->dataPath().toString();
+    bool ok = false;
+    int playlistId = dataPath.toInt(&ok);
+    if (!ok) {
+        return -1;
+    }
+    return playlistId;
+}
+
+bool CrateFeature::dropAcceptChild(const QModelIndex& index, QList<QUrl> urls,
+                                   QObject* pSource) {
+    int crateId = crateIdFromIndex(index);
+    if (crateId == -1) {
+        return false;
+    }
+    QList<QFileInfo> files = DragAndDropHelper::supportedTracksFromUrls(urls, false, true);
     QList<int> trackIds;
     if (pSource) {
         trackIds = m_pTrackCollection->getTrackDAO().getTrackIds(files);
+        m_pTrackCollection->getTrackDAO().unhideTracks(trackIds);
     } else {
         // Adds track, does not insert duplicates, handles unremoving logic.
         trackIds = m_pTrackCollection->getTrackDAO().addTracks(files, true);
@@ -110,23 +149,24 @@ bool CrateFeature::dropAcceptChild(const QModelIndex& index, QList<QUrl> urls,
     qDebug() << "CrateFeature::dropAcceptChild adding tracks"
             << trackIds.size() << " to crate "<< crateId;
     // remove tracks that could not be added
-    for (int trackId =0; trackId<trackIds.size() ; trackId++) {
+    for (int trackId = 0; trackId < trackIds.size(); ++trackId) {
         if (trackIds.at(trackId) < 0) {
             trackIds.removeAt(trackId--);
         }
     }
-    m_crateDao.addTracksToCrate(trackIds, crateId);
+    m_crateDao.addTracksToCrate(crateId, &trackIds);
     return true;
 }
 
 bool CrateFeature::dragMoveAcceptChild(const QModelIndex& index, QUrl url) {
-    //TODO: Filter by supported formats regex and reject anything that doesn't match.
-    QString crateName = index.data().toString();
-    int crateId = m_crateDao.getCrateIdByName(crateName);
+    int crateId = crateIdFromIndex(index);
+    if (crateId == -1) {
+        return false;
+    }
     bool locked = m_crateDao.isCrateLocked(crateId);
-
     QFileInfo file(url.toLocalFile());
-    bool formatSupported = SoundSourceProxy::isFilenameSupported(file.fileName());
+    bool formatSupported = SoundSourceProxy::isFilenameSupported(file.fileName()) ||
+            Parser::isPlaylistFilenameSupported(file.fileName());
     return !locked && formatSupported;
 }
 
@@ -136,9 +176,8 @@ void CrateFeature::bindWidget(WLibrary* libraryWidget,
     WLibraryTextBrowser* edit = new WLibraryTextBrowser(libraryWidget);
     edit->setHtml(getRootViewHtml());
     edit->setOpenLinks(false);
-    connect(edit,SIGNAL(anchorClicked(const QUrl)),
-        this,SLOT(htmlLinkClicked(const QUrl))
-    );
+    connect(edit, SIGNAL(anchorClicked(const QUrl)),
+            this, SLOT(htmlLinkClicked(const QUrl)));
     libraryWidget->registerView("CRATEHOME", edit);
 }
 
@@ -149,17 +188,35 @@ TreeItemModel* CrateFeature::getChildModel() {
 void CrateFeature::activate() {
     emit(switchToView("CRATEHOME"));
     emit(restoreSearch(QString())); //disable search on crate home
+    emit(enableCoverArtDisplay(true));
 }
 
 void CrateFeature::activateChild(const QModelIndex& index) {
     if (!index.isValid())
         return;
-    QString crateName = index.data().toString();
-    int crateId = m_crateDao.getCrateIdByName(crateName);
-    m_crateTableModel.setCrate(crateId);
+    int crateId = crateIdFromIndex(index);
+    if (crateId == -1) {
+        return;
+    }
+    m_crateTableModel.setTableModel(crateId);
     emit(showTrackModel(&m_crateTableModel));
+    emit(enableCoverArtDisplay(true));
 }
 
+void CrateFeature::activateCrate(int crateId) {
+    //qDebug() << "CrateFeature::activateCrate()" << crateId;
+    QModelIndex index = indexFromCrateId(crateId);   
+    if (crateId != -1 && index.isValid()) {
+        m_crateTableModel.setTableModel(crateId);
+        emit(showTrackModel(&m_crateTableModel));
+        emit(enableCoverArtDisplay(true));
+        // Update selection
+        emit(featureSelect(this, m_lastRightClickedIndex));
+        activateChild(m_lastRightClickedIndex);
+    }
+}
+
+
 void CrateFeature::onRightClick(const QPoint& globalPos) {
     m_lastRightClickedIndex = QModelIndex();
     QMenu menu(NULL);
@@ -170,15 +227,21 @@ void CrateFeature::onRightClick(const QPoint& globalPos) {
 void CrateFeature::onRightClickChild(const QPoint& globalPos, QModelIndex index) {
     //Save the model index so we can get it in the action slots...
     m_lastRightClickedIndex = index;
-
-    QString crateName = index.data().toString();
-    int crateId = m_crateDao.getCrateIdByName(crateName);
+    int crateId = crateIdFromIndex(index);
+    if (crateId == -1) {
+        return;
+    }
 
     bool locked = m_crateDao.isCrateLocked(crateId);
 
     m_pDeleteCrateAction->setEnabled(!locked);
     m_pRenameCrateAction->setEnabled(!locked);
 
+#ifdef __AUTODJCRATES__
+    bool bAutoDj = m_crateDao.isCrateInAutoDj(crateId);
+    m_pAutoDjTrackSource->setChecked(bAutoDj);
+#endif // __AUTODJCRATES__
+
     m_pLockCrateAction->setText(locked ? tr("Unlock") : tr("Lock"));
 
     QMenu menu(NULL);
@@ -189,21 +252,26 @@ void CrateFeature::onRightClickChild(const QPoint& globalPos, QModelIndex index)
     menu.addAction(m_pDeleteCrateAction);
     menu.addAction(m_pLockCrateAction);
     menu.addSeparator();
+#ifdef __AUTODJCRATES__
+    menu.addAction(m_pAutoDjTrackSource);
+    menu.addSeparator();
+#endif // __AUTODJCRATES__
+    menu.addAction(m_pAnalyzeCrateAction);
+    menu.addSeparator();
     menu.addAction(m_pImportPlaylistAction);
     menu.addAction(m_pExportPlaylistAction);
     menu.exec(globalPos);
 }
 
 void CrateFeature::slotCreateCrate() {
-
     QString name;
     bool validNameGiven = false;
 
-    do {
+    while (!validNameGiven) {
         bool ok = false;
         name = QInputDialog::getText(NULL,
-                                     tr("New Crate"),
-                                     tr("Crate name:"),
+                                     tr("Create New Crate"),
+                                     tr("Enter name for new crate:"),
                                      QLineEdit::Normal, tr("New Crate"),
                                      &ok).trimmed();
 
@@ -216,37 +284,34 @@ void CrateFeature::slotCreateCrate() {
             QMessageBox::warning(NULL,
                                  tr("Creating Crate Failed"),
                                  tr("A crate by that name already exists."));
-        }
-        else if (name.isEmpty()) {
+        } else if (name.isEmpty()) {
             QMessageBox::warning(NULL,
                                  tr("Creating Crate Failed"),
                                  tr("A crate cannot have a blank name."));
-        }
-        else {
+        } else {
             validNameGiven = true;
         }
-
-    } while (!validNameGiven);
+    }
 
     int crateId = m_crateDao.createCrate(name);
-
     if (crateId != -1) {
-        emit(showTrackModel(&m_crateTableModel));
+        activateCrate(crateId);
     } else {
         qDebug() << "Error creating crate with name " << name;
         QMessageBox::warning(NULL,
                              tr("Creating Crate Failed"),
                              tr("An unknown error occurred while creating crate: ")
                              + name);
-
     }
 }
 
 void CrateFeature::slotDeleteCrate() {
-    QString crateName = m_lastRightClickedIndex.data().toString();
-    int crateId = m_crateDao.getCrateIdByName(crateName);
-    bool locked = m_crateDao.isCrateLocked(crateId);
+    int crateId = crateIdFromIndex(m_lastRightClickedIndex);
+    if (crateId == -1) {
+        return;
+    }
 
+    bool locked = m_crateDao.isCrateLocked(crateId);
     if (locked) {
         qDebug() << "Skipping crate deletion because crate" << crateId << "is locked.";
         return;
@@ -254,7 +319,7 @@ void CrateFeature::slotDeleteCrate() {
 
     bool deleted = m_crateDao.deleteCrate(crateId);
 
-    if (deleted) {;
+    if (deleted) {
         activate();
     } else {
         qDebug() << "Failed to delete crateId" << crateId;
@@ -262,10 +327,13 @@ void CrateFeature::slotDeleteCrate() {
 }
 
 void CrateFeature::slotRenameCrate() {
-    QString oldName = m_lastRightClickedIndex.data().toString();
-    int crateId = m_crateDao.getCrateIdByName(oldName);
-    bool locked = m_crateDao.isCrateLocked(crateId);
+    int crateId = crateIdFromIndex(m_lastRightClickedIndex);
+    if (crateId == -1) {
+        return;
+    }
+    QString oldName = m_crateDao.crateName(crateId);
 
+    bool locked = m_crateDao.isCrateLocked(crateId);
     if (locked) {
         qDebug() << "Skipping crate rename because crate" << crateId << "is locked.";
         return;
@@ -274,11 +342,11 @@ void CrateFeature::slotRenameCrate() {
     QString newName;
     bool validNameGiven = false;
 
-    do {
+    while (!validNameGiven) {
         bool ok = false;
         newName = QInputDialog::getText(NULL,
                                         tr("Rename Crate"),
-                                        tr("New crate name:"),
+                                        tr("Enter new name for crate:"),
                                         QLineEdit::Normal,
                                         oldName,
                                         &ok).trimmed();
@@ -293,17 +361,14 @@ void CrateFeature::slotRenameCrate() {
             QMessageBox::warning(NULL,
                                 tr("Renaming Crate Failed"),
                                 tr("A crate by that name already exists."));
-        }
-        else if (newName.isEmpty()) {
+        } else if (newName.isEmpty()) {
             QMessageBox::warning(NULL,
                                 tr("Renaming Crate Failed"),
                                 tr("A crate cannot have a blank name."));
-        }
-        else {
+        } else {
             validNameGiven = true;
         }
-    } while (!validNameGiven);
-
+    }
 
     if (!m_crateDao.renameCrate(crateId, newName)) {
         qDebug() << "Failed to rename crateId" << crateId;
@@ -311,62 +376,62 @@ void CrateFeature::slotRenameCrate() {
 }
 
 void CrateFeature::slotDuplicateCrate() {
-    QString oldName = m_lastRightClickedIndex.data().toString();
-    int oldCrateId = m_crateDao.getCrateIdByName(oldName);
+    int oldCrateId = crateIdFromIndex(m_lastRightClickedIndex);
+    if (oldCrateId == -1) {
+        return;
+    }
+    QString oldName = m_crateDao.crateName(oldCrateId);
 
     QString name;
     bool validNameGiven = false;
-
-    do {
+    while (!validNameGiven) {
         bool ok = false;
         name = QInputDialog::getText(NULL,
-                                        tr("Duplicate Crate"),
-                                        tr("New crate name:"),
-                                        QLineEdit::Normal,
-                                        //: Appendix to default name when duplicating a crate
-                                        oldName + tr("_copy" , "[noun]"),
-                                        &ok).trimmed();
+                                     tr("Duplicate Crate"),
+                                     tr("Enter name for new crate:"),
+                                     QLineEdit::Normal,
+                                     //: Appendix to default name when duplicating a crate
+                                     oldName + tr("_copy" , "[noun]"),
+                                     &ok).trimmed();
 
         if (!ok || name == oldName) {
             return;
         }
 
         int existingId = m_crateDao.getCrateIdByName(name);
-
         if (existingId != -1) {
             QMessageBox::warning(NULL,
                                 tr("Renaming Crate Failed"),
                                 tr("A crate by that name already exists."));
-        }
-        else if (name.isEmpty()) {
+        } else if (name.isEmpty()) {
             QMessageBox::warning(NULL,
                                 tr("Renaming Crate Failed"),
                                 tr("A crate cannot have a blank name."));
-        }
-        else {
+        } else {
             validNameGiven = true;
         }
-    } while (!validNameGiven);
+    }
 
     int newCrateId = m_crateDao.createCrate(name);
     m_crateDao.copyCrateTracks(oldCrateId, newCrateId);
 
     if (newCrateId != -1) {
-        emit(showTrackModel(&m_crateTableModel));
+        activateCrate(newCrateId);
     } else {
         qDebug() << "Error creating crate with name " << name;
         QMessageBox::warning(NULL,
                              tr("Creating Crate Failed"),
                              tr("An unknown error occurred while creating crate: ")
                              + name);
-
     }
 }
 
-void CrateFeature::slotToggleCrateLock()
-{
-    QString crateName = m_lastRightClickedIndex.data().toString();
-    int crateId = m_crateDao.getCrateIdByName(crateName);
+void CrateFeature::slotToggleCrateLock() {
+    int crateId = crateIdFromIndex(m_lastRightClickedIndex);
+    if (crateId == -1) {
+        return;
+    }
+    QString crateName = m_crateDao.crateName(crateId);
     bool locked = !m_crateDao.isCrateLocked(crateId);
 
     if (!m_crateDao.setCrateLocked(crateId, locked)) {
@@ -374,26 +439,61 @@ void CrateFeature::slotToggleCrateLock()
     }
 }
 
+void CrateFeature::slotAutoDjTrackSourceChanged() {
+#ifdef __AUTODJCRATES__
+    int crateId = crateIdFromIndex(m_lastRightClickedIndex);
+    if (crateId != -1) {
+        m_crateDao.setCrateInAutoDj(crateId, m_pAutoDjTrackSource->isChecked());
+    }
+#endif // __AUTODJCRATES__
+}
+
 void CrateFeature::buildCrateList() {
     m_crateList.clear();
+
+    QString queryString = QString(
+        "CREATE TEMPORARY VIEW IF NOT EXISTS CratesCountsDurations "
+        "AS SELECT "
+        "  crates.id as id, "
+        "  crates.name as name, "
+        "  COUNT(library.id) as count, "
+        "  SUM(library.duration) as durationSeconds "
+        "FROM crates "
+        "LEFT JOIN crate_tracks ON crate_tracks.crate_id = crates.id "
+        "LEFT JOIN library ON crate_tracks.track_id = library.id "
+        "WHERE show = 1 "
+        "GROUP BY crates.id;");
+    QSqlQuery query(m_pTrackCollection->getDatabase());
+    if (!query.exec(queryString)) {
+        LOG_FAILED_QUERY(query);
+    }
+
     QSqlTableModel crateListTableModel(this, m_pTrackCollection->getDatabase());
-    crateListTableModel.setTable("crates");
+    crateListTableModel.setTable("CratesCountsDurations");
     crateListTableModel.setSort(crateListTableModel.fieldIndex("name"),
                                 Qt::AscendingOrder);
-    crateListTableModel.setFilter("show = 1");
     crateListTableModel.select();
     while (crateListTableModel.canFetchMore()) {
         crateListTableModel.fetchMore();
     }
-    int nameColumn = crateListTableModel.record().indexOf("name");
-    int idColumn = crateListTableModel.record().indexOf("id");
+    QSqlRecord record = crateListTableModel.record();
+    int nameColumn = record.indexOf("name");
+    int idColumn = record.indexOf("id");
+    int countColumn = record.indexOf("count");
+    int durationColumn = record.indexOf("durationSeconds");
 
     for (int row = 0; row < crateListTableModel.rowCount(); ++row) {
         int id = crateListTableModel.data(
             crateListTableModel.index(row, idColumn)).toInt();
         QString name = crateListTableModel.data(
             crateListTableModel.index(row, nameColumn)).toString();
-        m_crateList.append(qMakePair(id, name));
+        int count = crateListTableModel.data(
+            crateListTableModel.index(row, countColumn)).toInt();
+        int duration = crateListTableModel.data(
+            crateListTableModel.index(row, durationColumn)).toInt();
+        m_crateList.append(qMakePair(id, QString("%1 (%2) %3")
+                                     .arg(name, QString::number(count),
+                                          Time::formatSeconds(duration, false))));
     }
 }
 
@@ -402,8 +502,7 @@ void CrateFeature::buildCrateList() {
   * we require the sidebar model not to reset.
   * This method queries the database and does dynamic insertion
 */
-QModelIndex CrateFeature::constructChildModel(int selected_id)
-{
+QModelIndex CrateFeature::constructChildModel(int selected_id) {
     buildCrateList();
     QList<TreeItem*> data_list;
     int selected_row = -1;
@@ -423,7 +522,7 @@ QModelIndex CrateFeature::constructChildModel(int selected_id)
         }
 
         // Create the TreeItem whose parent is the invisible root item
-        TreeItem* item = new TreeItem(crate_name, crate_name, this, root);
+        TreeItem* item = new TreeItem(crate_name, QString::number(crate_id), this, root);
         bool locked = m_crateDao.isCrateLocked(crate_id);
         item->setIcon(locked ? QIcon(":/images/library/ic_library_locked.png") : QIcon());
         data_list.append(item);
@@ -437,6 +536,26 @@ QModelIndex CrateFeature::constructChildModel(int selected_id)
     return m_childModel.index(selected_row, 0);
 }
 
+void CrateFeature::updateChildModel(int selected_id) {
+    buildCrateList();
+
+    int row = 0;
+    for (QList<QPair<int, QString> >::const_iterator it = m_crateList.begin();
+         it != m_crateList.end(); ++it, ++row) {
+        int crate_id = it->first;
+        QString crate_name = it->second;
+
+        if (selected_id == crate_id) {
+            TreeItem* item = m_childModel.getItem(indexFromCrateId(crate_id));
+            item->setData(crate_name, QString::number(crate_id));
+            bool locked = m_crateDao.isCrateLocked(crate_id);
+            item->setIcon(locked ? QIcon(":/images/library/ic_library_locked.png") : QIcon());
+
+        }
+
+    }
+}
+
 /**
   * Clears the child model dynamically
   */
@@ -445,18 +564,30 @@ void CrateFeature::clearChildModel() {
     m_crateList.clear();
 }
 
-void CrateFeature::slotImportPlaylist()
-{
+void CrateFeature::slotImportPlaylist() {
     qDebug() << "slotImportPlaylist() row:" ; //<< m_lastRightClickedIndex.data();
 
+    QString lastCrateDirectory = m_pConfig->getValueString(
+            ConfigKey("[Library]", "LastImportExportCrateDirectory"),
+            QDesktopServices::storageLocation(QDesktopServices::MusicLocation));
 
     QString playlist_file = QFileDialog::getOpenFileName(
         NULL,
         tr("Import Playlist"),
-        QDesktopServices::storageLocation(QDesktopServices::MusicLocation),
+        lastCrateDirectory,
         tr("Playlist Files (*.m3u *.m3u8 *.pls *.csv)"));
     // Exit method if user cancelled the open dialog.
-    if (playlist_file.isNull() || playlist_file.isEmpty() ) return;
+    if (playlist_file.isNull() || playlist_file.isEmpty()) return;
+
+    // Update the import/export crate directory
+    QFileInfo fileName(playlist_file);
+    m_pConfig->set(ConfigKey("[Library]","LastImportExportCrateDirectory"),
+                   ConfigValue(fileName.dir().absolutePath()));
+
+    // The user has picked a new directory via a file dialog. This means the
+    // system sandboxer (if we are sandboxed) has granted us permission to this
+    // folder. We don't need access to this file on a regular basis so we do not
+    // register a security bookmark.
 
     Parser* playlist_parser = NULL;
 
@@ -472,28 +603,56 @@ void CrateFeature::slotImportPlaylist()
         return;
     }
 
-    QList<QString> entries = playlist_parser->parse(playlist_file);
-    //qDebug() << "Size of Imported Playlist: " << entries.size();
+    if (playlist_parser) {
+      QList<QString> entries = playlist_parser->parse(playlist_file);
+      //qDebug() << "Size of Imported Playlist: " << entries.size();
 
-    //Iterate over the List that holds URLs of playlist entires
-    m_crateTableModel.addTracks(QModelIndex(), entries);
-
-    //delete the parser object
-    if (playlist_parser)
-        delete playlist_parser;
+      //Iterate over the List that holds URLs of playlist entires
+      m_crateTableModel.addTracks(QModelIndex(), entries);
+      activateChild(m_lastRightClickedIndex);
+      
+      //delete the parser object
+      delete playlist_parser;
+    }
 }
 
-void CrateFeature::slotExportPlaylist(){
+void CrateFeature::slotAnalyzeCrate() {
+    if (m_lastRightClickedIndex.isValid()) {
+        int crateId = crateIdFromIndex(m_lastRightClickedIndex);
+        if (crateId >= 0) {
+            QList<int> ids = m_crateDao.getTrackIds(crateId);
+            emit(analyzeTracks(ids));
+        }
+    }
+}
+
+void CrateFeature::slotExportPlaylist() {
     qDebug() << "Export crate" << m_lastRightClickedIndex.data();
+
+    QString lastCrateDirectory = m_pConfig->getValueString(
+            ConfigKey("[Library]", "LastImportExportCrateDirectory"),
+            QDesktopServices::storageLocation(QDesktopServices::MusicLocation));
+
     QString file_location = QFileDialog::getSaveFileName(
         NULL,
         tr("Export Crate"),
-        QDesktopServices::storageLocation(QDesktopServices::MusicLocation),
+        lastCrateDirectory,
         tr("M3U Playlist (*.m3u);;M3U8 Playlist (*.m3u8);;PLS Playlist (*.pls);;Text CSV (*.csv);;Readable Text (*.txt)"));
     // Exit method if user cancelled the open dialog.
     if (file_location.isNull() || file_location.isEmpty()) {
         return;
     }
+
+    // Update the import/export crate directory
+    QFileInfo fileName(file_location);
+    m_pConfig->set(ConfigKey("[Library]","LastImportExportCrateDirectory"),
+                ConfigValue(fileName.dir().absolutePath()));
+
+    // The user has picked a new directory via a file dialog. This means the
+    // system sandboxer (if we are sandboxed) has granted us permission to this
+    // folder. We don't need access to this file on a regular basis so we do not
+    // register a security bookmark.
+
     // check config if relative paths are desired
     bool useRelativePath = static_cast<bool>(
         m_pConfig->getValueString(
@@ -504,13 +663,13 @@ void CrateFeature::slotExportPlaylist(){
     // Create a new table model since the main one might have an active search.
     QScopedPointer<CrateTableModel> pCrateTableModel(
         new CrateTableModel(this, m_pTrackCollection));
-    pCrateTableModel->setCrate(m_crateTableModel.getCrate());
+    pCrateTableModel->setTableModel(m_crateTableModel.getCrate());
     pCrateTableModel->select();
 
     if (file_location.endsWith(".csv", Qt::CaseInsensitive)) {
-            ParserCsv::writeCSVFile(file_location, pCrateTableModel.data(), useRelativePath);
+        ParserCsv::writeCSVFile(file_location, pCrateTableModel.data(), useRelativePath);
     } else if (file_location.endsWith(".txt", Qt::CaseInsensitive)) {
-             ParserCsv::writeReadableTextFile(file_location, pCrateTableModel.data(), false);
+        ParserCsv::writeReadableTextFile(file_location, pCrateTableModel.data(), false);
     } else{
         // populate a list of files of the crate
         QList<QString> playlist_items;
@@ -538,16 +697,22 @@ void CrateFeature::slotExportPlaylist(){
 }
 
 void CrateFeature::slotCrateTableChanged(int crateId) {
-    //qDebug() << "slotPlaylistTableChanged() playlistId:" << playlistId;
+    //qDebug() << "slotCrateTableChanged() crateId:" << crateId;
     clearChildModel();
     m_lastRightClickedIndex = constructChildModel(crateId);
-    // Switch the view to the crate.
-    m_crateTableModel.setCrate(crateId);
-    // Update selection
-    emit(featureSelect(this, m_lastRightClickedIndex));
 }
 
-void CrateFeature::htmlLinkClicked(const QUrl & link) {
+void CrateFeature::slotCrateContentChanged(int crateId) {
+    //qDebug() << "slotCrateContentChanged()crateId:" << crateId;
+    updateChildModel(crateId);
+}
+
+void CrateFeature::slotCrateTableRenamed(int a_iCrateId,
+                                         QString /* a_strName */) {
+    activateCrate(a_iCrateId);
+}
+
+void CrateFeature::htmlLinkClicked(const QUrl& link) {
     if (QString(link.path())=="create") {
         slotCreateCrate();
     } else {
@@ -562,7 +727,7 @@ QString CrateFeature::getRootViewHtml() const {
     QString cratesSummary3 = tr("Crates let you organize your music however you'd like!");
 
     QString html;
-    QString createCrateLink = tr("Create new crate");
+    QString createCrateLink = tr("Create New Crate");
     html.append(QString("<h2>%1</h2>").arg(cratesTitle));
     html.append("<table border=\"0\" cellpadding=\"5\"><tr><td>");
     html.append(QString("<p>%1</p>").arg(cratesSummary));
@@ -571,9 +736,22 @@ QString CrateFeature::getRootViewHtml() const {
     html.append("</td><td rowspan=\"2\">");
     html.append("<img src=\"qrc:/images/library/crates_art.png\">");
     html.append("</td></tr>");
-    html.append(
-        QString("<tr><td><a href=\"create\">%1</a>").arg(createCrateLink)
-    );
+    html.append(QString("<tr><td><a href=\"create\">%1</a>")
+                .arg(createCrateLink));
     html.append("</td></tr></table>");
     return html;
 }
+
+QModelIndex CrateFeature::indexFromCrateId(int crateId) {
+    int row = 0;
+    for (QList<QPair<int, QString> >::const_iterator it = m_crateList.begin();
+         it != m_crateList.end(); ++it, ++row) {
+        int current_id = it->first;
+        QString crate_name = it->second;
+
+        if (crateId == current_id) {
+            return m_childModel.index(row, 0);
+        }
+    }
+    return QModelIndex();
+}
