@@ -46,7 +46,7 @@ int CrateDAO::createCrate(const QString& name) {
     return crateId;
 }
 
-bool CrateDAO::renameCrate(int crateId, const QString& newName) {
+bool CrateDAO::renameCrate(const int crateId, const QString& newName) {
     QSqlQuery query(m_database);
     query.prepare("UPDATE " CRATE_TABLE " SET name = :name WHERE id = :id");
     query.bindValue(":name", newName);
@@ -56,11 +56,11 @@ bool CrateDAO::renameCrate(int crateId, const QString& newName) {
         LOG_FAILED_QUERY(query);
         return false;
     }
-    emit(renamed(crateId));
+    emit(renamed(crateId, newName));
     return true;
 }
 
-bool CrateDAO::setCrateLocked(int crateId, bool locked) {
+bool CrateDAO::setCrateLocked(const int crateId, const bool locked) {
     // SQLite3 doesn't support boolean value. Using integer instead.
     int lock = locked ? 1 : 0;
     QSqlQuery query(m_database);
@@ -76,7 +76,7 @@ bool CrateDAO::setCrateLocked(int crateId, bool locked) {
     return true;
 }
 
-bool CrateDAO::isCrateLocked(int crateId) {
+bool CrateDAO::isCrateLocked(const int crateId) {
     QSqlQuery query(m_database);
     query.prepare("SELECT locked FROM " CRATE_TABLE " WHERE id = :id");
     query.bindValue(":id", crateId);
@@ -93,7 +93,129 @@ bool CrateDAO::isCrateLocked(int crateId) {
     return false;
 }
 
-bool CrateDAO::deleteCrate(int crateId) {
+QList<int> CrateDAO::getTrackIds(const int crateId) {
+    QSqlQuery query(m_database);
+    query.prepare("SELECT track_id from crate_tracks WHERE crate_id = :id");
+    query.bindValue(":id", crateId);
+
+    if (!query.exec()) {
+        LOG_FAILED_QUERY(query);
+        return QList<int> ();
+    }
+
+    QList<int> ids;
+    const int trackIdColumn = query.record().indexOf("track_id");
+    while (query.next()) {
+        ids.append(query.value(trackIdColumn).toInt());
+    }
+    return ids;
+}
+
+#ifdef __AUTODJCRATES__
+
+bool CrateDAO::setCrateInAutoDj(int crateId, bool bIn) {
+    // SQLite3 doesn't support boolean value. Using integer instead.
+    int iIn = bIn ? 1 : 0;
+
+    // Mark this crate as being in/out-of the auto-DJ list.
+    QSqlQuery query(m_database);
+    // UPDATE crates SET autodj = :in WHERE id = :id AND autodj = :existing;
+    query.prepare(QString("UPDATE " CRATE_TABLE
+        " SET %1 = :in WHERE %2 = :id AND %1 = :existing")
+        .arg(CRATETABLE_AUTODJ_SOURCE)     // %1
+        .arg(CRATETABLE_ID));       // %2
+    query.bindValue(":in", iIn);
+    query.bindValue(":id", crateId);
+    query.bindValue(":existing", 1 - iIn);
+
+    if (!query.exec()) {
+        LOG_FAILED_QUERY(query);
+        return false;
+    }
+
+    // Notify listeners if the auto-DJ status of a crate has changed.
+    bool bChange = (query.numRowsAffected() > 0);
+    if (bChange) {
+        emit(autoDjChanged(crateId, bIn));
+    }
+
+    // Let our caller know if there was a change.
+    return bChange;
+}
+
+bool CrateDAO::isCrateInAutoDj(int crateId) {
+    // Query the database for this crate's auto-DJ status.
+    QSqlQuery query(m_database);
+    query.setForwardOnly(true);
+    // SELECT autodj FROM crates WHERE id = :id;
+    query.prepare(QString("SELECT %1 FROM " CRATE_TABLE " WHERE %2 = :id")
+        .arg(CRATETABLE_AUTODJ_SOURCE) // %1
+        .arg(CRATETABLE_ID)); // %2
+    query.bindValue(":id", crateId);
+
+    if (query.exec()) {
+        if (query.next()) {
+            //SQLite3 doesn't support boolean values, so convert the integer.
+            int inValue = query.value(0).toInt();
+            return inValue == 1;
+        }
+    } else {
+        LOG_FAILED_QUERY(query);
+    }
+
+    return false;
+}
+
+QList<int> CrateDAO::getCrateTracks(int crateId) {
+    // Get all track IDs that belong to this crate.
+    QSqlQuery query(m_database);
+    query.setForwardOnly(true);
+    // SELECT track_id FROM crate_tracks WHERE crate_id = :id;
+    query.prepare(QString("SELECT %1 FROM " CRATE_TRACKS_TABLE
+        " WHERE %2 = :id")
+        .arg(CRATETRACKSTABLE_TRACKID)      // %1
+        .arg(CRATETRACKSTABLE_CRATEID));    // %2
+    query.bindValue(":id", crateId);
+    QList<int> ids;
+    if (!query.exec()) {
+        LOG_FAILED_QUERY(query);
+        return ids;
+    }
+
+    // Put all those track IDs into a list.
+    while (query.next()) {
+        ids.append(query.value(0).toInt());
+    }
+
+    // Return the list to our caller.
+    return ids;
+}
+
+void CrateDAO::getAutoDjCrates(bool trackSource, QMap<QString,int>* pCrateMap) {
+    // Get the name and ID number of every crate in the auto-DJ queue.
+    QSqlQuery query(m_database);
+    query.setForwardOnly(true);
+    // SELECT name, id FROM crates WHERE autodj = 1 ORDER BY name;
+    query.prepare(QString("SELECT %1, %2 FROM " CRATE_TABLE
+            " WHERE %3 = :in ORDER BY %1")
+            .arg(CRATETABLE_NAME, // %1
+                 CRATETABLE_ID, // %2
+                 CRATETABLE_AUTODJ_SOURCE)); // %3
+    query.bindValue(":in", (trackSource) ? 1 : 0);
+    if (!query.exec()) {
+        LOG_FAILED_QUERY(query);
+        return;
+    }
+
+    // Create a map between the crate name and its ID number.
+    while (query.next()) {
+        pCrateMap->insert(query.value(0).toString(), query.value(1).toInt());
+    }
+}
+
+#endif // __AUTODJCRATES__
+
+bool CrateDAO::deleteCrate(const int crateId) {
     ScopedTransaction transaction(m_database);
     QSqlQuery query(m_database);
     query.prepare("DELETE FROM " CRATE_TRACKS_TABLE " WHERE crate_id = :id");
@@ -132,7 +254,7 @@ int CrateDAO::getCrateIdByName(const QString& name) {
     return -1;
 }
 
-int CrateDAO::getCrateId(int position) {
+int CrateDAO::getCrateId(const int position) {
     QSqlQuery query(m_database);
     query.prepare("SELECT id FROM " CRATE_TABLE);
     if (query.exec()) {
@@ -149,7 +271,7 @@ int CrateDAO::getCrateId(int position) {
     return -1;
 }
 
-QString CrateDAO::crateName(int crateId) {
+QString CrateDAO::crateName(const int crateId) {
     QSqlQuery query(m_database);
     query.prepare("SELECT name FROM " CRATE_TABLE " WHERE id = (:id)");
     query.bindValue(":id", crateId);
@@ -163,7 +285,7 @@ QString CrateDAO::crateName(int crateId) {
     return QString();
 }
 
-unsigned int CrateDAO::crateSize(int crateId) {
+unsigned int CrateDAO::crateSize(const int crateId) {
     QSqlQuery query(m_database);
     query.prepare("SELECT COUNT(*) FROM " CRATE_TRACKS_TABLE " WHERE crate_id = (:id)");
     query.bindValue(":id", crateId);
@@ -177,7 +299,7 @@ unsigned int CrateDAO::crateSize(int crateId) {
     return 0;
 }
 
-void CrateDAO::copyCrateTracks(int sourceCrateId, int targetCrateId) {
+void CrateDAO::copyCrateTracks(const int sourceCrateId, const int targetCrateId) {
     // Query Tracks from the source Playlist
     QSqlQuery query(m_database);
     query.prepare("SELECT track_id FROM crate_tracks "
@@ -193,10 +315,10 @@ void CrateDAO::copyCrateTracks(int sourceCrateId, int targetCrateId) {
     while (query.next()) {
         trackIds.append(query.value(0).toInt());
     }
-    addTracksToCrate(trackIds, targetCrateId);
+    addTracksToCrate(targetCrateId, &trackIds);
 }
 
-bool CrateDAO::addTrackToCrate(int trackId, int crateId) {
+bool CrateDAO::addTrackToCrate(const int trackId, const int crateId) {
     QSqlQuery query(m_database);
     query.prepare("INSERT INTO " CRATE_TRACKS_TABLE
                   " (crate_id, track_id) VALUES (:crate_id, :track_id)");
@@ -216,36 +338,36 @@ bool CrateDAO::addTrackToCrate(int trackId, int crateId) {
 }
 
 
-int CrateDAO::addTracksToCrate(QList<int> trackIdList, int crateId) {
+int CrateDAO::addTracksToCrate(const int crateId, QList<int>* trackIdList) {
     ScopedTransaction transaction(m_database);
     QSqlQuery query(m_database);
     query.prepare("INSERT INTO " CRATE_TRACKS_TABLE " (crate_id, track_id) VALUES (:crate_id, :track_id)");
 
-    for (int i = 0; i < trackIdList.size(); ++i) {
+    for (int i = 0; i < trackIdList->size(); ++i) {
         query.bindValue(":crate_id", crateId);
-        query.bindValue(":track_id", trackIdList.at(i));
+        query.bindValue(":track_id", trackIdList->at(i));
         if (!query.exec()) {
             LOG_FAILED_QUERY(query);
             // We must emit only those trackID that were added so we need to
             // remove the failed ones.
-            trackIdList.removeAt(i);
+            trackIdList->removeAt(i);
             --i; // account for reduced size of list
         }
     }
     transaction.commit();
 
     // Emitting the trackAdded signals for each trackID outside the transaction
-    foreach(int trackId, trackIdList) {
+    foreach(int trackId, *trackIdList) {
         emit(trackAdded(crateId, trackId));
     }
 
     emit(changed(crateId));
 
     // Return the number of tracks successfully added
-    return trackIdList.size();
+    return trackIdList->size();
 }
 
-bool CrateDAO::removeTrackFromCrate(int trackId, int crateId) {
+bool CrateDAO::removeTrackFromCrate(const int trackId, const int crateId) {
     QSqlQuery query(m_database);
     query.prepare("DELETE FROM " CRATE_TRACKS_TABLE " WHERE "
                   "crate_id = :crate_id AND track_id = :track_id");
@@ -262,7 +384,7 @@ bool CrateDAO::removeTrackFromCrate(int trackId, int crateId) {
     return true;
 }
 
-bool CrateDAO::removeTracksFromCrate(QList<int> ids, int crateId) {
+bool CrateDAO::removeTracksFromCrate(const QList<int>& ids, const int crateId) {
     QStringList idList;
     foreach (int id, ids) {
         idList << QString::number(id);
@@ -284,7 +406,7 @@ bool CrateDAO::removeTracksFromCrate(QList<int> ids, int crateId) {
     return true;
 }
 
-void CrateDAO::removeTracksFromCrates(QList<int> ids) {
+void CrateDAO::removeTracksFromCrates(const QList<int>& ids) {
     QStringList idList;
     foreach (int id, ids) {
         idList << QString::number(id);
