@@ -17,13 +17,30 @@ LibraryHashDAO::~LibraryHashDAO()
 {
 }
 
-void LibraryHashDAO::initialize()
-{
-    qDebug() << "LibraryHashDAO::initialize" << QThread::currentThread() << m_database.connectionName();
+void LibraryHashDAO::initialize() {
+    qDebug() << "LibraryHashDAO::initialize" << QThread::currentThread()
+             << m_database.connectionName();
 }
 
-int LibraryHashDAO::getDirectoryHash(QString dirPath)
-{
+QHash<QString, int> LibraryHashDAO::getDirectoryHashes() {
+    QSqlQuery query(m_database);
+    query.prepare("SELECT hash, directory_path FROM LibraryHashes");
+    QHash<QString, int> hashes;
+    if (!query.exec()) {
+        LOG_FAILED_QUERY(query);
+    }
+
+    int hashColumn = query.record().indexOf("hash");
+    int directoryPathColumn = query.record().indexOf("directory_path");
+    while (query.next()) {
+        hashes[query.value(directoryPathColumn).toString()] =
+                query.value(hashColumn).toInt();
+    }
+
+    return hashes;
+}
+
+int LibraryHashDAO::getDirectoryHash(const QString& dirPath) {
     //qDebug() << "LibraryHashDAO::getDirectoryHash" << QThread::currentThread() << m_database.connectionName();
     int hash = -1;
 
@@ -33,22 +50,20 @@ int LibraryHashDAO::getDirectoryHash(QString dirPath)
     query.bindValue(":directory_path", dirPath);
 
     if (!query.exec()) {
-        qDebug() << "SELECT hash failed:" << query.lastError();
+        LOG_FAILED_QUERY(query) << "SELECT hash failed:";
     }
     //Grab a hash for this directory from the database, from the last time it was scanned.
     if (query.next()) {
         hash = query.value(query.record().indexOf("hash")).toInt();
         //qDebug() << "prev hash exists" << hash << dirPath;
-    }
-    else {
+    } else {
         //qDebug() << "prev hash does not exist" << dirPath;
     }
 
     return hash;
 }
 
-void LibraryHashDAO::saveDirectoryHash(QString dirPath, int hash)
-{
+void LibraryHashDAO::saveDirectoryHash(const QString& dirPath, const int hash) {
     //qDebug() << "LibraryHashDAO::saveDirectoryHash" << QThread::currentThread() << m_database.connectionName();
     QSqlQuery query(m_database);
     query.prepare("INSERT INTO LibraryHashes (directory_path, hash, directory_deleted) "
@@ -57,26 +72,29 @@ void LibraryHashDAO::saveDirectoryHash(QString dirPath, int hash)
     query.bindValue(":hash", hash);
     query.bindValue(":directory_deleted", 0);
 
-
     if (!query.exec()) {
-        qDebug() << "Creating new dirhash failed:" << query.lastError();
+        LOG_FAILED_QUERY(query) << "Creating new dirhash failed.";
     }
     //qDebug() << "created new hash" << hash;
 }
 
-void LibraryHashDAO::updateDirectoryHash(QString dirPath, int newHash, int dir_deleted)
-{
+void LibraryHashDAO::updateDirectoryHash(const QString& dirPath,
+                                         const int newHash,
+                                         const int dir_deleted) {
     //qDebug() << "LibraryHashDAO::updateDirectoryHash" << QThread::currentThread() << m_database.connectionName();
     QSqlQuery query(m_database);
+    // By definition if we have calculated a new hash for a directory then it
+    // exists and no longer needs verification.
     query.prepare("UPDATE LibraryHashes "
-            "SET hash=:hash, directory_deleted=:directory_deleted "
+            "SET hash=:hash, directory_deleted=:directory_deleted, "
+            "needs_verification=0 "
             "WHERE directory_path=:directory_path");
     query.bindValue(":hash", newHash);
     query.bindValue(":directory_deleted", dir_deleted);
     query.bindValue(":directory_path", dirPath);
 
     if (!query.exec()) {
-        qDebug() << "Updating existing dirhash failed:" << query.lastError();
+        LOG_FAILED_QUERY(query) << "Updating existing dirhash failed.";
     }
     //qDebug() << "updated old existing hash" << newHash << dirPath << dir_deleted;
 
@@ -84,13 +102,12 @@ void LibraryHashDAO::updateDirectoryHash(QString dirPath, int newHash, int dir_d
     //qDebug() << getDirectoryHash(dirPath);
 }
 
-void LibraryHashDAO::updateDirectoryStatuses(QStringList dirPaths, bool deleted, bool verified) {
+void LibraryHashDAO::updateDirectoryStatuses(const QStringList& dirPaths,
+                                             const bool deleted,
+                                             const bool verified) {
     //qDebug() << "LibraryHashDAO::updateDirectoryStatus" << QThread::currentThread() << m_database.connectionName();
     FieldEscaper escaper(m_database);
-    QMutableStringListIterator it(dirPaths);
-    while (it.hasNext()) {
-        it.setValue(escaper.escapeString(it.next()));
-    }
+    QStringList escapedDirPaths = escaper.escapeStrings(dirPaths);
 
     QSqlQuery query(m_database);
     query.prepare(
@@ -98,7 +115,7 @@ void LibraryHashDAO::updateDirectoryStatuses(QStringList dirPaths, bool deleted,
                 "SET directory_deleted=:directory_deleted, "
                 "needs_verification=:needs_verification "
                 "WHERE directory_path IN (%1)")
-        .arg(dirPaths.join(",")));
+        .arg(escapedDirPaths.join(",")));
     query.bindValue(":directory_deleted", deleted ? 1 : 0);
     query.bindValue(":needs_verification", !verified ? 1 : 0);
     if (!query.exec()) {
@@ -106,8 +123,7 @@ void LibraryHashDAO::updateDirectoryStatuses(QStringList dirPaths, bool deleted,
     }
 }
 
-void LibraryHashDAO::markAsExisting(QString dirPath)
-{
+void LibraryHashDAO::markAsExisting(const QString& dirPath) {
     //qDebug() << "LibraryHashDAO::markExisting" << QThread::currentThread() << m_database.connectionName();
     QSqlQuery query(m_database);
     query.prepare("UPDATE LibraryHashes "
@@ -116,39 +132,23 @@ void LibraryHashDAO::markAsExisting(QString dirPath)
     query.bindValue(":directory_deleted", 0);
     query.bindValue(":directory_path", dirPath);
     if (!query.exec()) {
-        qDebug() << "Updating dirhash to mark as existing failed:" << query.lastError();
+        LOG_FAILED_QUERY(query) << "Updating dirhash to mark as existing failed.";
     }
 }
 
-void LibraryHashDAO::markAsVerified(QString dirPath)
-{
-    //qDebug() << "LibraryHashDAO::markExisting" << QThread::currentThread() << m_database.connectionName();
-    QSqlQuery query(m_database);
-    query.prepare("UPDATE LibraryHashes "
-                  "SET needs_verification=0 "
-                  "WHERE directory_path=:directory_path");
-   // query.bindValue(":directory_deleted", 0);
-    query.bindValue(":directory_path", dirPath);
-    if (!query.exec()) {
-        qDebug() << "Updating dirhash to mark as verified failed:" << query.lastError();
-    }
-}
-
-void LibraryHashDAO::invalidateAllDirectories()
-{
+void LibraryHashDAO::invalidateAllDirectories() {
     //qDebug() << "LibraryHashDAO::invalidateAllDirectories"
     //<< QThread::currentThread() << m_database.connectionName();
     QSqlQuery query(m_database);
     query.prepare("UPDATE LibraryHashes "
-                  "SET needs_verification=:needs_verification");
-    query.bindValue(":needs_verification", 1);
+                  "SET needs_verification=1");
     if (!query.exec()) {
-        qDebug() << query.lastError();
+        LOG_FAILED_QUERY(query)
+                << "Couldn't mark directories previously hashed as needing verification.";
     }
 }
 
-void LibraryHashDAO::markUnverifiedDirectoriesAsDeleted()
-{
+void LibraryHashDAO::markUnverifiedDirectoriesAsDeleted() {
     //qDebug() << "LibraryHashDAO::markUnverifiedDirectoriesAsDeleted"
     //<< QThread::currentThread() << m_database.connectionName();
     QSqlQuery query(m_database);
@@ -157,17 +157,32 @@ void LibraryHashDAO::markUnverifiedDirectoriesAsDeleted()
                   "WHERE needs_verification=1");
     query.bindValue(":directory_deleted", 1);
     if (!query.exec()) {
-        qDebug() << query.lastError();
+        LOG_FAILED_QUERY(query);
     }
 }
 
-void LibraryHashDAO::removeDeletedDirectoryHashes()
-{
+void LibraryHashDAO::removeDeletedDirectoryHashes() {
     QSqlQuery query(m_database);
     query.prepare("DELETE FROM LibraryHashes WHERE "
                "directory_deleted=:directory_deleted");
     query.bindValue(":directory_deleted", 1);
     if (!query.exec()) {
-        qDebug() << query.lastError();
+        LOG_FAILED_QUERY(query);
     }
 }
+
+QStringList LibraryHashDAO::getDeletedDirectories() {
+    QStringList result;
+    QSqlQuery query(m_database);
+    query.prepare("SELECT directory_path FROM LibraryHashes "
+                  "WHERE directory_deleted=1");
+    if (!query.exec()) {
+        LOG_FAILED_QUERY(query);
+    }
+    const int directoryPathColumn = query.record().indexOf("directory_path");
+    while (query.next()) {
+        QString directory = query.value(directoryPathColumn).toString();
+        result << directory;
+    }
+    return result;
+}
