@@ -10,11 +10,14 @@
 #include <QList>
 #include <QObject>
 #include <QAbstractItemModel>
+#include <QFont>
 
 #include "configobject.h"
 #include "trackinfoobject.h"
 #include "recording/recordingmanager.h"
-#include "preparefeature.h"
+#include "analysisfeature.h"
+#include "library/coverartcache.h"
+#include "library/setlogfeature.h"
 
 class TrackModel;
 class TrackCollection;
@@ -29,13 +32,15 @@ class PlaylistFeature;
 class CrateFeature;
 class LibraryControl;
 class MixxxKeyboard;
+class PlayerManagerInterface;
 
 class Library : public QObject {
     Q_OBJECT
 public:
     Library(QObject* parent,
             ConfigObject<ConfigValue>* pConfig,
-            bool firstRun, RecordingManager* pRecordingManager);
+            PlayerManagerInterface* pPlayerManager,
+            RecordingManager* pRecordingManager);
     virtual ~Library();
 
     void bindWidget(WLibrary* libraryWidget,
@@ -43,7 +48,7 @@ public:
     void bindSidebarWidget(WLibrarySidebar* sidebarWidget);
 
     void addFeature(LibraryFeature* feature);
-    QList<TrackPointer> getTracksToAutoLoad();
+    QStringList getDirs();
 
     // TODO(rryan) Transitionary only -- the only reason this is here is so the
     // waveform widgets can signal to a player to load a track. This can be
@@ -53,8 +58,24 @@ public:
         return m_pTrackCollection;
     }
 
+    inline int getTrackTableRowHeight() const {
+        return m_iTrackTableRowHeight;
+    }
+
+    inline const QFont& getTrackTableFont() const {
+        return m_trackTableFont;
+    }
+
     //static Library* buildDefaultLibrary();
 
+    enum RemovalType {
+        LeaveTracksUnchanged = 0,
+        HideTracks,
+        PurgeTracks
+    };
+
+    static const int kDefaultRowHeightPx;
+
   public slots:
     void slotShowTrackModel(QAbstractItemModel* model);
     void slotSwitchToView(const QString& view);
@@ -65,7 +86,12 @@ public:
     void slotRefreshLibraryModels();
     void slotCreatePlaylist();
     void slotCreateCrate();
+    void slotRequestAddDir(QString directory);
+    void slotRequestRemoveDir(QString directory, Library::RemovalType removalType);
+    void slotRequestRelocateDir(QString previousDirectory, QString newDirectory);
     void onSkinLoadFinished();
+    void slotSetTrackTableFont(const QFont& font);
+    void slotSetTrackTableRowHeight(int rowHeight);
 
   signals:
     void showTrackModel(QAbstractItemModel* model);
@@ -76,6 +102,12 @@ public:
     void search(const QString& text);
     void searchCleared();
     void searchStarting();
+    // emit this signal to enable/disable the cover art widget
+    void enableCoverArtDisplay(bool);
+    void trackSelected(TrackPointer pTrack);
+
+    void setTrackTableFont(const QFont& font);
+    void setTrackTableRowHeight(int rowHeight);
 
   private:
     ConfigObject<ConfigValue>* m_pConfig;
@@ -87,13 +119,11 @@ public:
     MixxxLibraryFeature* m_pMixxxLibraryFeature;
     PlaylistFeature* m_pPlaylistFeature;
     CrateFeature* m_pCrateFeature;
-#ifdef __PROMO__
-    class PromoTracksFeature;
-    PromoTracksFeature* m_pPromoTracksFeature;
-#endif
-    PrepareFeature* m_pPrepareFeature;
+    AnalysisFeature* m_pAnalysisFeature;
     LibraryControl* m_pLibraryControl;
     RecordingManager* m_pRecordingManager;
+    QFont m_trackTableFont;
+    int m_iTrackTableRowHeight;
 };
 
 #endif /* LIBRARY_H */
