@@ -5,15 +5,23 @@
 #define MIXXXLIBRARYFEATURE_H
 
 #include <QStringListModel>
+#include <QUrl>
+#include <QVariant>
+#include <QIcon>
+#include <QModelIndex>
+#include <QList>
+#include <QString>
+#include <QSharedPointer>
+#include <QObject>
 
 #include "library/libraryfeature.h"
 #include "library/dao/trackdao.h"
 #include "treeitemmodel.h"
 #include "configobject.h"
-#include "dlghidden.h"
-#include "dlgmissing.h"
-
 
+class DlgHidden;
+class DlgMissing;
+class Library;
 class BaseTrackCache;
 class LibraryTableModel;
 class TrackCollection;
@@ -21,14 +29,14 @@ class TrackCollection;
 class MixxxLibraryFeature : public LibraryFeature {
     Q_OBJECT
     public:
-    MixxxLibraryFeature(QObject* parent,
+    MixxxLibraryFeature(Library* pLibrary,
                         TrackCollection* pTrackCollection,
                         ConfigObject<ConfigValue>* pConfig);
     virtual ~MixxxLibraryFeature();
 
     QVariant title();
     QIcon getIcon();
-    bool dropAccept(QList<QUrl> urls, QWidget *pSource);
+    bool dropAccept(QList<QUrl> urls, QObject* pSource);
     bool dragMoveAccept(QUrl url);
     TreeItemModel* getChildModel();
     void bindWidget(WLibrary* pLibrary,
@@ -42,6 +50,7 @@ class MixxxLibraryFeature : public LibraryFeature {
   private:
     const QString kMissingTitle;
     const QString kHiddenTitle;
+    Library* m_pLibrary;
     QSharedPointer<BaseTrackCache> m_pBaseTrackCache;
     LibraryTableModel* m_pLibraryTableModel;
     DlgMissing* m_pMissingView;
