@@ -14,15 +14,16 @@
 #include "library/proxytrackmodel.h"
 #include "recording/recordingmanager.h"
 
+class Library;
 class TrackCollection;
 
 class RecordingFeature : public LibraryFeature {
     Q_OBJECT
   public:
-    RecordingFeature(QObject* parent,
-                  ConfigObject<ConfigValue>* pConfig,
-                  TrackCollection* pTrackCollection,
-                  RecordingManager* pRecordingManager);
+    RecordingFeature(Library* parent,
+                     ConfigObject<ConfigValue>* pConfig,
+                     TrackCollection* pTrackCollection,
+                     RecordingManager* pRecordingManager);
     virtual ~RecordingFeature();
 
     QVariant title();
@@ -43,6 +44,7 @@ class RecordingFeature : public LibraryFeature {
 
   private:
     ConfigObject<ConfigValue>* m_pConfig;
+    Library* m_pLibrary;
     TrackCollection* m_pTrackCollection;
     FolderTreeModel m_childModel;
     const static QString m_sRecordingViewName;
