@@ -1,28 +1,30 @@
 // schemamanager.cpp
 // Created 12/29/2009 by RJ Ryan (rryan@mit.edu)
 
-#include <QtCore>
 #include <QtDebug>
 
 #include "library/schemamanager.h"
 #include "library/queryutil.h"
-#include "xmlparse.h"
+#include "util/xml.h"
+#include "util/assert.h"
 
 const QString SchemaManager::SETTINGS_VERSION_STRING = "mixxx.schema.version";
 const QString SchemaManager::SETTINGS_MINCOMPATIBLE_STRING = "mixxx.schema.min_compatible_version";
 
 // static
-int SchemaManager::upgradeToSchemaVersion(const QString& schemaFilename,
-                                           QSqlDatabase& db, int targetVersion) {
-
+SchemaManager::Result SchemaManager::upgradeToSchemaVersion(
+        const QString& schemaFilename,
+        QSqlDatabase& db, const int targetVersion) {
     SettingsDAO settings(db);
     int currentVersion = getCurrentSchemaVersion(settings);
-    Q_ASSERT(currentVersion >= 0);
+    DEBUG_ASSERT_AND_HANDLE(currentVersion >= 0) {
+        return RESULT_UPGRADE_FAILED;
+    }
 
     if (currentVersion == targetVersion) {
         qDebug() << "SchemaManager::upgradeToSchemaVersion already at version"
                  << targetVersion;
-        return 0;
+        return RESULT_OK;
     } else if (currentVersion < targetVersion) {
         qDebug() << "SchemaManager::upgradeToSchemaVersion upgrading"
                  << targetVersion-currentVersion << "versions to version"
@@ -35,7 +37,9 @@ int SchemaManager::upgradeToSchemaVersion(const QString& schemaFilename,
 
         if (isBackwardsCompatible(settings, currentVersion, targetVersion)) {
             qDebug() << "Current schema version is backwards-compatible with" << targetVersion;
-            return 0;
+            return RESULT_OK;
+        } else {
+            return RESULT_BACKWARDS_INCOMPATIBLE;
         }
     }
 
@@ -44,7 +48,7 @@ int SchemaManager::upgradeToSchemaVersion(const QString& schemaFilename,
 
     if (schemaRoot.isNull()) {
         // Error parsing xml file
-        return -3;
+        return RESULT_SCHEMA_ERROR;
     }
 
     QDomNodeList revisions = schemaRoot.childNodes();
@@ -54,30 +58,27 @@ int SchemaManager::upgradeToSchemaVersion(const QString& schemaFilename,
     for (int i = 0; i < revisions.count(); i++) {
         QDomElement revision = revisions.at(i).toElement();
         QString version = revision.attribute("version");
-        Q_ASSERT(!version.isNull());
+        DEBUG_ASSERT_AND_HANDLE(!version.isNull()) {
+            // xml file is not valid
+            return RESULT_SCHEMA_ERROR;
+        }
         int iVersion = version.toInt();
         revisionMap[iVersion] = revision;
     }
 
-    int success = 0;
-
-    while (currentVersion != targetVersion) {
-        int thisTarget;
-        if (currentVersion > targetVersion) {
-            thisTarget = currentVersion - 1;
-            qDebug() << "Downgrade not yet supported.";
-            success = -1;
-            break;
-        } else {
-            thisTarget = currentVersion + 1;
-        }
+    // The checks above guarantee that currentVersion < targetVersion when we
+    // get here.
+    while (currentVersion < targetVersion) {
+        int thisTarget = currentVersion + 1;
 
+        // Now that we bake the schema.xml into the binary it is a programming
+        // error if we include a schema.xml that does not have information on
+        // how to get all the way to targetVersion.
         if (!revisionMap.contains(thisTarget)) {
             qDebug() << "SchemaManager::upgradeToSchemaVersion"
                      << "Don't know how to get to"
                      << thisTarget << "from" << currentVersion;
-            success = -1;
-            break;
+            return RESULT_SCHEMA_ERROR;
         }
 
         QDomElement revision = revisionMap[thisTarget];
@@ -91,12 +92,14 @@ int SchemaManager::upgradeToSchemaVersion(const QString& schemaFilename,
             minCompatibleVersion = QString::number(thisTarget);
         }
 
-        Q_ASSERT(!eDescription.isNull() && !eSql.isNull());
+        DEBUG_ASSERT_AND_HANDLE(!eSql.isNull()) {
+            // xml file is not valid
+            return RESULT_SCHEMA_ERROR;
+        }
 
         QString description = eDescription.text();
         QString sql = eSql.text();
 
-
         qDebug() << "Applying version" << thisTarget << ":"
                  << description.trimmed();
 
@@ -129,15 +132,13 @@ int SchemaManager::upgradeToSchemaVersion(const QString& schemaFilename,
             settings.setValue(SETTINGS_MINCOMPATIBLE_STRING, minCompatibleVersion);
             transaction.commit();
         } else {
-            success = -2;
             qDebug() << "Failed to move from version" << currentVersion
                      << "to version" << thisTarget;
             transaction.rollback();
-            break;
+            return RESULT_UPGRADE_FAILED;
         }
     }
-
-    return success;
+    return RESULT_OK;
 }
 
 // static
@@ -158,12 +159,13 @@ bool SchemaManager::isBackwardsCompatible(SettingsDAO& settings,
                                           int targetVersion) {
     QString backwardsCompatibleVersion =
             settings.getValue(SETTINGS_MINCOMPATIBLE_STRING);
-    int iBackwardsCompatibleVersion = -1;
+    bool ok = false;
+    int iBackwardsCompatibleVersion = backwardsCompatibleVersion.toInt(&ok);
 
     // If the current backwards compatible schema version is not stored in the
     // settings table, assume the current schema version is only backwards
     // compatible with itself.
-    if (backwardsCompatibleVersion.isNull()) {
+    if (backwardsCompatibleVersion.isNull() || !ok) {
         // rryan 11/2010 We just added the backwards compatible flags, and some
         // people using the Mixxx trunk are already on schema version 7. This
         // special case is for them. Schema version 7 is backwards compatible
