@@ -11,16 +11,24 @@
 
 class SchemaManager {
   public:
-    static int upgradeToSchemaVersion(const QString& schemaFilename,
-                                       QSqlDatabase& db, int targetVersion);
+    enum Result {
+        RESULT_OK,
+        RESULT_BACKWARDS_INCOMPATIBLE,
+        RESULT_UPGRADE_FAILED,
+        RESULT_SCHEMA_ERROR
+    };
+
+    static Result upgradeToSchemaVersion(const QString& schemaFilename,
+                                         QSqlDatabase& db, const int targetVersion);
+
+    static const QString SETTINGS_VERSION_STRING;
+    static const QString SETTINGS_MINCOMPATIBLE_STRING;
+
   private:
     static bool isBackwardsCompatible(SettingsDAO& settings,
                                       int currentVersion,
                                       int targetVersion);
     static int getCurrentSchemaVersion(SettingsDAO& settings);
-
-    static const QString SETTINGS_VERSION_STRING;
-    static const QString SETTINGS_MINCOMPATIBLE_STRING;
 };
 
 #endif /* SCHEMAMANAGER_H */
