@@ -9,7 +9,6 @@
 
 #include "library/baseplaylistfeature.h"
 #include "configobject.h"
-#include "controlobjectthreadmain.h"
 
 class TrackCollection;
 class TreeItem;
@@ -31,22 +30,25 @@ public:
     void onRightClick(const QPoint& globalPos);
     void onRightClickChild(const QPoint& globalPos, QModelIndex index);
     void slotJoinWithPrevious();
+    void slotGetNewPlaylist();
 
   protected:
     void buildPlaylistList();
     void decorateChild(TreeItem *pChild, int playlist_id);
 
   private slots:
-    void slotPlayingDeckChanged(int deck);
+    void slotPlayingTrackChanged(TrackPointer currentPlayingTrack);
     void slotPlaylistTableChanged(int playlistId);
-
+    void slotPlaylistContentChanged(int playlistId);
+    void slotPlaylistTableRenamed(int playlistId, QString a_strName);
 
   private:
-    virtual QString getRootViewHtml() const;
+    QString getRootViewHtml() const;
 
     QLinkedList<int> m_recentTracks;
-    QAction *m_pJoinWithPreviousAction;
+    QAction* m_pJoinWithPreviousAction;
+    QAction* m_pGetNewPlaylist;
     int m_playlistId;
 };
 
-#endif /* SETLOGFEATURE_H */
+#endif // SETLOGFEATURE_H
