@@ -13,8 +13,11 @@
     display track lists. */
 class TrackModel {
   public:
+    static const int kHeaderWidthRole = Qt::UserRole + 0;
+    static const int kHeaderNameRole = Qt::UserRole + 1;
+
     TrackModel(QSqlDatabase db,
-               QString settingsNamespace)
+               const char* settingsNamespace)
             : m_db(db),
               m_settingsNamespace(settingsNamespace),
               m_iDefaultSortColumn(-1),
@@ -37,15 +40,13 @@ class TrackModel {
         TRACKMODELCAPS_LOADTOSAMPLER     = 0x00100,
         TRACKMODELCAPS_LOADTOPREVIEWDECK = 0x00200,
         TRACKMODELCAPS_REMOVE            = 0x00400,
-        TRACKMODELCAPS_RELOCATE          = 0x00800,
-        TRACKMODELCAPS_BPMLOCK           = 0x01000,
-        TRACKMODELCAPS_CLEAR_BEATS       = 0x02000,
-        TRACKMODELCAPS_RESETPLAYED       = 0x04000,
-        TRACKMODELCAPS_HIDE              = 0x08000,
-        TRACKMODELCAPS_UNHIDE            = 0x10000,
-        TRACKMODELCAPS_PURGE             = 0x20000
+        TRACKMODELCAPS_MANIPULATEBEATS   = 0x00800,
+        TRACKMODELCAPS_CLEAR_BEATS       = 0x01000,
+        TRACKMODELCAPS_RESETPLAYED       = 0x02000,
+        TRACKMODELCAPS_HIDE              = 0x04000,
+        TRACKMODELCAPS_UNHIDE            = 0x08000,
+        TRACKMODELCAPS_PURGE             = 0x10000
     };
-
     typedef int CapabilitiesFlags; /** Enables us to do ORing */
 
     // Deserialize and return the track at the given QModelIndex in this result
@@ -63,7 +64,7 @@ class TrackModel {
     virtual const QLinkedList<int> getTrackRows(int trackId) const = 0;
 
     bool isTrackModel() { return true;}
-    virtual void search(const QString& searchText) = 0;
+    virtual void search(const QString& searchText, const QString& extraFilter=QString()) = 0;
     virtual const QString currentSearch() const = 0;
     virtual bool isColumnInternal(int column) = 0;
     // if no header state exists, we may hide some columns so that the user can
@@ -71,9 +72,7 @@ class TrackModel {
     virtual bool isColumnHiddenByDefault(int column) = 0;
     virtual const QList<int>& showableColumns() const { return m_emptyColumns; }
     virtual const QList<int>& searchColumns() const { return m_emptyColumns; }
-    virtual void removeTrack(const QModelIndex& index) {
-        Q_UNUSED(index);
-    }
+
     virtual void removeTracks(const QModelIndexList& indices) {
         Q_UNUSED(indices);
     }
@@ -86,12 +85,7 @@ class TrackModel {
     virtual void purgeTracks(const QModelIndexList& indices) {
         Q_UNUSED(indices);
     }
-    virtual bool addTrack(const QModelIndex& index, QString location) {
-        Q_UNUSED(index);
-        Q_UNUSED(location);
-        return false;
-    }
-    virtual int addTracks(const QModelIndex& index, QList<QString> locations) {
+    virtual int addTracks(const QModelIndex& index, const QList<QString>& locations) {
         Q_UNUSED(index);
         Q_UNUSED(locations);
         return 0;
@@ -101,6 +95,9 @@ class TrackModel {
         Q_UNUSED(sourceIndex);
         Q_UNUSED(destIndex);
     }
+    virtual bool isLocked() {
+        return false;
+    }
     virtual QAbstractItemDelegate* delegateForColumn(const int i, QObject* pParent) {
         Q_UNUSED(i);
         Q_UNUSED(pParent);
@@ -134,9 +131,17 @@ class TrackModel {
         m_eDefaultSortOrder = sortOrder;
     }
 
+    virtual bool isColumnSortable(int column) {
+        Q_UNUSED(column);
+        return true;
+    }
+
     virtual int fieldIndex(const QString& fieldName) const {
         Q_UNUSED(fieldName);
-        return 0;
+        return -1;
+    }
+
+    virtual void select() {
     }
 
   private:
