@@ -15,38 +15,39 @@
 #include "library/queryutil.h"
 #include "library/trackcollection.h"
 #include "library/treeitem.h"
+#include "util/sandbox.h"
 
 TraktorTrackModel::TraktorTrackModel(QObject* parent,
-                                     TrackCollection* pTrackCollection)
+                                     TrackCollection* pTrackCollection,
+                                     QSharedPointer<BaseTrackCache> trackSource)
         : BaseExternalTrackModel(parent, pTrackCollection,
                                  "mixxx.db.model.traktor_tablemodel",
                                  "traktor_library",
-                                 "traktor") {
+                                 trackSource) {
 }
 
 bool TraktorTrackModel::isColumnHiddenByDefault(int column) {
-    if (column == fieldIndex(LIBRARYTABLE_KEY) ||
-        column == fieldIndex(LIBRARYTABLE_BITRATE)) {
+    if (column == fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_BITRATE)) {
         return true;
     }
-    return false;
+    return BaseSqlTableModel::isColumnHiddenByDefault(column);
 }
 
 TraktorPlaylistModel::TraktorPlaylistModel(QObject* parent,
-                                           TrackCollection* pTrackCollection)
+                                           TrackCollection* pTrackCollection,
+                                           QSharedPointer<BaseTrackCache> trackSource)
         : BaseExternalPlaylistModel(parent, pTrackCollection,
                                     "mixxx.db.model.traktor.playlistmodel",
                                     "traktor_playlists",
                                     "traktor_playlist_tracks",
-                                    "traktor") {
+                                    trackSource) {
 }
 
 bool TraktorPlaylistModel::isColumnHiddenByDefault(int column) {
-    if (column == fieldIndex(LIBRARYTABLE_KEY) ||
-        column == fieldIndex(LIBRARYTABLE_BITRATE)) {
+    if (column == fieldIndex(ColumnCache::COLUMN_LIBRARYTABLE_BITRATE)) {
         return true;
     }
-    return false;
+    return BaseSqlTableModel::isColumnHiddenByDefault(column);
 }
 
 TraktorFeature::TraktorFeature(QObject* parent, TrackCollection* pTrackCollection)
@@ -70,13 +71,21 @@ TraktorFeature::TraktorFeature(QObject* parent, TrackCollection* pTrackCollectio
             << "bitrate"
             << "bpm"
             << "key";
-    pTrackCollection->addTrackSource(QString("traktor"), QSharedPointer<BaseTrackCache>(
-        new BaseTrackCache(m_pTrackCollection, tableName, idColumn,
-                           columns, false)));
+    m_trackSource = QSharedPointer<BaseTrackCache>(
+            new BaseTrackCache(m_pTrackCollection, tableName, idColumn,
+                           columns, false));
+    QStringList searchColumns;
+    searchColumns << "artist"
+                  << "album"
+                  << "location"
+                  << "comment"
+                  << "title"
+                  << "genre";
+    m_trackSource->setSearchColumns(searchColumns);
 
     m_isActivated = false;
-    m_pTraktorTableModel = new TraktorTrackModel(this, m_pTrackCollection);
-    m_pTraktorPlaylistModel = new TraktorPlaylistModel(this, m_pTrackCollection);
+    m_pTraktorTableModel = new TraktorTrackModel(this, m_pTrackCollection, m_trackSource);
+    m_pTraktorPlaylistModel = new TraktorPlaylistModel(this, m_pTrackCollection, m_trackSource);
 
     m_title = tr("Traktor");
 
@@ -101,7 +110,7 @@ TraktorFeature::~TraktorFeature() {
 }
 
 BaseSqlTableModel* TraktorFeature::getPlaylistModelForPlaylist(QString playlist) {
-    TraktorPlaylistModel* pModel = new TraktorPlaylistModel(this, m_pTrackCollection);
+    TraktorPlaylistModel* pModel = new TraktorPlaylistModel(this, m_pTrackCollection, m_trackSource);
     pModel->setPlaylist(playlist);
     return pModel;
 }
@@ -145,10 +154,11 @@ void TraktorFeature::activate() {
         m_future_watcher.setFuture(m_future);
         m_title = tr("(loading) Traktor");
         //calls a slot in the sidebar model such that 'iTunes (isLoading)' is displayed.
-        emit (featureIsLoading(this));
+        emit (featureIsLoading(this, true));
     }
 
     emit(showTrackModel(m_pTraktorTableModel));
+    emit(enableCoverArtDisplay(false));
 }
 
 void TraktorFeature::activateChild(const QModelIndex& index) {
@@ -162,13 +172,14 @@ void TraktorFeature::activateChild(const QModelIndex& index) {
         qDebug() << "Activate Traktor Playlist: " << item->dataPath().toString();
         m_pTraktorPlaylistModel->setPlaylist(item->dataPath().toString());
         emit(showTrackModel(m_pTraktorPlaylistModel));
+        emit(enableCoverArtDisplay(false));
     }
 }
 
 TreeItem* TraktorFeature::importLibrary(QString file) {
     //Give thread a low priority
     QThread* thisThread = QThread::currentThread();
-    thisThread->setPriority(QThread::LowestPriority);
+    thisThread->setPriority(QThread::LowPriority);
     //Invisible root item of Traktor's child model
     TreeItem* root = NULL;
     //Delete all table entries of Traktor feature
@@ -194,8 +205,6 @@ TreeItem* TraktorFeature::importLibrary(QString file) {
     }
     QXmlStreamReader xml(&traktor_file);
     bool inCollectionTag = false;
-    //TODO(XXX) is this still needed to parse the library correctly?
-    bool inEntryTag = false;
     bool inPlaylistsTag = false;
     bool isRootFolderParsed = false;
     int nAudioFiles = 0;
@@ -207,8 +216,7 @@ TreeItem* TraktorFeature::importLibrary(QString file) {
                 inCollectionTag = true;
             }
             // Each "ENTRY" tag in <COLLECTION> represents a track
-            if (inCollectionTag && xml.name() == "ENTRY" ) {
-                inEntryTag = true;
+            if (inCollectionTag && xml.name() == "ENTRY") {
                 //parse track
                 parseTrack(xml, query);
                 ++nAudioFiles; //increment number of files in the music collection
@@ -231,9 +239,6 @@ TreeItem* TraktorFeature::importLibrary(QString file) {
             if (xml.name() == "COLLECTION") {
                 inCollectionTag = false;
             }
-            if (xml.name() == "ENTRY" && inCollectionTag) {
-                inEntryTag = false;
-            }
             if (xml.name() == "PLAYLISTS" && inPlaylistsTag) {
                 inPlaylistsTag = false;
             }
@@ -380,8 +385,6 @@ TreeItem* TraktorFeature::parsePlaylists(QXmlStreamReader &xml) {
     TreeItem *rootItem = new TreeItem();
     TreeItem * parent = rootItem;
 
-    bool inPlaylistTag = false;
-
     QSqlQuery query_insert_to_playlists(m_database);
     query_insert_to_playlists.prepare("INSERT INTO traktor_playlists (name) "
                   "VALUES (:name)");
@@ -410,8 +413,7 @@ TreeItem* TraktorFeature::parsePlaylists(QXmlStreamReader &xml) {
                     TreeItem * item = new TreeItem(name,current_path, this, parent);
                     parent->appendChild(item);
                     parent = item;
-               }
-               if (type == "PLAYLIST") {
+               } else if (type == "PLAYLIST") {
                     current_path += delimiter;
                     current_path += name;
                     //qDebug() << "Playlist: " +current_path << " has parent " << parent->data().toString();
@@ -425,8 +427,6 @@ TreeItem* TraktorFeature::parsePlaylists(QXmlStreamReader &xml) {
                                          query_insert_to_playlist_tracks);
                 }
             }
-            if (xml.name() == "ENTRY" && inPlaylistTag) {
-            }
         }
 
         if (xml.isEndElement()) {
@@ -441,9 +441,6 @@ TreeItem* TraktorFeature::parsePlaylists(QXmlStreamReader &xml) {
 
                 current_path.remove(lastSlash, path_length - lastSlash);
             }
-            if (xml.name() == "PLAYLIST") {
-                inPlaylistTag = false;
-            }
             //We leave the infinte loop, if twe have the closing "PLAYLIST" tag
             if (xml.name() == "PLAYLISTS") {
                 break;
@@ -482,8 +479,9 @@ void TraktorFeature::parsePlaylistEntries(
 
     //playlist_id = id_query.lastInsertId().toInt();
     int playlist_id = -1;
+    const int idColumn = id_query.record().indexOf("id");
     while (id_query.next()) {
-        playlist_id = id_query.value(id_query.record().indexOf ("id")).toInt();
+        playlist_id = id_query.value(idColumn).toInt();
     }
 
     int playlist_position = 1;
@@ -516,7 +514,7 @@ void TraktorFeature::parsePlaylistEntries(
                     }
 
                     if (finder_query.next()) {
-                        track_id = finder_query.value(finder_query.record().indexOf ("id")).toInt();
+                        track_id = finder_query.value(finder_query.record().indexOf("id")).toInt();
                     }
 
                     query_insert_into_playlisttracks.bindValue(":playlist_id", playlist_id);
@@ -547,7 +545,7 @@ void TraktorFeature::clearTable(QString table_name) {
         qDebug() << "Could not delete remove old entries from table "
                  << table_name << " : " << query.lastError();
     else
-        qDebug() << "Traktor table entries of '" << table_name <<"' have been cleared.";
+        qDebug() << "Traktor table entries of '" << table_name << "' have been cleared.";
 }
 
 QString TraktorFeature::getTraktorMusicDatabase() {
@@ -561,7 +559,13 @@ QString TraktorFeature::getTraktorMusicDatabase() {
     //Let's try to detect the latest Traktor version and its collection.nml
     QString myDocuments = QDesktopServices::storageLocation(QDesktopServices::DocumentsLocation);
     QDir ni_directory(myDocuments +"/Native Instruments/");
-    ni_directory.setFilter(QDir::Dirs | QDir::NoDotAndDotDot | QDir::NoSymLinks) ;
+    ni_directory.setFilter(QDir::Dirs | QDir::NoDotAndDotDot | QDir::NoSymLinks);
+
+    // We may not have access to this directory since it is in the user's
+    // Documents folder. Ask for access if we don't have it.
+    if (ni_directory.exists()) {
+        Sandbox::askForAccess(ni_directory.canonicalPath());
+    }
 
     //Iterate over the subfolders
     QFileInfoList list = ni_directory.entryInfoList();
@@ -600,8 +604,8 @@ void TraktorFeature::onTrackCollectionLoaded() {
     TreeItem* root = m_future.result();
     if (root) {
         m_childModel.setRootItem(root);
-        // Tell the rhythmbox track source that it should re-build its index.
-        m_pTrackCollection->getTrackSource("traktor")->buildIndex();
+        // Tell the traktor track source that it should re-build its index.
+        m_trackSource->buildIndex();
 
         //m_pTraktorTableModel->select();
         emit(showTrackModel(m_pTraktorTableModel));
