@@ -19,17 +19,21 @@ class TreeItemModel : public QAbstractItemModel {
     virtual QVariant headerData(int section, Qt::Orientation orientation, int role = Qt::DisplayRole) const;
     virtual QModelIndex index(int row, int column, const QModelIndex &parent = QModelIndex()) const;
     virtual QModelIndex parent(const QModelIndex &index) const;
+    // Tell the compiler we don't mean to shadow insertRows.
+    using QAbstractItemModel::insertRows;
     virtual bool insertRows(QList<TreeItem*> &data, int position, int rows, const QModelIndex &parent = QModelIndex());
     virtual bool removeRows(int position, int rows, const QModelIndex &parent = QModelIndex());
+    virtual bool setData(const QModelIndex &a_rIndex, const QVariant &a_rValue,
+                         int a_iRole = Qt::EditRole);
     virtual int rowCount(const QModelIndex &parent = QModelIndex()) const;
     virtual int columnCount(const QModelIndex &parent = QModelIndex()) const;
     void setRootItem(TreeItem *item);
-    /** Return the underlying TreeItem.
-     * If the index is invalid, the root item is returned. **/
+    // Return the underlying TreeItem.
+    // If the index is invalid, the root item is returned.
     TreeItem* getItem(const QModelIndex &index) const;
 
   private:
-    TreeItem *m_rootItem;
+    TreeItem *m_pRootItem;
 };
 
 #endif
