@@ -18,26 +18,23 @@
 #ifndef MIXXXKEYBOARD_H
 #define MIXXXKEYBOARD_H
 
-#include <qobject.h>
-#include <qevent.h>
-#include <qkeysequence.h>
-//Added by qt3to4:
+#include <QObject>
+#include <QEvent>
 #include <QKeyEvent>
-#include "configobject.h"
+#include <QMultiHash>
 
-/**
- * This class provides handling of keyboard events
- */
+#include "configobject.h"
 
 class ControlObject;
 
-class MixxxKeyboard : public QObject
-{
+// This class provides handling of keyboard events.
+class MixxxKeyboard : public QObject {
     Q_OBJECT
-public:
-    MixxxKeyboard(ConfigObject<ConfigValueKbd> *pKbdConfigObject, QObject *parent=0, const char *name=0);
-    ~MixxxKeyboard();
-    /** Event filter */
+  public:
+    MixxxKeyboard(ConfigObject<ConfigValueKbd> *pKbdConfigObject,
+                  QObject *parent=NULL, const char* name=NULL);
+    virtual ~MixxxKeyboard();
+
     bool eventFilter(QObject *obj, QEvent *e);
 
     // Set the keyboard config object. MixxxKeyboard does NOT take ownership of
@@ -45,13 +42,27 @@ public:
     void setKeyboardConfig(ConfigObject<ConfigValueKbd> *pKbdConfigObject);
     ConfigObject<ConfigValueKbd>* getKeyboardConfig();
 
-private:
-    /** Returns a valid QString with modifier keys from a QKeyEvent */
+  private:
+    struct KeyDownInformation {
+        KeyDownInformation(int keyId, int modifiers, ControlObject* pControl)
+                : keyId(keyId),
+                  modifiers(modifiers),
+                  pControl(pControl) {
+        }
+
+        int keyId;
+        int modifiers;
+        ControlObject* pControl;
+    };
+
+    // Returns a valid QString with modifier keys from a QKeyEvent
     QKeySequence getKeySeq(QKeyEvent *e);
-    /** List containing keys which is currently pressed */
-    QList<QPair<int, ConfigKey*> > m_qActiveKeyList;
-    /** Pointer to keyboard config object */
+    // List containing keys which is currently pressed
+    QList<KeyDownInformation> m_qActiveKeyList;
+    // Pointer to keyboard config object
     ConfigObject<ConfigValueKbd> *m_pKbdConfigObject;
+    // Multi-hash of key sequence to
+    QMultiHash<QKeySequence, ConfigKey> m_keySequenceToControlHash;
 };
 
 #endif
