@@ -3,10 +3,11 @@
 PreviewDeck::PreviewDeck(QObject* pParent,
                          ConfigObject<ConfigValue> *pConfig,
                          EngineMaster* pMixingEngine,
+                         EffectsManager* pEffectsManager,
                          EngineChannel::ChannelOrientation defaultOrientation,
                          QString group) :
-        BaseTrackPlayer(pParent, pConfig, pMixingEngine, defaultOrientation,
-                group, false, true) {
+        BaseTrackPlayerImpl(pParent, pConfig, pMixingEngine, pEffectsManager,
+                            defaultOrientation, group, false, true) {
 }
 
 PreviewDeck::~PreviewDeck() {
