@@ -3,12 +3,13 @@
 
 #include "basetrackplayer.h"
 
-class PreviewDeck : public BaseTrackPlayer {
+class PreviewDeck : public BaseTrackPlayerImpl {
     Q_OBJECT
   public:
     PreviewDeck(QObject* pParent,
                 ConfigObject<ConfigValue> *pConfig,
                 EngineMaster* pMixingEngine,
+                EffectsManager* pEffectsManager,
                 EngineChannel::ChannelOrientation defaultOrientation,
                 QString group);
     virtual ~PreviewDeck();
