@@ -1,6 +1,9 @@
 #ifndef DEFS_SHOUTCAST_H
 #define DEFS_SHOUTCAST_H
 
+#define SHOUTCAST_PREF_KEY "[Shoutcast]"
+#define SHOUTCAST_DEFAULT_PORT "8000"
+
 #define SHOUTCAST_CHANNELS_STEREO 2
 
 #define SHOUTCAST_SERVER_SHOUTCAST "Shoutcast"
