@@ -2,6 +2,8 @@
 #define SKINLOADER_H
 
 #include <QWidget>
+#include <QList>
+#include <QDir>
 
 #include "configobject.h"
 
@@ -9,26 +11,36 @@ class MixxxKeyboard;
 class PlayerManager;
 class ControllerManager;
 class Library;
-class MixxxView;
 class VinylControlManager;
+class EffectsManager;
+class LaunchImage;
 
 class SkinLoader {
   public:
     SkinLoader(ConfigObject<ConfigValue>* pConfig);
-    ~SkinLoader();
+    virtual ~SkinLoader();
+
     QWidget* loadDefaultSkin(QWidget* pParent,
                              MixxxKeyboard* pKeyboard,
                              PlayerManager* pPlayerManager,
                              ControllerManager* pControllerManager,
                              Library* pLibrary,
-                             VinylControlManager* pVCMan);
+                             VinylControlManager* pVCMan,
+                             EffectsManager* pEffectsManager);
 
-    QString getConfiguredSkinPath();
+    LaunchImage* loadLaunchImage(QWidget* pParent);
+
+    QString getSkinPath();
+    QList<QDir> getSkinSearchPaths();
 
   private:
+    QString getConfiguredSkinPath();
+    QString getDefaultSkinName() const;
+    QString getDefaultSkinPath();
+    QString pickResizableSkin(QString oldSkin);
+
     ConfigObject<ConfigValue>* m_pConfig;
 };
 
 
 #endif /* SKINLOADER_H */
-
