@@ -14,9 +14,12 @@
 *                                                                         *
 ***************************************************************************/
 
+#include "soundsourcemp3.h"
+#include "soundsourcetaglib.h"
+#include "util/math.h"
+
 #include <taglib/mpegfile.h>
 
-#include "soundsourcemp3.h"
 #include <QtDebug>
 
 
@@ -24,6 +27,7 @@ SoundSourceMp3::SoundSourceMp3(QString qFilename) :
         Mixxx::SoundSource(qFilename),
         m_file(qFilename)
 {
+    setType("mp3");
     inputbuf = NULL;
     Stream = new mad_stream;
     mad_stream_init(Stream);
@@ -36,6 +40,13 @@ SoundSourceMp3::SoundSourceMp3(QString qFilename) :
     m_iAvgFrameSize = 0;
     m_iChannels = 0;
     rest = 0;
+
+    bitrate = 0;
+    framecount = 0;
+    currentframe = 0;
+    pos = mad_timer_zero;
+    filelength = mad_timer_zero;
+    inputbuf_len = 0;
 }
 
 SoundSourceMp3::~SoundSourceMp3()
@@ -69,11 +80,10 @@ QList<QString> SoundSourceMp3::supportedFileExtensions()
     return list;
 }
 
-int SoundSourceMp3::open()
-{
-    m_file.setFileName(m_qFilename);
+Result SoundSourceMp3::open() {
+    m_file.setFileName(getFilename());
     if (!m_file.open(QIODevice::ReadOnly)) {
-        //qDebug() << "MAD: Open failed:" << m_qFilename;
+        //qDebug() << "MAD: Open failed:" << getFilename();
         return ERR;
     }
 
@@ -119,18 +129,19 @@ int SoundSourceMp3::open()
             continue;
         }
 
-        // Grab data from Header
+        // Grab data from madHeader
 
         // This warns us only when the reported sample rate changes. (and when
         // it is first set)
-        if (m_iSampleRate == 0 && Header.samplerate > 0) {
+        if (getSampleRate() == 0 && Header.samplerate > 0) {
             setSampleRate(Header.samplerate);
-        } else if (m_iSampleRate != Header.samplerate) {
+        } else if (getSampleRate() != Header.samplerate) {
             qDebug() << "SSMP3: file has differing samplerate in some headers:"
-                     << m_qFilename
-                     << m_iSampleRate << "vs" << Header.samplerate;
+                     << getFilename()
+                     << getSampleRate() << "vs" << Header.samplerate;
         }
 
+        setChannels(2); // always pretend to read 2 channels
         m_iChannels = MAD_NCHANNELS(&Header);
         mad_timer_add (&filelength, Header.duration);
         bitrate += Header.bitrate;
@@ -148,6 +159,7 @@ int SoundSourceMp3::open()
 
     // This is not a working MP3 file.
     if (currentframe == 0) {
+        qDebug() << "SSMP3: This is not a working MP3 file:" << getFilename();
         return ERR;
     }
 
@@ -204,15 +216,15 @@ long SoundSourceMp3::seek(long filepos) {
     }
 
     if (!isValid()) {
+        qDebug() << "SSMP3: Error wile seeking file " << getFilename();
         return 0;
     }
 
-//     qDebug() << "SEEK " << filepos;
+    //qDebug() << "SEEK " << filepos;
 
     MadSeekFrameType* cur = NULL;
 
-    if (filepos==0)
-    {
+    if (filepos == 0) {
         // Seek to beginning of file
 
         // Re-init buffer:
@@ -225,11 +237,9 @@ long SoundSourceMp3::seek(long filepos) {
         rest=-1;
 
         m_currentSeekFrameIndex = 0;
-        cur = getSeekFrame(0);
+        //cur = getSeekFrame(0);
         //frameIterator.toFront(); //Might not need to do this -- Albert June 19/2010 (during Qt3 purge)
-    }
-    else
-    {
+    } else {
         //qDebug() << "seek precise";
         // Perform precise seek accomplished by using a frame in the seek list
 
@@ -245,8 +255,7 @@ long SoundSourceMp3::seek(long filepos) {
          */
 
         int framePos = findFrame(filepos);
-        if (framePos==0 || framePos>filepos || m_currentSeekFrameIndex < 5)
-        {
+        if (framePos == 0 || framePos > filepos || m_currentSeekFrameIndex < 5) {
             //qDebug() << "Problem finding good seek frame (wanted " << filepos << ", got " << framePos << "), starting from 0";
 
             // Re-init buffer:
@@ -259,12 +268,10 @@ long SoundSourceMp3::seek(long filepos) {
             rest = -1;
             m_currentSeekFrameIndex = 0;
             cur = getSeekFrame(m_currentSeekFrameIndex);
-        }
-        else
-        {
-//             qDebug() << "frame pos " << cur->pos;
+        } else {
+            //qDebug() << "frame pos " << cur->pos;
 
-            // Start four frame before wanted frame to get in sync...
+            // Start four frames before wanted frame to get in sync...
             m_currentSeekFrameIndex -= 4;
             cur = getSeekFrame(m_currentSeekFrameIndex);
             if (cur != NULL) {
@@ -283,10 +290,10 @@ long SoundSourceMp3::seek(long filepos) {
                 mad_frame_mute(Frame);
 
                 // Decode the three frames before
-                mad_frame_decode(Frame,Stream);
-                mad_frame_decode(Frame,Stream);
-                mad_frame_decode(Frame,Stream);
-                mad_frame_decode(Frame,Stream);
+                mad_frame_decode(Frame, Stream);
+                mad_frame_decode(Frame, Stream);
+                mad_frame_decode(Frame, Stream);
+                mad_frame_decode(Frame, Stream);
 
                 // this is also explained in the above mad-dev post
                 mad_synth_frame(Synth, Frame);
@@ -298,7 +305,7 @@ long SoundSourceMp3::seek(long filepos) {
             }
         }
 
-        // Synthesize the the samples from the frame which should be discard to reach the requested position
+        // Synthesize the samples from the frame which should be discard to reach the requested position
         if (cur != NULL) //the "if" prevents crashes on bad files.
             discard(filepos-cur->pos);
     }
@@ -347,13 +354,12 @@ long SoundSourceMp3::seek(long filepos) {
     // Unfortunately we don't know the exact fileposition. The returned position is thus an
     // approximation only:
     return filepos;
-
 }
 
 inline long unsigned SoundSourceMp3::length() {
     enum mad_units units;
 
-    switch (m_iSampleRate)
+    switch (getSampleRate())
     {
     case 8000:
         units = MAD_UNITS_8000_HZ;
@@ -384,49 +390,44 @@ inline long unsigned SoundSourceMp3::length() {
         break;
     default:             //By the MP3 specs, an MP3 _has_ to have one of the above samplerates...
         units = MAD_UNITS_44100_HZ;
-        qWarning() << "MP3 with corrupt samplerate (" << m_iSampleRate << "), defaulting to 44100";
+        qWarning() << "MP3 with corrupt samplerate (" << getSampleRate() << "), defaulting to 44100";
 
-        m_iSampleRate = 44100; //Prevents division by zero errors.
+        setSampleRate(44100); //Prevents division by zero errors.
     }
 
-    return (long unsigned) 2 *mad_timer_count(filelength, units);
+    return 2 * mad_timer_count(filelength, units);
 }
 
 /*
   decode the chosen number of samples and discard
 */
 
-unsigned long SoundSourceMp3::discard(unsigned long samples_wanted)
-{
+unsigned long SoundSourceMp3::discard(unsigned long samples_wanted) {
     unsigned long Total_samples_decoded = 0;
-    int no;
+    int no = 0;
 
-    if(rest > 0)
+    if (rest > 0)
         Total_samples_decoded += 2*(Synth->pcm.length-rest);
 
-    while (Total_samples_decoded < samples_wanted)
-    {
-        if(mad_frame_decode(Frame,Stream))
-        {
-            if(MAD_RECOVERABLE(Stream->error))
-            {
+    while (Total_samples_decoded < samples_wanted) {
+        if (mad_frame_decode(Frame, Stream)) {
+            if (MAD_RECOVERABLE(Stream->error)) {
                 continue;
-            } else if(Stream->error==MAD_ERROR_BUFLEN) {
+            } else if(Stream->error == MAD_ERROR_BUFLEN) {
                 break;
             } else {
                 break;
             }
         }
-        mad_synth_frame(Synth,Frame);
-        no = math_min(Synth->pcm.length,(samples_wanted-Total_samples_decoded)/2);
+        mad_synth_frame(Synth, Frame);
+        no = math_min<int>(Synth->pcm.length,(samples_wanted-Total_samples_decoded)/2);
         Total_samples_decoded += 2*no;
     }
 
     if (Synth->pcm.length > no)
-	rest = no;
+        rest = no;
     else
-	rest = -1;
-
+        rest = -1;
 
     return Total_samples_decoded;
 }
@@ -437,8 +438,10 @@ unsigned long SoundSourceMp3::discard(unsigned long samples_wanted)
  */
 unsigned SoundSourceMp3::read(unsigned long samples_wanted, const SAMPLE * _destination)
 {
-    if (!isValid())
+    if (!isValid()) {
+        qDebug() << "SSMP3: Error while reading " << getFilename();
         return 0;
+    }
 
     // Ensure that we are reading an even number of samples. Otherwise this function may
     // go into an infinite loop
@@ -481,7 +484,6 @@ unsigned SoundSourceMp3::read(unsigned long samples_wanted, const SAMPLE * _dest
                 rest = -1;
             return Total_samples_decoded;
         }
-
     }
 
 //     qDebug() << "Decoding";
@@ -532,7 +534,7 @@ unsigned SoundSourceMp3::read(unsigned long samples_wanted, const SAMPLE * _dest
 
 
 //         qDebug() << "synthlen " << Synth->pcm.length << ", remain " << (samples_wanted-Total_samples_decoded);
-        no = math_min(Synth->pcm.length,(samples_wanted-Total_samples_decoded)/2);
+        no = math_min<int>(Synth->pcm.length,(samples_wanted-Total_samples_decoded)/2);
         for (i=0; i<no; i++)
         {
             // Left channel
@@ -560,47 +562,54 @@ unsigned SoundSourceMp3::read(unsigned long samples_wanted, const SAMPLE * _dest
     return Total_samples_decoded;
 }
 
-int SoundSourceMp3::parseHeader()
-{
-    setType("mp3");
-#ifdef __WINDOWS__
-    /* From Tobias: A Utf-8 string did not work on my Windows XP (German edition)
-     * If you try this conversion, f.isValid() will return false in many cases
-     * and processTaglibFile() will fail
-     *
-     * The method toLocal8Bit() returns the local 8-bit representation of the string as a QByteArray.
-     * The returned byte array is undefined if the string contains characters not supported
-     * by the local 8-bit encoding.
-     */
-    QByteArray qBAFilename = m_qFilename.toLocal8Bit();
-#else
-    QByteArray qBAFilename = m_qFilename.toUtf8();
-#endif
-    TagLib::MPEG::File f(qBAFilename.constData());
-
-    // Takes care of all the default metadata
-    bool result = processTaglibFile(f);
+Result SoundSourceMp3::parseHeader() {
+    TagLib::MPEG::File f(TAGLIB_FILENAME_FROM_QSTRING(getFilename()));
+    if (!readFileHeader(this, f)) {
+        return ERR;
+    }
 
     // Now look for MP3 specific metadata (e.g. BPM)
     TagLib::ID3v2::Tag* id3v2 = f.ID3v2Tag();
     if (id3v2) {
-        processID3v2Tag(id3v2);
+        readID3v2Tag(this, *id3v2);
+    } else {
+        TagLib::APE::Tag *ape = f.APETag();
+        if (ape) {
+            readAPETag(this, *ape);
+        } else {
+            // fallback
+            const TagLib::Tag *tag(f.tag());
+            if (tag) {
+                readTag(this, *tag);
+            } else {
+                return ERR;
+            }
+        }
     }
 
-    TagLib::APE::Tag *ape = f.APETag();
-    if (ape) {
-        processAPETag(ape);
-    }
+    return OK;
+}
 
-    if (result)
-        return OK;
-    return ERR;
+QImage SoundSourceMp3::parseCoverArt() {
+    QImage coverArt;
+    TagLib::MPEG::File f(TAGLIB_FILENAME_FROM_QSTRING(getFilename()));
+    TagLib::ID3v2::Tag* id3v2 = f.ID3v2Tag();
+    if (id3v2) {
+        coverArt = Mixxx::getCoverInID3v2Tag(*id3v2);
+    }
+    if (coverArt.isNull()) {
+        TagLib::APE::Tag *ape = f.APETag();
+        if (ape) {
+            coverArt = Mixxx::getCoverInAPETag(*ape);
+        }
+    }
+    return coverArt;
 }
 
 int SoundSourceMp3::findFrame(int pos)
 {
     // Guess position of frame in m_qSeekList based on average frame size
-    m_currentSeekFrameIndex = math_min(m_qSeekList.count()-1,
+    m_currentSeekFrameIndex = math_min((unsigned int) m_qSeekList.count()-1,
                                        m_iAvgFrameSize ? (unsigned int)(pos/m_iAvgFrameSize) : 0);
     MadSeekFrameType* temp = getSeekFrame(m_currentSeekFrameIndex);
 
@@ -661,4 +670,3 @@ inline signed int SoundSourceMp3::madScale(mad_fixed_t sample)
 
     return sample >> (MAD_F_FRACBITS + 1 - 16);
 }
-
