@@ -17,30 +17,38 @@
 #ifndef SOUNDSOURCEOGGVORBIS_H
 #define SOUNDSOURCEOGGVORBIS_H
 
-#include <qstring.h>
-#include "soundsource.h"
-#include <vorbis/codec.h>
+#include <QString>
+#define OV_EXCLUDE_STATIC_CALLBACKS
 #include <vorbis/vorbisfile.h>
 
-class TrackInfoObject;
+#include "soundsource.h"
 
+class QFile;
 
-class SoundSourceOggVorbis : public SoundSource {
+class SoundSourceOggVorbis : public Mixxx::SoundSource {
  public:
-  SoundSourceOggVorbis(QString qFilename);
+  explicit SoundSourceOggVorbis(QString qFilename);
   ~SoundSourceOggVorbis();
+  Result open();
   long seek(long);
   unsigned read(unsigned long size, const SAMPLE*);
   inline long unsigned length();
-  static int ParseHeader( TrackInfoObject * );
+  Result parseHeader();
+  QImage parseCoverArt();
+  static QList<QString> supportedFileExtensions();
  private:
+  static size_t ReadCallback(void *ptr,
+          size_t size, size_t nmemb, void *datasource);
+  static int SeekCallback(void *datasource, ogg_int64_t offset, int whence);
+  static int CloseCallback(void *datasource);
+  static long TellCallback(void *datasource);
+  static ov_callbacks s_callbacks;
+
   int channels;
   unsigned long filelength;
   OggVorbis_File vf;
   int current_section;
-  unsigned long ret, needed, index;
-  SAMPLE* dest;
-  char* pRead;
+  QFile* m_pFile;
 };
 
 #endif
