@@ -18,50 +18,61 @@
 #ifndef SOUNDSOURCEPROXY_H
 #define SOUNDSOURCEPROXY_H
 
+#include <QMap>
 #include <QMutex>
+#include <QString>
+#include <QLibrary>
+#include <QRegExp>
+
 #include "soundsource.h"
 #include "trackinfoobject.h"
-
-class QLibrary;
+#include "util/sandbox.h"
 
 /**
   *@author Tue Haste Andersen
   */
 
 
-/*
-  Base class for sound sources.
-*/
-class SoundSourceProxy : public SoundSource
+/**
+ * Creates sound sources for filenames or tracks.
+ */
+class SoundSourceProxy
 {
 public:
-    SoundSourceProxy(QString qFilename);
-    SoundSourceProxy(TrackPointer pTrack);
-    ~SoundSourceProxy();
     static void loadPlugins();
-    int open();
-    long seek(long);
-    unsigned read(unsigned long size, const SAMPLE*);
-    long unsigned length();
-    int parseHeader();
-    static int ParseHeader(TrackInfoObject* p);
-    unsigned int getSampleRate();
-    /** Returns filename */
-    QString getFilename();
-    static QList<QString> supportedFileExtensions();
+
+    static QStringList supportedFileExtensions();
+    static QStringList supportedFileExtensionsByPlugins();
     static QString supportedFileExtensionsString();
     static QString supportedFileExtensionsRegex();
+    static bool isFilenameSupported(QString filename);
+
+    SoundSourceProxy(QString qFilename, SecurityTokenPointer pToken);
+    explicit SoundSourceProxy(TrackPointer pTrack);
+
+    const Mixxx::SoundSourcePointer& getSoundSource() const {
+        return m_pSoundSource;
+    }
+
+    // Opens the audio data through the proxy will
+    // update the some metadata of the track object.
+    // Returns a null pointer on failure.
+    Mixxx::SoundSourcePointer open() const;
 
 private:
-    static SoundSource* initialize(QString qFilename);
-    //void initPlugin(QString lib_filename, QString track_filename);
-    static QLibrary* getPlugin(QString lib_filename);
-
-    SoundSource *m_pSoundSource;
-    TrackPointer m_pTrack;
+    static QRegExp m_supportedFileRegex;
     static QMap<QString, QLibrary*> m_plugins;
     static QMap<QString, getSoundSourceFunc> m_extensionsSupportedByPlugins;
     static QMutex m_extensionsMutex;
+
+    static QLibrary* getPlugin(QString lib_filename);
+
+    static Mixxx::SoundSourcePointer initialize(const QString& qFilename);
+
+    const TrackPointer m_pTrack;
+    const SecurityTokenPointer m_pSecurityToken;
+
+    const Mixxx::SoundSourcePointer m_pSoundSource;
 };
 
 #endif
