@@ -16,25 +16,40 @@
 ***************************************************************************/
 
 #include <QPixmap>
-#include <QtCore>
 #include <QMessageBox>
+#include <QPushButton>
 #include <QTranslator>
+#include <QScopedPointer>
+
 #include "defs_version.h"
 #include "controllers/defs_controllers.h"
 #include "track/beat_preferences.h"
+#include "library/trackcollection.h"
+#include "library/library_preferences.h"
+#include "util/math.h"
+#include "util/cmdlineargs.h"
 #include "configobject.h"
 #include "upgrade.h"
 
 
 Upgrade::Upgrade()
-{
-    m_bFirstRun = false;
-    m_bUpgraded = false;
+        : m_bFirstRun(false),
+          m_bUpgraded(false),
+          m_bRescanLibrary(false) {
 }
 
-Upgrade::~Upgrade()
-{
+Upgrade::~Upgrade() {
+}
 
+// static 
+QString Upgrade::mixxx17HomePath() {
+#ifdef __LINUX__
+    return QDir::homePath().append("/").append(".mixxx/");
+#elif __WINDOWS__
+    return QDir::homePath().append("/").append("Local Settings/Application Data/Mixxx/");
+#elif __APPLE__
+    return QDir::homePath().append("/").append(".mixxx/");
+#endif
 }
 
 // We return the ConfigObject here because we have to make changes to the
@@ -48,33 +63,32 @@ ConfigObject<ConfigValue>* Upgrade::versionUpgrade(const QString& settingsPath)
 *   since we moved them in 1.7.0. This code takes care of moving them.
 */
 
-    QString oldLocation = QDir::homePath().append("/%1");
+    QDir oldLocation = QDir(QDir::homePath());
 #ifdef __WINDOWS__
-    QFileInfo* pre170Config = new QFileInfo(oldLocation.arg("mixxx.cfg"));
+    QFileInfo* pre170Config = new QFileInfo(oldLocation.filePath("mixxx.cfg"));
 #else
-    QFileInfo* pre170Config = new QFileInfo(oldLocation.arg(".mixxx.cfg"));
+    QFileInfo* pre170Config = new QFileInfo(oldLocation.filePath(".mixxx.cfg"));
 #endif
 
     if (pre170Config->exists()) {
 
         // Move the files to their new location
-        QString newLocation = settingsPath;
+        QDir newLocation = QDir(settingsPath);
 
-        if (!QDir(newLocation).exists()) {
-            qDebug() << "Creating new settings directory" << newLocation;
-            QDir().mkpath(newLocation);
+        if (!newLocation.exists()) {
+            qDebug() << "Creating new settings directory" << newLocation.absolutePath();
+            newLocation.mkpath(".");
         }
 
-        newLocation.append("%1");
         QString errorText = "Error moving your %1 file %2 to the new location %3: \n";
 
 #ifdef __WINDOWS__
-        QString oldFilePath = oldLocation.arg("mixxxtrack.xml");
+        QString oldFilePath = oldLocation.filePath("mixxxtrack.xml");
 #else
-        QString oldFilePath = oldLocation.arg(".mixxxtrack.xml");
+        QString oldFilePath = oldLocation.filePath(".mixxxtrack.xml");
 #endif
 
-        QString newFilePath = newLocation.arg("mixxxtrack.xml");
+        QString newFilePath = newLocation.filePath("mixxxtrack.xml");
         QFile* oldFile = new QFile(oldFilePath);
         if (oldFile->exists()) {
             if (oldFile->copy(newFilePath)) {
@@ -89,11 +103,11 @@ ConfigObject<ConfigValue>* Upgrade::versionUpgrade(const QString& settingsPath)
         delete oldFile;
 
 #ifdef __WINDOWS__
-        oldFilePath = oldLocation.arg("mixxxbpmschemes.xml");
+        oldFilePath = oldLocation.filePath("mixxxbpmschemes.xml");
 #else
-        oldFilePath = oldLocation.arg(".mixxxbpmscheme.xml");
+        oldFilePath = oldLocation.filePath(".mixxxbpmscheme.xml");
 #endif
-        newFilePath = newLocation.arg("mixxxbpmscheme.xml");
+        newFilePath = newLocation.filePath("mixxxbpmscheme.xml");
         oldFile = new QFile(oldFilePath);
         if (oldFile->exists()) {
             if (oldFile->copy(newFilePath))
@@ -105,11 +119,11 @@ ConfigObject<ConfigValue>* Upgrade::versionUpgrade(const QString& settingsPath)
         }
         delete oldFile;
 #ifdef __WINDOWS__
-        oldFilePath = oldLocation.arg("MixxxMIDIBindings.xml");
+        oldFilePath = oldLocation.filePath("MixxxMIDIBindings.xml");
 #else
-        oldFilePath = oldLocation.arg(".MixxxMIDIBindings.xml");
+        oldFilePath = oldLocation.filePath(".MixxxMIDIBindings.xml");
 #endif
-        newFilePath = newLocation.arg("MixxxMIDIBindings.xml");
+        newFilePath = newLocation.filePath("MixxxMIDIBindings.xml");
         oldFile = new QFile(oldFilePath);
         if (oldFile->exists()) {
             qWarning() << "The MIDI mapping file format has changed in this version of Mixxx. You will need to reconfigure your MIDI controller. See the Wiki for full details on the new format.";
@@ -122,15 +136,18 @@ ConfigObject<ConfigValue>* Upgrade::versionUpgrade(const QString& settingsPath)
         }
         // Tidy up
         delete oldFile;
-
-        QFile::remove(oldLocation.arg(".MixxxMIDIDevice.xml")); // Obsolete file, so just delete it
+#ifdef __WINDOWS__
+        QFile::remove(oldLocation.filePath("MixxxMIDIDevice.xml")); // Obsolete file, so just delete it
+#else
+        QFile::remove(oldLocation.filePath(".MixxxMIDIDevice.xml")); // Obsolete file, so just delete it
+#endif
 
 #ifdef __WINDOWS__
-        oldFilePath = oldLocation.arg("mixxx.cfg");
+        oldFilePath = oldLocation.filePath("mixxx.cfg");
 #else
-        oldFilePath = oldLocation.arg(".mixxx.cfg");
+        oldFilePath = oldLocation.filePath(".mixxx.cfg");
 #endif
-        newFilePath = newLocation.arg(SETTINGS_FILE);
+        newFilePath = newLocation.filePath(SETTINGS_FILE);
         oldFile = new QFile(oldFilePath);
         if (oldFile->copy(newFilePath))
             oldFile->remove();
@@ -155,7 +172,7 @@ ConfigObject<ConfigValue>* Upgrade::versionUpgrade(const QString& settingsPath)
 ****************************************************************************/
 
     // Read the config file from home directory
-    ConfigObject<ConfigValue> *config = new ConfigObject<ConfigValue>(settingsPath + SETTINGS_FILE);
+    ConfigObject<ConfigValue> *config = new ConfigObject<ConfigValue>(QDir(settingsPath).filePath(SETTINGS_FILE));
 
     QString configVersion = config->getValueString(ConfigKey("[Config]","Version"));
 
@@ -163,24 +180,44 @@ ConfigObject<ConfigValue>* Upgrade::versionUpgrade(const QString& settingsPath)
 
 #ifdef __APPLE__
         qDebug() << "Config version is empty, trying to read pre-1.9.0 config";
-        //Try to read the config from the pre-1.9.0 final directory on OS X (we moved it in 1.9.0 final)
-        QFile* oldFile = new QFile(QDir::homePath().append("/").append(".mixxx/mixxx.cfg"));
-        if (oldFile->exists()) {
+        // Try to read the config from the pre-1.9.0 final directory on OS X (we moved it in 1.9.0 final)
+        QScopedPointer<QFile> oldConfigFile(new QFile(QDir::homePath().append("/").append(".mixxx/mixxx.cfg")));
+        if (oldConfigFile->exists() && ! CmdlineArgs::Instance().getSettingsPathSet()) {
             qDebug() << "Found pre-1.9.0 config for OS X";
-            config = new ConfigObject<ConfigValue>(QDir::homePath().append("/").append(".mixxx/mixxx.cfg"));
-            //Note: We changed SETTINGS_PATH in 1.9.0 final on OS X so it must be hardcoded to ".mixxx" here for legacy.
+            // Note: We changed SETTINGS_PATH in 1.9.0 final on OS X so it must be hardcoded to ".mixxx" here for legacy.
+            config = new ConfigObject<ConfigValue>(QDir::homePath().append("/.mixxx/mixxx.cfg"));
+            // Just to be sure all files like logs and soundconfig go with mixxx.cfg
+            // TODO(XXX) Trailing slash not needed anymore as we switches from String::append
+            // to QDir::filePath elsewhere in the code. This is candidate for removal.
+            CmdlineArgs::Instance().setSettingsPath(QDir::homePath().append("/.mixxx/"));
+            configVersion = config->getValueString(ConfigKey("[Config]","Version"));
+        }
+        else {
+#elif __WINDOWS__
+        qDebug() << "Config version is empty, trying to read pre-1.12.0 config";
+        // Try to read the config from the pre-1.12.0 final directory on Windows (we moved it in 1.12.0 final)
+        QScopedPointer<QFile> oldConfigFile(new QFile(QDir::homePath().append("/Local Settings/Application Data/Mixxx/mixxx.cfg")));
+        if (oldConfigFile->exists() && ! CmdlineArgs::Instance().getSettingsPathSet()) {
+            qDebug() << "Found pre-1.12.0 config for Windows";
+            // Note: We changed SETTINGS_PATH in 1.12.0 final on Windows so it must be hardcoded to "Local Settings/Application Data/Mixxx/" here for legacy.
+            config = new ConfigObject<ConfigValue>(QDir::homePath().append("/Local Settings/Application Data/Mixxx/mixxx.cfg"));
+            // Just to be sure all files like logs and soundconfig go with mixxx.cfg
+            // TODO(XXX) Trailing slash not needed anymore as we switches from String::append
+            // to QDir::filePath elsewhere in the code. This is candidate for removal.
+            CmdlineArgs::Instance().setSettingsPath(QDir::homePath().append("/Local Settings/Application Data/Mixxx/")); 
             configVersion = config->getValueString(ConfigKey("[Config]","Version"));
-            delete oldFile;
         }
         else {
 #endif
-            //This must have been the first run... right? :)
+            // This must have been the first run... right? :)
             qDebug() << "No version number in configuration file. Setting to" << VERSION;
             config->set(ConfigKey("[Config]","Version"), ConfigValue(VERSION));
             m_bFirstRun = true;
             return config;
 #ifdef __APPLE__
         }
+#elif __WINDOWS__
+        }
 #endif
     }
 
@@ -208,10 +245,10 @@ ConfigObject<ConfigValue>* Upgrade::versionUpgrade(const QString& settingsPath)
     }
     */
 
-    //We use the following blocks to detect if this is the first time
-    //you've run the latest version of Mixxx. This lets us show
-    //the promo tracks stats agreement stuff for all users that are
-    //upgrading Mixxx.
+    // We use the following blocks to detect if this is the first time
+    // you've run the latest version of Mixxx. This lets us show
+    // the promo tracks stats agreement stuff for all users that are
+    // upgrading Mixxx.
 
     if (configVersion.startsWith("1.7")) {
         qDebug() << "Upgrading from v1.7.x...";
@@ -250,12 +287,12 @@ ConfigObject<ConfigValue>* Upgrade::versionUpgrade(const QString& settingsPath)
             qDebug() << "Moving" << curPair.first << "to" << curPair.second;
             QDir oldSubDir(curPair.first);
             QDir newSubDir(curPair.second);
-            newSubDir.mkpath(curPair.second); //Create the new destination directory
+            newSubDir.mkpath(curPair.second); // Create the new destination directory
 
             QStringList contents = oldSubDir.entryList(QDir::Files | QDir::NoDotAndDotDot);
             QStringListIterator it(contents);
             QString cur;
-            //Iterate over all the files in the source directory and copy them to the dest dir.
+            // Iterate over all the files in the source directory and copy them to the dest dir.
             while (it.hasNext())
             {
                 cur = it.next();
@@ -268,12 +305,12 @@ ConfigObject<ConfigValue>* Upgrade::versionUpgrade(const QString& settingsPath)
                 }
             }
 
-            //Rename the old directory.
+            // Rename the old directory.
             newOSXDir.rename(OSXLocation180, OSXLocation180 + "-1.8");
         }
-        //Reload the configuration file from the new location.
-        //(We want to make sure we save to the new location...)
-        config = new ConfigObject<ConfigValue>(settingsPath + SETTINGS_FILE);
+        // Reload the configuration file from the new location.
+        // (We want to make sure we save to the new location...)
+        config = new ConfigObject<ConfigValue>(QDir(settingsPath).filePath(SETTINGS_FILE));
 #endif
         configVersion = "1.9.0";
         config->set(ConfigKey("[Config]","Version"), ConfigValue("1.9.0"));
@@ -293,7 +330,7 @@ ConfigObject<ConfigValue>* Upgrade::versionUpgrade(const QString& settingsPath)
         QStringList contents = oldDir.entryList(QDir::Files | QDir::NoDotAndDotDot);
         QStringListIterator it(contents);
         QString cur;
-        //Iterate over all the files in the source directory and copy them to the dest dir.
+        // Iterate over all the files in the source directory and copy them to the dest dir.
         while (it.hasNext()) {
             cur = it.next();
             if (newDir.exists(cur)) {
@@ -325,13 +362,42 @@ ConfigObject<ConfigValue>* Upgrade::versionUpgrade(const QString& settingsPath)
             qDebug() << "Upgrade Failed";
         }
     }
-    // Next applicable release goes here
-    /*
+
     if (configVersion.startsWith("1.11")) {
         qDebug() << "Upgrading from v1.11.x...";
 
-        // Upgrade tasks go here
+        // upgrade to the multi library folder settings
+        QString currentFolder = config->getValueString(PREF_LEGACY_LIBRARY_DIR);
+        // to migrate the DB just add the current directory to the new
+        // directories table
+        TrackCollection tc(config);
+        DirectoryDAO directoryDAO = tc.getDirectoryDAO();
 
+        // NOTE(rryan): We don't have to ask for sandbox permission to this
+        // directory because the normal startup integrity check in Library will
+        // notice if we don't have permission and ask for access. Also, the
+        // Sandbox isn't setup yet at this point in startup because it relies on
+        // the config settings path and this function is what loads the config
+        // so it's not ready yet.
+        bool successful = directoryDAO.addDirectory(currentFolder);
+
+        // ask for library rescan to activate cover art. We can later ask for
+        // this variable when the library scanner is constructed.
+        m_bRescanLibrary = askReScanLibrary();
+
+        // Versions of mixxx until 1.11 had a hack that multiplied gain by 1/2,
+        // which was compensation for another hack that set replaygain to a
+        // default of 6.  We've now removed all of the hacks, so subtracting
+        // 6 from everyone's replay gain should keep things consistent for
+        // all users.
+        int oldReplayGain = config->getValueString(
+                ConfigKey("[ReplayGain]", "InitialReplayGainBoost"), "6").toInt();
+        int newReplayGain = math_max(-6, oldReplayGain - 6);
+        config->set(ConfigKey("[ReplayGain]", "InitialReplayGainBoost"),
+                    ConfigValue(newReplayGain));
+
+        // if everything until here worked fine we can mark the configuration as
+        // updated
         if (successful) {
             configVersion = VERSION;
             m_bUpgraded = true;
@@ -341,7 +407,6 @@ ConfigObject<ConfigValue>* Upgrade::versionUpgrade(const QString& settingsPath)
             qDebug() << "Upgrade failed!\n";
         }
     }
-    */
 
     if (configVersion == VERSION) qDebug() << "Configuration file is now at the current version" << VERSION;
     else {
@@ -358,6 +423,21 @@ ConfigObject<ConfigValue>* Upgrade::versionUpgrade(const QString& settingsPath)
     return config;
 }
 
+bool Upgrade::askReScanLibrary() {
+    QMessageBox msgBox;
+    msgBox.setIconPixmap(QPixmap(":/images/mixxx-icon.png"));
+    msgBox.setWindowTitle(QMessageBox::tr("Upgrading Mixxx"));
+    msgBox.setText(QMessageBox::tr("Mixxx now supports displaying cover art.\n"
+                      "Do you want to scan your library for cover files now?"));
+    QPushButton* rescanButton = msgBox.addButton(
+        QMessageBox::tr("Scan"), QMessageBox::AcceptRole);
+    msgBox.addButton(QMessageBox::tr("Later"), QMessageBox::RejectRole);
+    msgBox.setDefaultButton(rescanButton);
+    msgBox.exec();
+
+    return msgBox.clickedButton() == rescanButton;
+}
+
 bool Upgrade::askReanalyzeBeats() {
     QString windowTitle =
             QMessageBox::tr("Upgrading Mixxx from v1.9.x/1.10.x.");
