@@ -24,12 +24,16 @@ class Upgrade
         Upgrade();
         ~Upgrade();
         ConfigObject<ConfigValue>* versionUpgrade(const QString& settingsPath);
+        static QString mixxx17HomePath();
         bool isFirstRun() { return m_bFirstRun; };
         bool isUpgraded() { return m_bUpgraded; };
+        bool rescanLibrary() {return m_bRescanLibrary; };
     private:
         bool askReanalyzeBeats();
+        bool askReScanLibrary();
         bool m_bFirstRun;
         bool m_bUpgraded;
+        bool m_bRescanLibrary;
 };
 
 #endif
