@@ -6,13 +6,13 @@
 #include <QBrush>
 #include <QPen>
 
-#include <vector>
+#include <QVector>
 
 class ControlObject;
 
 class QtWaveformRendererSimpleSignal : public WaveformRendererSignalBase {
 public:
-    explicit QtWaveformRendererSimpleSignal( WaveformWidgetRenderer* waveformWidgetRenderer);
+    explicit QtWaveformRendererSimpleSignal(WaveformWidgetRenderer* waveformWidgetRenderer);
     virtual ~QtWaveformRendererSimpleSignal();
 
     virtual void onSetup(const QDomNode &node);
@@ -24,7 +24,7 @@ protected:
 private:
     QBrush m_brush;
     QPen m_borderPen;
-    std::vector<QPointF> m_polygon;
+    QVector<QPointF> m_polygon;
 };
 
 #endif // QTWAVEFORMRENDERERSIMPLESIGNAL_H
