@@ -8,6 +8,7 @@
 
 #include "util.h"
 #include "waveformrendererabstract.h"
+#include "skin/skincontext.h"
 
 class WaveformWidgetRenderer;
 
@@ -16,7 +17,7 @@ class WaveformRenderBackground : public WaveformRendererAbstract {
     explicit WaveformRenderBackground(WaveformWidgetRenderer* waveformWidgetRenderer);
     virtual ~WaveformRenderBackground();
 
-    virtual void setup(const QDomNode& node);
+    virtual void setup(const QDomNode& node, const SkinContext& context);
     virtual void draw(QPainter* painter, QPaintEvent* event);
 
   private:
