@@ -3,41 +3,43 @@
 
 #include <QColor>
 #include <QTime>
+//#include <QLinearGradient>
 
 #include "util.h"
 #include "waveformrendererabstract.h"
+#include "skin/skincontext.h"
+#include "waveform/waveformwidgetfactory.h"
 
 class ControlObject;
-class ControlObjectThreadMain;
+class ControlObjectSlave;
 
 class WaveformRendererEndOfTrack : public WaveformRendererAbstract {
-public:
+  public:
     static const int s_maxAlpha = 125;
     explicit WaveformRendererEndOfTrack(
             WaveformWidgetRenderer* waveformWidgetRenderer);
     virtual ~WaveformRendererEndOfTrack();
 
     virtual bool init();
-    virtual void setup(const QDomNode& node);
+    virtual void setup(const QDomNode& node, const SkinContext& context);
     virtual void onResize();
     virtual void draw(QPainter* painter, QPaintEvent* event);
 
-private:
-    ControlObjectThreadMain*  m_endOfTrackControl;
+  private:
+    ControlObjectSlave* m_pEndOfTrackControl;
     bool m_endOfTrackEnabled;
-
-    ControlObjectThreadMain* m_trackSampleRate;
-    ControlObjectThreadMain* m_playControl;
-    ControlObjectThreadMain* m_loopControl;
+    ControlObjectSlave* m_pTrackSampleRate;
+    ControlObjectSlave* m_pPlayControl;
+    ControlObjectSlave* m_pLoopControl;
 
     QColor m_color;
     QTime m_timer;
+    int m_remainingTimeTriggerSeconds;
     int m_blinkingPeriodMillis;
-    double m_remainingTimeTriggerSeconds;
 
-    QRect m_rect;
     QVector<QRect> m_backRects;
     QPen m_pen;
+    //QLinearGradient m_gradient;
 
     DISALLOW_COPY_AND_ASSIGN(WaveformRendererEndOfTrack);
 };
