@@ -4,17 +4,16 @@
 #include "waveform/renderers/waveformrendererabstract.h"
 #include "waveformmarkset.h"
 #include "util.h"
-
-class ControlObjectThreadMain;
+#include "skin/skincontext.h"
 
 class WaveformRenderMark : public WaveformRendererAbstract {
   public:
     explicit WaveformRenderMark(WaveformWidgetRenderer* waveformWidgetRenderer);
 
-    virtual void setup(const QDomNode& node);
+    virtual void setup(const QDomNode& node, const SkinContext& context);
     virtual void draw(QPainter* painter, QPaintEvent* event);
 
-private:
+  private:
     void generateMarkImage(WaveformMark& mark);
 
     WaveformMarkSet m_marks;
