@@ -25,6 +25,12 @@ void EmptyWaveformWidget::castToQWidget() {
 }
 
 void EmptyWaveformWidget::paintEvent(QPaintEvent* event) {
+    // Only render if Qt thinks it is required
     QPainter painter(this);
     draw(&painter,event);
 }
+
+int EmptyWaveformWidget::render() {
+    // skip update every frame
+    return 0;
+}
