@@ -18,13 +18,14 @@ class EmptyWaveformWidget : public QWidget, public WaveformWidgetAbstract {
     static inline QString getWaveformWidgetName() { return tr("Empty"); }
     static inline bool useOpenGl() { return false; }
     static inline bool useOpenGLShaders() { return false; }
+    static inline bool developerOnly() { return false; }
 
   protected:
     virtual void castToQWidget();
     virtual void paintEvent(QPaintEvent* event);
+    virtual int render();
 
   private:
-    EmptyWaveformWidget() {}
     EmptyWaveformWidget(const char* group, QWidget* parent);
     friend class WaveformWidgetFactory;
 };
