@@ -11,19 +11,19 @@ class GLSimpleWaveformWidget : public QGLWidget, public WaveformWidgetAbstract {
     GLSimpleWaveformWidget(const char* group, QWidget* parent);
     virtual ~GLSimpleWaveformWidget();
 
-    virtual WaveformWidgetType::Type getType() const { return WaveformWidgetType::GLSimpleWaveform;}
+    virtual WaveformWidgetType::Type getType() const { return WaveformWidgetType::GLSimpleWaveform; }
 
-    static inline QString getWaveformWidgetName() { return tr("Simple");}
-    static inline bool useOpenGl() { return true;}
-    static inline bool useOpenGLShaders() { return false;}
+    static inline QString getWaveformWidgetName() { return tr("Simple"); }
+    static inline bool useOpenGl() { return true; }
+    static inline bool useOpenGLShaders() { return false; }
+    static inline bool developerOnly() { return false; }
 
   protected:
     virtual void castToQWidget();
     virtual void paintEvent(QPaintEvent* event);
-    virtual void postRender();
+    virtual int render();
 
   private:
-    GLSimpleWaveformWidget() {}
     friend class WaveformWidgetFactory;
 };
 #endif // GLSIMPLEWAVEFORMWIDGET_H
