@@ -11,19 +11,19 @@ class GLWaveformWidget : public QGLWidget, public WaveformWidgetAbstract {
     GLWaveformWidget(const char* group, QWidget* parent);
     virtual ~GLWaveformWidget();
 
-    virtual WaveformWidgetType::Type getType() const { return WaveformWidgetType::GLWaveform; }
+    virtual WaveformWidgetType::Type getType() const { return WaveformWidgetType::GLFilteredWaveform; }
 
-    static inline QString getWaveformWidgetName() { return tr("Filtered");}
-    static inline bool useOpenGl() { return true;}
-    static inline bool useOpenGLShaders() { return false;}
+    static inline QString getWaveformWidgetName() { return tr("Filtered"); }
+    static inline bool useOpenGl() { return true; }
+    static inline bool useOpenGLShaders() { return false; }
+    static inline bool developerOnly() { return false; }
 
   protected:
     virtual void castToQWidget();
     virtual void paintEvent(QPaintEvent* event);
-    virtual void postRender();
+    virtual int render();
 
   private:
-    GLWaveformWidget() {}
     friend class WaveformWidgetFactory;
 };
 
