@@ -18,31 +18,34 @@
 #ifndef WLABEL_H
 #define WLABEL_H
 
-#include "wwidget.h"
-#include <qlabel.h>
-#include <qevent.h>
+#include <QLabel>
+#include <QEvent>
 
-/**
-  *@author Tue Haste Andersen
-  */
+#include "widget/wbasewidget.h"
+#include "skin/skincontext.h"
 
-class WLabel : public WWidget {
+class WLabel : public QLabel, public WBaseWidget {
     Q_OBJECT
   public:
-    WLabel(QWidget *parent=0);
+    WLabel(QWidget* pParent=NULL);
     virtual ~WLabel();
-    void setup(QDomNode node);
 
-    void setAlignment(Qt::Alignment);
-    void setConstFactor(double);
-    virtual QWidget* getComposedWidget() { return m_pLabel; }
+    virtual void setup(QDomNode node, const SkinContext& context);
+
+    QString text() const;
+    void setText(const QString& text);
 
   protected:
-    /** Multiplication factor */
-    QLabel *m_pLabel;
-    QString m_qsText;
-    /** Foreground and background colors */
-    QColor m_qFgColor, m_qBgColor;
+    virtual bool event(QEvent* pEvent);
+    virtual void resizeEvent(QResizeEvent* event);
+    void fillDebugTooltip(QStringList* debug);
+    QString m_skinText;
+    // Foreground and background colors.
+    QColor m_qFgColor;
+    QColor m_qBgColor;
+  private: 
+    QString m_longText;
+    Qt::TextElideMode m_elideMode;
 };
 
 #endif
