@@ -6,10 +6,12 @@
 
 #include <QString>
 #include <QTableView>
+#include <QFont>
 
 #include "configobject.h"
 #include "library/libraryview.h"
 #include "trackinfoobject.h"
+#include "library/coverartcache.h"
 
 
 class WLibraryTableView : public QTableView, public virtual LibraryView {
@@ -24,11 +26,17 @@ class WLibraryTableView : public QTableView, public virtual LibraryView {
 
   signals:
     void loadTrack(TrackPointer pTrack);
-    void loadTrackToPlayer(TrackPointer pTrack, QString group, bool play = false);
+    void loadTrackToPlayer(TrackPointer pTrack, QString group,
+            bool play = false);
+    void trackSelected(TrackPointer pTrack);
+    void onlyCachedCoverArt(bool);
+    void scrollValueChanged(int);
 
   public slots:
     void saveVScrollBarPos();
     void restoreVScrollBarPos();
+    void setTrackTableFont(const QFont& font);
+    void setTrackTableRowHeight(int rowHeight);
 
   private:
     void loadVScrollBarPosState();
