@@ -18,36 +18,27 @@
 #ifndef WNUMBER_H
 #define WNUMBER_H
 
-#include "wwidget.h"
-#include <qlabel.h>
-#include <qevent.h>
+#include <QLabel>
 
-/**
-  *@author Tue & Ken Haste Andersen
-  */
+#include "widget/wlabel.h"
+#include "skin/skincontext.h"
 
-class WNumber : public WWidget  {
+class WNumber : public WLabel  {
     Q_OBJECT
   public:
-    WNumber(QWidget *parent=0);
+    WNumber(QWidget* pParent = NULL);
     virtual ~WNumber();
 
-    void setup(QDomNode node);
-    void setNumDigits(int);
-    void setConstFactor(double);
-    virtual QWidget* getComposedWidget() { return m_pLabel; }
+    virtual void setup(QDomNode node, const SkinContext& context);
+
+    virtual void onConnectedControlChanged(double dParameter, double dValue);
 
   public slots:
-    void setValue(double dValue);
+    virtual void setValue(double dValue);
 
   protected:
-    QLabel* m_pLabel;
-    QString m_qsText;
+    // Number of digits to round to.
     int m_iNoDigits;
-    /** Foreground and background colors */
-    QColor m_qFgColor, m_qBgColor;
-    /** Constant factor added to value */
-    double m_dConstFactor;
 };
 
 #endif
