@@ -1,7 +1,7 @@
 //
 // C++ Interface: wnumberpos
 //
-// Description: 
+// Description:
 //
 //
 // Author: Tue Haste Andersen <haste@diku.dk>, (C) 2003
@@ -12,24 +12,24 @@
 #ifndef WNUMBERRATE_H
 #define WNUMBERRATE_H
 
-#include "wnumber.h"
-class ControlObjectThreadMain;
+#include "widget/wnumber.h"
 
-/**
-@author Tue Haste Andersen
-*/
+class ControlObjectSlave;
 
-class WNumberRate : public WNumber
-{
+class WNumberRate : public WNumber {
     Q_OBJECT
-public:
+  public:
     WNumberRate(const char *group, QWidget *parent=0);
-    ~WNumberRate();
+    virtual ~WNumberRate();
+
+  private slots:
     void setValue(double dValue);
 
-private:
-    /** Pointer to control object for rate */
-    ControlObjectThreadMain *m_pRateControl, *m_pRateRangeControl, *m_pRateDirControl;
+  private:
+    // Pointer to control objects for rate.
+    ControlObjectSlave* m_pRateControl;
+    ControlObjectSlave* m_pRateRangeControl;
+    ControlObjectSlave* m_pRateDirControl;
 };
 
 #endif
