@@ -7,21 +7,25 @@
 #include <QTimer>
 #include <QDomNode>
 #include <QColor>
+#include <QEvent>
 
 #include "configobject.h"
+#include "skin/skincontext.h"
+#include "widget/wbasewidget.h"
 
-class WSearchLineEdit : public QLineEdit {
+class WSearchLineEdit : public QLineEdit, public WBaseWidget {
     Q_OBJECT
   public:
-    WSearchLineEdit(ConfigObject<ConfigValue>* pConfig,
-                    QWidget* pParent);
+    WSearchLineEdit(QWidget* pParent);
     virtual ~WSearchLineEdit();
-    void setup(QDomNode node);
+
+    void setup(QDomNode node, const SkinContext& context);
 
   protected:
     void resizeEvent(QResizeEvent*);
     virtual void focusInEvent(QFocusEvent*);
     virtual void focusOutEvent(QFocusEvent*);
+    bool event(QEvent* pEvent);
 
   signals:
     void search(const QString& text);
@@ -30,11 +34,13 @@ class WSearchLineEdit : public QLineEdit {
 
   public slots:
     void restoreSearch(const QString& text);
+    void slotTextChanged(const QString& text);
 
   private slots:
     void updateCloseButton(const QString& text);
     void slotSetupTimer(const QString& text);
     void triggerSearch();
+    void onSearchTextCleared();
 
   private:
     void showPlaceholder();
@@ -42,7 +48,7 @@ class WSearchLineEdit : public QLineEdit {
     QTimer m_searchTimer;
     QToolButton* m_clearButton;
     bool m_place;
-    QColor m_fgc; //Foreground colour
+    QColor m_fgc; //Foreground color
 };
 
 #endif
